# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module UserPointTag
      extend MethodDescription

      # TODO 現在１注文ごとにポイント付与・利用をしていますが、
      # １商品ごとにポイント利用、付与となるとこの処理を１商品ごとに表示するように変更する必要があります。

      desc <<-EOS
        [概要:ログインユーザーの保有ポイントを表示します。]
        [引数:なし]
        [戻り値:ログインユーザーの保有ポイント]
        [使用例:<ec:full_name/>]
      EOS
      def user_point_total_point
        UserPoint.total_point_for_user(@template.current_user.id) if @template.current_user
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント有効期限を表示します。]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [戻り値:ログインユーザーのポイント有効期限]
        [使用例:
           {{{
           <ec:user_point_term_date format="%y/%m/%d %H:%M"/>
           ⇒ 2009/03/01 15:30
           }}}
        ]
        [使用例:
           {{{
           <ec:user_point_term_date format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def user_point_term_date(*args)
        format = extract(args, :format => '%y/%m/%d')
        term_date = UserPoint.term_date_for_user(@template.current_user.id) if @template.current_user
        unless term_date.nil?
          term_date.strftime("#{format}")
        end
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント有効期限までの有効ポイントを表示します。]
        [引数:なし]
        [戻り値:ログインユーザーのポイント有効期限までの有効ポイント]
        [使用例:<ec:full_name/>]
      EOS
      def user_point_term_point
        if @template.current_user
          term_date = UserPoint.term_date_for_user(@template.current_user.id)
          point = UserPoint.point_to_term_date(@template.current_user.id, term_date)
          return point
        end
      end

      desc <<-EOS
        [概要:ログインユーザーのポイント履歴分繰り返しをします。]
        [引数:なし]
        [使用例:
           {{{
           <ec:user_point_term_point>
             ポイント情報詳細
           </ec:user_point_term_point>
           }}}
        ]
      EOS
      def user_point_point_list(*args, &block)
        if @template.current_user
          user_id = @template.current_user.id
          @points = UserPoint.find(:all,:select => "*, sum(given_point)as order_given_point, sum(used_point) as order_used_point",:group => "order_id", :conditions => ["user_id =? ", user_id], :order => "created_at desc")
          #結果が0行の場合空の行を表示する
          if @points.size == 0
            @point = 0
            @template.concat(@template.capture(&block))
          end
          _each_with_index(:@points, :@point) do
            @template.concat(@template.capture(&block))
          end
          ""
        end
        ""
      end

      desc <<-EOS
        [概要：ポイントの注文Noを表示します。user_point_list内で使用できます。 ]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
      EOS
      def user_point_order_code(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format = extract(args, :empty_format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @point.order.ec_order_code
        end
      end

      desc <<-EOS
        [概要:注文日を出力します。]
        [引数 format(任意):日付のフォーマットを指定します。年は"%y、月は"%m"、日は"%d"、時は%H、分は%Mで表します。指定しない場合は'%y/%m/%d'になります。]
        [使用例:
           {{{
           <ec:point_order_datetime format="%y/%m/%d %H:%M"/>
           ⇒ 2009/03/01 15:30
           }}}
        ]
        [使用例:
           {{{
           <ec:point_order_datetime format="%y年%m月%d日"/>
           ⇒ 2009年03月01日
           }}}
        ]
      EOS
      def user_point_order_datetime(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format, format = extract(args, :empty_format => "", :format => '%y/%m/%d')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @point.order.ordered_at.strftime("#{format}")
        end
      end

      desc <<-EOS
        [概要：ポイントの注文金額を表示します。user_point_list内で使用できます。 ]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [使用例:
           {{{
           <ec:user_point_order_price empty_format="-"/>
           ⇒ -
           }}}
        ]
        [使用例:
           {{{
           <ec:user_point_order_price format="円"/>
           ⇒ 1,000円
           }}}
        ]
      EOS
      def user_point_order_price(*args)
        raise UsageError, "point_order_codeはec:user_point_listの中で使用してください" unless @point
        empty_format, format = extract(args, :empty_format => '', :format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @template.fix_num_format(@point.order.total_info[:total]) + format
        end
      end

      desc <<-EOS
        [概要:注文商品の数だけ繰り返します。ec:user_point_list内で使用してください。]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [使用例:
           {{{
           <ec:user_point_list>
             <ec:user_point_order_product_list>
                AAA
             </ec:user_point_order_product_list>
           </ec:user_point_list>
           }}}
        ]
      EOS
      def user_point_order_product_list(*args,&block)
        raise UsageError, "ec:user_point_order_product_listはec:user_point_listの中で使用してください" unless @point
        empty_format = extract(args, :empty_format => '')
        if @point == 0
          @order_item = 0
          @template.concat(@template.capture(&block))
        elsif @point.order.nil?
          empty_format
        else
          order_shippings = @point.order.order_shippings
          shipping_order_items = []
          order_shippings.each do |order_shipping|
            shipping_order_items << order_shipping.order_items
          end
          @order_items = shipping_order_items.flatten
          _each_with_index(:@order_items, :@order_item, :@point_product_index) do
            @template.concat(@template.capture(&block))
          end
          ""
        end
      end

      desc <<-EOS
        [概要:注文商品の名前を表示します。ec:user_point_order_product_list内で使用してください。]
        [引数 empty_format(任意):注文がない場合のフォーマットを指定します。指定しない場合は空白になります。]
        [戻り値:注文商品の名前。]
        [使用例:
           {{{
           <ec:order_shipping_product_name empty_format="-"/>
           }}}
        ]
      EOS
      def user_point_order_product_name(*args)
        raise UsageError, "ec:point_order_product_nameはec:user_point_order_product_listの中で使用してください" unless @order_item
        empty_format = extract(args, :empty_format => '')
        if @point == 0 || @point.order.nil?
          empty_format
        else
          @order_item.product_name
        end
      end

      desc <<-EOS
        [概要:注文時の加算ポイントを表示します。ec:user_point_list内で使用してください。]
        [引数:なし]
        [戻り値:注文時の加算ポイント。]
        [使用例:
           {{{
           <ec:user_point_given_point/>ポイント
           }}}
        ]
      EOS
      def user_point_given_point
        raise UsageError, "ec:user_point_given_pointはec:user_point_listの中で使用してください" unless @point
        @point.order_given_point unless @point == 0
      end

      desc <<-EOS
        [概要:注文時の利用ポイントを表示します。ec:user_point_list内で使用してください。]
        [引数:なし]
        [戻り値:注文時の利用ポイント。]
        [使用例:
           {{{
           <ec:user_point_used_point/>ポイント
           }}}
        ]
      EOS
      def user_point_used_point
        raise UsageError, "ec:user_point_given_pointはec:user_point_listの中で使用してください" unless @point
        @point.order_used_point unless @point == 0
      end

    end
  end
end
