# -*- coding: utf-8 -*-
#
# クレジットフォーム
#
class CreditForm < ActiveForm
  include SelectableAttr::Base
  include ActiveRecord::Validations

  attr_accessor :kind, :card_no, :month, :year, :name

  # 名義
  validates_presence_of :name
  validates_format_of   :name, :with => /^([a-zA-Z ])+$/, :message => :EM990007

  # カード番号桁数とフォーマットは、カードの種類によって異なるので、バリデーションも別々に宣言します。
  # AMEX
  with_options(:if => :amex?) do |m|
    m.validates_length_of :card_no, :maximum => 15, :message => :EM990011
    m.validates_format_of :card_no, :with => /\d{15}/, :message => :EM990017
  end

  # Diners
  with_options(:if => :diners?) do |m|
    m.validates_length_of :card_no, :maximum => 14, :message => :EM990011
    m.validates_format_of :card_no, :with => /\d{14}/, :message => :EM990017
  end

  # AMEXとDiners以外
  with_options(:if => :etc?) do |m|
    m.validates_length_of :card_no, :maximum => 16, :message => :EM990011
    m.validates_format_of :card_no, :with => /\d{16}/, :message => :EM990017
  end

  selectable_attr :kind, :enum => OrderForm.card_kind_enum

  # 「AMEX」「Dinners」以外の場合にtrueを返す
  def etc?
    (kind_key != :amex) && (kind_key != :diners)
  end

  #「AMEX」の場合にtrueを返す
  def amex?
    kind_key == :amex
  end

  #「Dinners」の場合にtrueを返す
  def diners?
    kind_key == :diners
  end

  # 有効期限を「yymm」に整形します
  def expire
    '%2s%02d' % [year.to_s[-2..-1], month.to_i]
  end
end
