class ProductGroupSku < ActiveRecord::Base
  versioned
  belongs_to :product_sku
  belongs_to :product_group
  
  # フロントサイトに表示するための関連商品の一覧を取得する
  #   all : "all"ならば指定されたsku_idを含むかどうかを気にしない
  #   group_name : 指定されていたら、該当する関連商品グループのみを対象にする
  def self.find_by_include_sku_id(sku_id = nil, all = nil, group_name = nil)
    where = []
    param = []
    
    # 対象の商品を含む関連商品グループを取得する
    if all != "all"
      return [] unless sku_id
      target_groups = find(:all, :conditions => ["product_sku_id = ?", sku_id])
      return [] if target_groups.empty?
      
      where << ["product_group_skus.product_group_id IN (?)"]
      param << target_groups.map{|group| group.product_group_id}
      #自身を含まない
      where << ["product_group_skus.product_sku_id <> ?"]
      param << [sku_id]
    end
    
    # 指定した関連商品名の商品グループを取得する
    if group_name
      where << "product_groups.name = ?"
      param << group_name
    end
    
    # 環境に従って取得する商品の条件を制御する
    unless EcRider.config.ec_admin
      time_now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
      where << "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?))"
      param << time_now
      param << time_now
    end
    unless (EcRider.config.ec_test ||  EcRider.config.ec_admin)
      where << '(invisible_flg is null or invisible_flg = 0)'
    end
    where << "(present_flg is null or present_flg = 0)"

    joins = <<-"EOS"
      JOIN product_groups ON product_groups.id = product_group_skus.product_group_id
      JOIN product_skus ON product_skus.id = product_group_skus.product_sku_id AND product_skus.deleted_at IS NULL
      JOIN products ON products.id = product_skus.product_id AND products.deleted_at IS NULL
      JOIN product_displays ON products.id = product_displays.product_id AND product_displays.locale = '#{I18n.locale || "ja"}'
    EOS
    order = "product_groups.priority desc, product_groups.id, products.display_order is null, products.display_order, products.product_code"
    product_group_skus = ProductGroupSku.find(:all, :joins => joins, :conditions => [where.join(" AND ")] + param, :order => order)
    
    # SKUの重複を取り除く
    included_ids = []
    result = []
    product_group_skus.each do |sku|
      unless included_ids.include?(sku.product_sku_id)
        included_ids << sku.product_sku_id
        result << sku
      end
    end
    return result
  end
end
