# -*- coding: utf-8 -*-
require 'digest/sha1'
require 'iconv'
require 'base64'
class SpsParameterHistory < ActiveRecord::Base
  has_many :sps_parameter_history_items, :dependent => :destroy
  has_one :sps_result_history
  belongs_to :order

  acts_as_paranoid
  # 支払状況区分
  selectable_attr :payment_state_div do
    entry '1', :not_pay, '未購入'
    entry '2', :pay, '購入済'
  end

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  #
  # = OrderインスタンスをもとにSpsParameterHistoryインスタンスを生成し保存する =
  # 入力：Orderインスタンス
  # 出力：SpsParameterHistoryインスタンス
  #
  def self.create_from_order(order)
    sps_parameter_history = self.new
    sps_parameter_history.pay_method = 'alipay'
    sps_parameter_history.merchant_id = EcRider.config.sps_merchant_id
    sps_parameter_history.service_id = EcRider.config.sps_service_id
    sps_parameter_history.cust_code = 'x' # dummy
    sps_parameter_history.sps_cust_no = ''
    sps_parameter_history.sps_payment_no = ''
    sps_parameter_history.sps_order_id = 'x' # dummy
    sps_parameter_history.item_id = 'x' # dummy
    sps_parameter_history.pay_item_id = nil
    sps_parameter_history.item_name = nil
    sps_parameter_history.tax = nil
    amount = 0
    order.order_shippings.each do |order_shipping|
      amount += order_shipping.total_amount + order_shipping.shipping_cost
    end
    sps_parameter_history.amount = amount
    sps_parameter_history.pay_type = 0
    sps_parameter_history.auto_charge_type = nil
    sps_parameter_history.service_type = 0
    sps_parameter_history.div_settele = nil
    sps_parameter_history.last_charge_month = nil
    sps_parameter_history.camp_type = nil
    sps_parameter_history.tracking_id = nil
    sps_parameter_history.terminal_type = 0
    sps_parameter_history.success_url = EcRider.config.sps_success_url
    sps_parameter_history.cancel_url = EcRider.config.sps_cancel_url
    sps_parameter_history.error_url = EcRider.config.sps_error_url
    sps_parameter_history.pagecon_url = EcRider.config.sps_pagecon_url
    sps_parameter_history.free1 = nil
    sps_parameter_history.free2 = nil
    sps_parameter_history.free3 = nil
    sps_parameter_history.free_csv = self.encode_free_csv
    sps_parameter_history.request_date =  Time.now.strftime('%Y%m%d%H%M%S').to_i
    sps_parameter_history.limit_second = EcRider.config.sps_limit_second
    sps_parameter_history.payment_state_div = 1
    # ※Alipay決済ではsps_parameter_history_itemsのレコードは作成しない。他の決済方式では必要な場合がある。
    sps_parameter_history.save!

    # 連番を採番
    sps_parameter_history.cust_code = 'cust' + sps_parameter_history.id.to_s # 連番を採番（このカラムにおいてUniqueな値）
    sps_parameter_history.sps_order_id = 'order' + sps_parameter_history.id.to_s # 連番を採番（このカラムにおいてUniqueな値）
    sps_parameter_history.item_id = 'item' +  sps_parameter_history.id.to_s # 連番を採番（このカラムにおいてUniqueな値）

    # チェックサムを計算
    sps_parameter_history.calc_sps_hashcode

    sps_parameter_history.save!
    sps_parameter_history
  end

  #
  # 電文の値に対するチェックサムを計算する
  #
  def calc_sps_hashcode
    str = ATTRIBUTE_NAMES_FOR_CALC_SPS_HASHCODE.map{|attr| self.send(attr)}.join
    str << EcRider.config.sps_hash_key
    self.sps_hashcode = Digest::SHA1.hexdigest(str)
  end

  #
  # 決済取り消し依頼メールで表示するチェック前の項目情報を文字列にして返却する
  #
  def create_payment_content
    columns = ATTRIBUTE_NAMES_FOR_CALC_SPS_HASHCODE.map{|attr| attr + " : " + self.send(attr).to_s}
    # Alipay決済ではsps_parameter_history_itemsを必要としていないため
    # sps_parameter_history_itemsは今回は作成しない。
    # よってコメントアウトします。
#    self.sps_parameter_history_items.each do |history_item|
#      columns << "dtl_rowno : " +  history_item.dtl_rowno
#      columns << "dtl_item_id : " +  history_item.dtl_item_id
#      columns << "dtl_item_name : " +  history_item.dtl_item_name
#      columns << "dtl_item_count : " +  history_item.dtl_item_count
#      columns << "dtl_tax : " +  history_item.dtl_tax
#      columns << "dtl_amount : " +  history_item.dtl_amount
#      columns << "dtl_free1 : " +  history_item.dtl_free1
#      columns << "dtl_free2 : " +  history_item.dtl_free2
#      columns << "dtl_free3 : " +  history_item.dtl_free3
#    end
    # 改行
    str = ""
    columns.each{|column| str << column + "\n"}
    return str
  end
  
#使われていないのでコメントアウト ⇒　admin 受注管理で使用しています。
  def self.find_succeed(order_id)
    succeed_res_result = 'OK'
    conditions = 'order_id = ? AND sps_result_histories.res_result = ? AND sps_parameter_histories.order_id IS NOT NULL'
    
    find(:first, 
      :conditions => [conditions, order_id, succeed_res_result],
      :joins => 'JOIN sps_result_histories ON sps_parameter_histories.id = sps_result_histories.sps_parameter_history_id')
  end

  #
  # = 決済成功時に支払状況区分とECサイト注文見出しIDを更新する =
  # 入力：ECサイト注文見出しID
  #
  def update_when_pay_success(order_id)
    self.payment_state_div = self.class.payment_state_div_id_by_key(:pay)
    self.order_id = order_id
    self.save!
  end

  private
  #
  # アリペイ決済の表示文言（商品名またはサイト名）のBASE64エンコードを生成
  #
  def self.encode_free_csv
    free_csv = ShopSetting.get_setting(:aliyap_product_name, I18n.locale)
    if free_csv.blank?
      nil
    else
      free_csv = 'item_name=' << free_csv
      # 改行コードは削除する。
      Base64.encode64(Iconv.iconv('GBK', 'UTF-8', free_csv).to_s).gsub!("\n", "")
    end
  end

  #
  # チェックサムを計算する際のモデルの属性名
  #
  ATTRIBUTE_NAMES_FOR_CALC_SPS_HASHCODE = [
    'pay_method',
    'merchant_id',
    'service_id',
    'cust_code',
    'sps_cust_no',
    'sps_payment_no',
    'sps_order_id',
    'item_id',
    'pay_item_id',
    'item_name',
    'tax',
    'amount',
    'pay_type',
    'auto_charge_type',
    'service_type',
    'div_settele',
    'last_charge_month',
    'camp_type',
    'tracking_id',
    'terminal_type',
    'success_url',
    'cancel_url',
    'error_url',
    'pagecon_url',
    'free1',
    'free2',
    'free3',
    'free_csv',
    'request_date',
    'limit_second'
  ]
end
