# -*- coding: utf-8 -*-
require 'digest/sha1'
class SpsUserTemporary < ActiveRecord::Base
  has_one  :sps_user_email_temporary, :dependent => :destroy
  has_many :sps_user_address_temporaries, :order => :id, :dependent => :destroy

  attr_accessor :password
  
  # 会員自身の住所
  has_one  :address, :class_name => 'SpsUserAddressTemporary', :conditions => ['address_div=?',UserAddress.address_div_id_by_key(:user)]
  # Userモデルと同様に扱うための関連付け
  has_one  :user_email, :class_name => 'SpsUserEmailTemporary'
  
  before_save :encrypt_password

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # Encrypts some data with the salt.
  def self.encrypt(password, salt)
    Digest::SHA1.hexdigest("--#{salt}--#{password}--")
  end

  # Encrypts the password with the user salt
  def encrypt(password)
    self.class.encrypt(password, salt)
  end

  #
  # 姓名を連結して返す
  #
  def fullname
    (address.nil? ? '' : address.fullname)
  end
  
  #
  # = UserインスタンスをもとにSpsUserTemporaryインスタンスを生成し保存する =
  # 入力：Userインスタンス, SPS購入ID
  # 出力：SpsUserTemporaryインスタンス
  #
  def self.create_from_user(user, sps_order_id)
    sps_user_temporary_attributes = user.attributes.dup
    sps_user_temporary_attributes['sps_order_id'] = sps_order_id
    sps_user_temporary_attributes['password'] = user.password
    sps_user_temporary = self.new(sps_user_temporary_attributes)

    unless user.user_email.nil?
      sps_email_address_temporary_attributes = user.user_email.attributes.dup
      sps_email_address_temporary_attributes.delete('user_id')
      sps_email_address_temporary = SpsUserEmailTemporary.new(sps_email_address_temporary_attributes)
      sps_user_temporary.sps_user_email_temporary = sps_email_address_temporary
    end

    user.user_addresses.each do |address|
      sps_address_temporary_attributes = address.attributes.dup
      sps_address_temporary_attributes.delete('user_id')
      sps_address_temporary = SpsUserAddressTemporary.new(sps_address_temporary_attributes)
      sps_user_temporary.sps_user_address_temporaries << sps_address_temporary
    end

    sps_user_temporary.save!
    sps_user_temporary
  end

  protected
  # before filter
  def encrypt_password
    return if password.blank?
    self.salt = Digest::SHA1.hexdigest("--#{Time.now.to_s}--#{kana_name_first}--") if new_record?
    self.crypted_password = encrypt(password)
  end
end
