# -*- coding: utf-8 -*-
class UserEmail < ActiveRecord::Base
  versioned

  belongs_to :user

  selectable_attr :mail_address_pc_mobile_div do
    entry '1', :pc, 'PC'
    entry '2', :mobile, 'モバイル'
  end

  validates_presence_of :email,  :message => :EM990001
  validates_format_of      :email,  :with => /^([a-zA-Z0-9])+([a-zA-Z0-9\._+-])*@([a-zA-Z0-9_-])*\.([a-zA-Z0-9_-])+([a-zA-Z0-9\._-]+)+$/,   :message => :EM990007

  # メールアドレスのアカウント(@の前)を返す
  def account
    email[0..email.index("@")-1]
  end

  # メールアドレスのドメイン(@の後ろ)を返す
  def career
    email[email.index("@")..email.size]
  end

  def validate
    errors.add(:email, :NM990009) unless is_unique_email?
  end

  def before_save
    self.mail_address_pc_mobile_div = UserEmail.check_by_emall(email)
  end

  # メールアドレスがユニークな場合はtrueを返す
  #　会員ユーザーのみが対象。非会員、退会会員の重複は許可します。
  #　本番会員とテスト会員の重複は許可します。
  def is_unique_email?
    if EcRider.config.ec_test
      conditions = ['user_emails.email = ? and member_div = ? and user_emails.user_id <> ? and test_flg = ?', self.email, User.member_div_id_by_key(:membership), self.user_id || '', true]
    else
      conditions = ['user_emails.email = ? and member_div = ? and user_emails.user_id <> ? and test_flg is null', self.email, User.member_div_id_by_key(:membership), self.user_id || '']
    end
    user_count = User.count(
      :joins => "LEFT OUTER JOIN user_emails ON user_emails.user_id = users.id  AND (user_emails.deleted_at IS NULL OR user_emails.deleted_at > '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}')" ,
      :conditions => conditions
    )
    return (user_count <= 0) || (self.user.member_div_key != :membership)
  end

  # ユーザー情報を含んだメールアドレス情報を返す。
  def self.email_with_user(email)
    if EcRider.config.ec_test
      conditions = ['user_emails.email = ? and users.member_div = ? and users.test_flg = ?', email, User.member_div_id_by_key(:membership), true]
    else
      conditions = ['user_emails.email = ? and users.member_div = ? and users.test_flg is null', email, User.member_div_id_by_key(:membership)]
    end
    self.find(:first,
      :joins => "LEFT OUTER JOIN users ON user_emails.user_id = users.id AND (user_emails.deleted_at IS NULL OR user_emails.deleted_at > '#{Time.now.strftime('%Y-%m-%d %H:%M:%S')}')" ,
      :conditions => conditions)
  end

  # 指定ユーザーのメールアドレス情報を返す
  def self.find_by_user_id(user_id)
    raise ArgumentError, "Illegal arguments! user_id is nil" unless user_id
    self.find(:first, :conditions => ['user_id = ?', user_id])
  end

  MOBILE_DOMAIN_PATTERN = /^[^@]+@(docomo|ezweb|softbank|[^\.]+\.pdx|[^\.]+\.vodafone).ne.jp$/

  # メールアドレスが携帯のアドレスかどうかを判定しPCモバイル区分で返す。
  def self.check_by_emall(email)
    UserEmail.mail_address_pc_mobile_div_id_by_key(
      email =~ MOBILE_DOMAIN_PATTERN ? :mobile : :pc)
  end

end
