# -*- coding: utf-8 -*-
module EcRider
  module CmsSetup

    ECR_CONTROLLERS = ['account', 'cart', 'inquiry', 'member', 'order', 'reminder', 'resigned_reason', 'thanks']

    #
    # ベースプロジェクトのec-rider提供ページをデータベースに登録します。
    # initial_data/cms/ecrider_pages以下のファイルを登録します。
    #
    def create_ecrider_cms_pages
      ECR_CONTROLLERS.each do |controller|
        files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'ecrider_pages', controller, "*.xml"))
        update_template_content_from_fixture(files)
      end
    end

    #
    # ベースプロジェクトの店舗独自ページをデータベースに登録します。
    # initial_data/cms/cms_pages以下のファイルをすべて登録します。
    #
    def create_shop_base_cms_pages
      files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_pages', '*'))
      files.reject!{|f| f =~ /\/\.{1,2}$/}
      update_template_content_from_fixture(files)
    end

    #
    # ベースプロジェクトの中央以外のテンプレートをデータベースに登録します。
    # initial_data/cms/layouts以下のファイルを登録します。
    #
    def create_shop_base_cms_templates
      CmsTemplate.display_position_keys.each do |position|
        files = Dir.glob(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'cms_layouts', "#{position}*.xml"))
        update_template_content_from_fixture(files)
      end
      # topとbottomをデフォルトテンプレートに設定します
      template = CmsTemplate.find(:first, :conditions => ['development_code = ?', 'cms/top'])
      template.default_template_position_key = :top
      template.save!
      template = CmsTemplate.find(:first, :conditions => ['development_code = ?', 'cms/bottom'])
      template.default_template_position_key = :bottom
      template.save!
    end
    
    #
    # test/fixtures/public以下のファイルをベースプロジェクトのファイルとして登録します。
    #
    def create_base_cms_file
      src_root_dir = File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms')
      src_files = Dir.glob(File.join(src_root_dir, 'public', "**", '*'))
      dest_root_dir = File.join(EcRider.config.ec_admin_root, 'public', 'projects', CmsProject::BASE_PROJECT_ID.to_s)
      
      src_files.each do |src_file|
        next if File.basename(src_file) =~ /^\./
        if File.directory?(src_file)
          # ディレクトリの場合は末尾に/をつける
          public_file_path = src_file.sub(src_root_dir + '/', '') + '/'
          dest_file = File.join(dest_root_dir, public_file_path)
          FileUtils.mkdir_p(dest_file) unless File.exist?(dest_file)
          # ディレクトリの場合はcms_project_id=nil
          cms_project_id = nil
        else
          public_file_path = src_file.sub(src_root_dir + '/', '')
          dest_file = File.join(dest_root_dir, public_file_path)
          FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
          FileUtils.cp(src_file, dest_file)
          # ファイルの場合はcms_project_id=ベースプロジェクトのID
          cms_project_id = CmsProject::BASE_PROJECT_ID
        end
        unless CmsFile.exists?(:cms_project_id => cms_project_id, :public_file_path => public_file_path)
          CmsFile.create(:cms_project_id => cms_project_id, :public_file_path => public_file_path)
        end
      end
    end

    private
    # EC-Riderが想定している（initial_dataのテンプレートに含まれている）ロケール群から、
    # 日本語を除いたロケールのXMLのファイル名にマッチする正規表現
    ECR_KNOWN_FOREIGN_LOCALES_REGEXP = Regexp.union(
      %w(en ko th zh_CN zh_TW).map{|locale| /_#{locale}\.xml$/}
      )

    # 実際に店舗担当者が扱うロケール群
    SUPPORED_LOCALES = I18n.supported_locales.
      reject{|locale| locale.to_s == 'ja'}.map{|locale| locale.to_s.gsub('-', '_')}

    #
    # ファイルの内容でテンプレートを更新します。
    #
    def update_template_content_from_fixture(files)
      files_ja = files.reject{|file| file =~ ECR_KNOWN_FOREIGN_LOCALES_REGEXP}
      files_ja.each do |file_ja|
        controller = (File.dirname(file_ja) =~ /\/cms_layouts$|\/cms_pages$/) ? 'cms' : "#{File.dirname(file_ja).split('/').last}"
        if template = CmsTemplate.find(:first, :include => :contents,
          :conditions => ["cms_project_id = ? and development_code = ?", CmsProject::BASE_PROJECT_ID, "#{controller}/#{File.basename(file_ja, '.xml')}"])

          template.content_ja.descriptions = IO.read(file_ja)
          template.content_ja.save!
          SUPPORED_LOCALES.each do |foreign_locale|
            if File.exists?(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
              if template.send("content_#{foreign_locale}")
                template.send("content_#{foreign_locale}").descriptions = IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
                template.send("content_#{foreign_locale}").save!
              else
                template.contents << CmsTemplateContent.new(:locale => foreign_locale.to_s, :descriptions => IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml")))
              end
            end
          end
          template.save!
        else
          create_cms_template_from_fixture(file_ja)
        end
      end
    end

    #
    # ファイルの内容でテンプレートとページのデータを作成します。
    #
    def create_cms_template_from_fixture(file_ja)
      # cms_templates,cms_template_contents
      controller = (File.dirname(file_ja) =~ /\/cms_layouts$|\/cms_pages$/) ? 'cms' : "#{File.dirname(file_ja).split('/').last}"
      cms_template = CmsTemplate.new(
        :name => "#{controller}/#{File.basename(file_ja, '.xml')}",
        :display_position => CmsTemplate.display_position_id_by_key(:center),
        :cms_project_id => CmsProject::BASE_PROJECT_ID,
        :development_code => "#{controller}/#{File.basename(file_ja, '.xml')}")

      cms_template.contents << CmsTemplateContent.new(:locale => 'ja', :descriptions => IO.read(file_ja))
      SUPPORED_LOCALES.each do |foreign_locale|
        if File.exists?(file_ja.sub(".xml", "_#{foreign_locale}.xml"))
          cms_template.contents << CmsTemplateContent.new(:locale => foreign_locale.to_s, :descriptions => IO.read(file_ja.sub(".xml", "_#{foreign_locale}.xml")))
        end
      end
      cms_template.template_code = Time.now.strftime("%Y%m%d%H%M%S") + cms_template.object_id.abs.to_s
      begin
        cms_template.save!
      rescue
        ActiveRecord::Base.logger.error("#{$!.to_s}\n#{cms_template.inspect}\n  " << $!.backtrace.join("\n  ") )
        raise
      end

      if File.dirname(file_ja) !~ /\/layouts$/
        # cms_pages,cms_page_titles,cms_binds
        cms_page = CmsPage.new(:cms_project_id => CmsProject::BASE_PROJECT_ID, :controller => controller, :action => File.basename(file_ja, ".xml"))
        cms_page.binds << CmsBind.new(:cms_template_id => cms_template.id, :position_key => :center)
        CmsBind.position_keys.each{|key| cms_page.binds << CmsBind.new(:position_key => key) unless key == :center}
        I18n.supported_locales.each do |locale|
          cms_page.titles << CmsPageTitle.new(:locale => locale.to_s, :name => File.basename(file_ja, ".xml"))
        end

        begin
          cms_page.save!
        rescue
          ActiveRecord::Base.logger.error("#{$!.to_s}\n#{cms_page.inspect}\n  " << $!.backtrace.join("\n  ") )
          raise
        end
      end
    end

    public
    #
    # ベースプロジェクト以外のプロジェクトのデータを削除します。
    # ベースプロジェクトを初期化します。
    #
    def cleanup_project
      cms_projects = CmsProject.find(:all,
        :include => [{:pages => [{:binds => {:items => :contents}}, :titles]}, :files, {:templates => :contents}, :assigns],
        :conditions => ["cms_projects.id <> ?", CmsProject::BASE_PROJECT_ID])

      cms_projects.each do |project|
        project.pages.each do |page|
          page.binds.each do |bind|
            bind.items.each do |item|
              item.contents.each{|content| content.destroy}
              item.destroy
            end if bind.items
            bind.destroy
          end
          page.titles.each{|title| title.destroy}
          page.destroy
        end
        project.templates.each do |template|
          template.contents.each{|content| content.destroy}
          template.destroy
        end
        project.files.each{|file| file.destroy}
        project.assigns.each{|assign| assign.destroy}
        project.destroy
      end
      CmsProjectAssign.destroy_all
      CmsPublish.destroy_all
      CmsPublishedProject.destroy_all
      CmsProjectApproval.destroy_all
      CmsProject.update(1, :acceptance_type_key => :no_approval, :status_key => :approved)
    end
  end
end
