# -*- coding: utf-8 -*-
require File.join(File.dirname(__FILE__), '..', 'spec_helper')

describe PaymentsController do
  fixtures :gmo_transaction_histories, :users, :user_emails, :user_addresses

  before do
    # adminでログイン
    login_as("admin")
  end

  describe "GET init_index" do
    it "init_index" do
      get :init_index
      response.should redirect_to(:action => 'index')
    end
  end

  describe "GET user_detail" do
    it "get user detail" do
      get :user_detail, :id => 1007
      controller.instance_variable_get(:@user).should_not be_nil
      controller.instance_variable_get(:@user_address).should_not be_nil
      response.should be_success
    end
  end

  describe "POST update" do
    it "update" do
      post :update, :payment_system_div => "1", :updatable_id => "1", :page => 1, :history => { :status_div => "1" }
      response.should redirect_to(:action => 'search', :page => 1)
      result = PaymentErrorHistory.find(:first, :conditions => ["payment_system_div = ? AND updatable_id = ? ", 1, 1])
      result.status_div.should == "1"
    end
  end

  describe "POST update_form" do
    it "update_form" do
      post :update_form, :payment_system_div => 1, :updatable_id => 1
      controller.instance_variable_get(:@user).should_not be_nil
      controller.instance_variable_get(:@address).should_not be_nil
      controller.instance_variable_get(:@email).should_not be_nil
      response.should be_success
    end
  end

  describe "GET index" do
    it "index new" do
      session[:payments_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
    end

    it "index with session" do
      now_date = Time.now.strftime('%Y/%m/%d')
      session[:payments_search_cond] = { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      flash[:error] = nil
      get :index
      response.should be_success
    end

    it "index error" do
      now_date = Time.now.strftime('%Y/%m/%d')
      session[:payments_search_cond] = { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      flash[:error] = "AA"
      get :index
      response.should be_success
    end

    it "index reload" do
      session[:payments_search_cond] = nil
      flash[:error] = nil
      get :index
      response.should be_success
      cond = controller.instance_variable_get(:@condition)
      cond.exe_time_from.should ==  '00:00:00'
      cond.exe_time_to.should == '23:59:59'
    end
  end

  describe "GET search" do
    it "search" do
      now_date = Time.now.strftime('%Y/%m/%d')
      get :search, :condition => { :exe_date_from => now_date, :exe_time_from => '00:00:00', :exe_date_to => now_date, :exe_time_to => '23:59:59' }
      response.should be_success
      controller.instance_variable_get(:@condition).should_not == nil
    end

    it "search with condition error" do
      get :search, :condition => { :exe_time_from => 'ERROR' }
      response.should redirect_to :action => 'index'
    end
  end

  describe "delete" do
    it "テスト会員を全て削除 テスト会員がない場合" do
      lambda {
        delete :destroy, :destroy_all_test => true, :page => '1'
      }.should change(PaymentErrorHistory, :count).by(0)
      response.should redirect_to(:action => 'search', :page => '1')
    end

    it "テスト会員を全て削除 テスト会員がある場合" do
      GmoTransactionHistory.update(1, :test_flg => true)
      SpsParameterHistory.update(1, :test_flg => true)
      SpsResultHistory.update(1, :test_flg => true)
      gth_size = GmoTransactionHistory.find(:all).size
      sph_size = SpsParameterHistory.find(:all).size
      srh_size = SpsResultHistory.find(:all).size
      delete :destroy, :destroy_all_test => true, :page => '1'
      GmoTransactionHistory.find(:all).size.should == gth_size - 1
      SpsParameterHistory.find(:all).size.should == sph_size - 1
      SpsResultHistory.find(:all).size.should == srh_size - 1
      response.should redirect_to(:action => 'search', :page => '1')
    end
  end
  
  describe "POST mail" do
    it "mail from no error" do
      flash[:mail_form] = nil
      post :mail, :payment_system_div => "1", :updatable_id => "1"
      response.should be_success
    end

    it "mail from error" do
      mf = MailForm.new()
      mf.from = "test@ecrider.com"
      mf.to = "test@ecrider.com"
      mf.subject = "title"
      mf.body = "mail body"
      flash[:mail_form] = mf
      post :mail, :payment_system_div => "1", :updatable_id => "1"
      response.should be_success
    end
  end

  describe "POST send_mail" do
    it "ok" do
      post :send_mail, :payment_system_div => "1", :updatable_id => "1", :page => 1, :mail_form => {:from => "test@ecrider.com", :to => "test@ecrider.com", :subject => "メールタイトル", :body => "本文" }
      response.should be_success
    end

    it "ng" do
      post :send_mail , :payment_system_div => "1", :updatable_id => "1", :page => 1, :mail_form => { :subject => "メールタイトル", :body => "本文" }
      response.should redirect_to(:action => :mail, :payment_system_div => "1", :updatable_id => "1", :page => 1)
    end
  end

end
