require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ProductComponentsController do

  before(:each) do
    login_as("admin")
  end

  def mock_product_component(stubs={})
    @mock_product_component ||= mock_model(ProductComponent, stubs)
  end
  
  describe "GET index" do
    it "assigns all product_components as @product_components" do
      ProductComponent.stub!(:find).with(:all).and_return([mock_product_component])
      get :index
      assigns[:product_components].should == [mock_product_component]
    end
  end

  describe "GET show" do
    it "assigns the requested product_component as @product_component" do
      ProductComponent.stub!(:find).with("37").and_return(mock_product_component)
      get :show, :id => "37"
      assigns[:product_component].should equal(mock_product_component)
    end
  end

  describe "GET new" do
    it "assigns a new product_component as @product_component" do
      ProductComponent.stub!(:new).and_return(mock_product_component)
      get :new
      assigns[:product_component].should equal(mock_product_component)
    end
  end

  describe "GET edit" do
    it "assigns the requested product_component as @product_component" do
      ProductComponent.stub!(:find).with("37").and_return(mock_product_component)
      get :edit, :id => "37"
      assigns[:product_component].should equal(mock_product_component)
    end
  end

  describe "POST create" do
    
    describe "with valid params" do
      it "assigns a newly created product_component as @product_component" do
        ProductComponent.stub!(:new).with({'these' => 'params'}).and_return(mock_product_component(:save => true))
        post :create, :product_component => {:these => 'params'}
        assigns[:product_component].should equal(mock_product_component)
      end

      it "redirects to the created product_component" do
        ProductComponent.stub!(:new).and_return(mock_product_component(:save => true))
        post :create, :product_component => {}
        response.should redirect_to(product_component_url(mock_product_component))
      end
    end
    
    describe "with invalid params" do
      it "assigns a newly created but unsaved product_component as @product_component" do
        ProductComponent.stub!(:new).with({'these' => 'params'}).and_return(mock_product_component(:save => false))
        post :create, :product_component => {:these => 'params'}
        assigns[:product_component].should equal(mock_product_component)
      end

    end
    
  end

  describe "PUT update" do
    
    describe "with valid params" do
      it "updates the requested product_component" do
        ProductComponent.should_receive(:find).with("37").and_return(mock_product_component)
        mock_product_component.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_component => {:these => 'params'}
      end

      it "assigns the requested product_component as @product_component" do
        ProductComponent.stub!(:find).and_return(mock_product_component(:update_attributes => true))
        put :update, :id => "1"
        assigns[:product_component].should equal(mock_product_component)
      end

      it "redirects to the product_component" do
        ProductComponent.stub!(:find).and_return(mock_product_component(:update_attributes => true))
        put :update, :id => "1"
        response.should redirect_to(product_component_url(mock_product_component))
      end
    end
    
    describe "with invalid params" do
      it "updates the requested product_component" do
        ProductComponent.should_receive(:find).with("37").and_return(mock_product_component)
        mock_product_component.should_receive(:update_attributes).with({'these' => 'params'})
        put :update, :id => "37", :product_component => {:these => 'params'}
      end

      it "assigns the product_component as @product_component" do
        ProductComponent.stub!(:find).and_return(mock_product_component(:update_attributes => false))
        put :update, :id => "1"
        assigns[:product_component].should equal(mock_product_component)
      end

    end
    
  end

  describe "DELETE destroy" do
    it "destroys the requested product_component" do
      ProductComponent.should_receive(:find).with("37").and_return(mock_product_component)
      mock_product_component.should_receive(:destroy)
      delete :destroy, :id => "37"
    end
  
    it "redirects to the product_components list" do
      ProductComponent.stub!(:find).and_return(mock_product_component(:destroy => true))
      delete :destroy, :id => "1"
      response.should redirect_to(product_components_url)
    end
  end

end
