# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :products, :product_displays, :distributors, :distributor_delivery_times, :distributor_delivery_time_displays

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @order_form = OrderForm.new(:send_address_index => 0)
    @template.instance_variable_set(:@order_form, @order_form)

    @shipping_address = UserAddress.new()
    @template.instance_variable_set(:@shipping_address, @shipping_address)
    @template.extend(ApplicationHelper)
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:order_form" do
    it "normal usage" do
      ec.order_form do
        concat(ec.order_form_error_messages || '')
        concat(ec.order_form_payment_type)
        concat(ec.gift_check)
        concat(ec.send_invoice_separately_check)
        concat(ec.write_sender_check)
        ec.order_form_card_kind do
          @template.concat(ec.order_form_card_kind_radio)
          @template.concat(ec.order_form_card_kind_label)
        end
        ec.order_form_card_no :card_kind => "normal", :name_for_error => "カード番号" do
          @template.concat(ec.order_form_card_no_field)
          @template.concat(ec.order_form_card_no_example {"例：1234-5678-9000-0000 ※16桁"})
        end
        concat(ec.order_form_delivery_time_radio_blank)
        ec.order_form_delivery_time do
          @template.concat(ec.order_form_delivery_time_radio)
          @template.concat(ec.order_form_delivery_time_label)
        end
        concat(ec.order_form_card_no_error_message {""})
        concat(ec.order_form_expire_year)
        concat(ec.order_form_expire_month)
        concat(ec.order_form_free_request)
        concat(ec.order_form_card_name)
        
        @template.output_buffer
      end
      
      @template.output_buffer.should have_tag('form[action=?]', "/order/confirmation_order") do
        with_tag("select#order_form_normal_payment_type")
        with_tag("input#order_form_gift_flg[type=?]", 'checkbox')
        with_tag("input#order_form_send_invoice_separately_flg[type=?]", 'checkbox')
        with_tag("input#order_form_write_sender_flg[type=?]", 'checkbox')
        
        with_tag("input#order_form_card_kind_visa[type=?][value=?]", 'radio', 'visa')
        with_tag("label[for=?]", 'order_form_card_kind_visa', 'Visa')
        with_tag("input#order_form_card_kind_master[type=?][value=?]", 'radio', 'master')
        with_tag("label[for=?]", 'order_form_card_kind_master', 'MasterCard')

        with_tag("input#order_form_no_1")
        with_tag("input#order_form_no_2")
        with_tag("input#order_form_no_3")
        with_tag("input#order_form_no_4")
        with_tag("span#card_no_err_msg")
        with_tag("p[class=?]", ' sample', '例：1234-5678-9000-0000 ※16桁')

        with_tag("input#order_form_expire_year")
        with_tag("input#order_form_expire_month")
        
        with_tag("input#order_form_delivery_time_")
        with_tag("input#order_form_delivery_time_1")

        with_tag("input#order_form_card_name")

        with_tag("textarea#order_form_free_request")
      end
    end
  end
  
  describe "order_confirmation_form" do
    # 
    # 次にこのファイルを編集する方は、下記の fixtures :all の記述を、適切なフィクスチャのみを指定するように
    # 書き換えてください。その書き換えがうまくいくかどうかは、このコマンドで確認できます。
    # 
    # rake spec SPEC=spec/helpers/ec_rider/ec_helper/order_form_tag_spec.rb
    # 
    fixtures :all

    before(:each) do
      @user_address = {:tel_num => "000-0000-0000", :kana_name_last => "テスト", :kana_name_first => "ジロウ", :name_last => "てすと",
        :name_first => "じろう", :zip_code => "0000000", :address_1 => "福岡県", :address_2 => "福岡市中央区",
        :address_3 => "天神１－１－１", :country_code => 'JP', :locale => 'ja'}
      @user_email = {:email => "test@test.jp"}
      @user_profile = {:birth_month => "1", :dm_send_div => "0", :job_div => "1",
        :birth_day => "23", :sex => "1", :birth_year => "1970",
        :password => 'test1', :password_confirmation => 'test1', :remember_question => 'Q', :remember_answer => 'A',
        :kana_name_last => "テスト",  :kana_name_first => "ジロウ"}
        
      user_email = UserEmail.new(@user_email)
      user = User.new(@user_profile.merge(:user_address => UserAddress.new(@user_address.merge(:country => Country.new(:code=>'JP', :name=>'日本'))), :user_email => user_email))
      user_address = UserAddress.new(@user_address)
      order_form = OrderForm.new(:send_address_index => 0, :free_request => 'free', :normal_payment_type => 1)
      cart = Cart.new()
      cart.add('0000000000001-00-00-00', :normal, 1)
      cart.calculate
      
      order = Order.create_new_order(user, user_address, order_form, cart)
      order.write_sender_flg = true
      
      @template.instance_variable_set(:@user_email, user_email)
      @template.instance_variable_set(:@user_address, user_address)
      @template.instance_variable_set(:@user, user)
      @template.instance_variable_set(:@order, order)

    end

    it "ec:order_confirmation_form" do
      ec.order_confirmation_form do 
        ec.stock_error_messages.should == nil
        ec.order_product_shipping_cost.should == "110"
        ec.order_product_total_price_include_shipping_cost.should == "8,110"
        ec.order_confirmation_form_full_address.should == '〒0000000<br/>福岡県福岡市中央区 天神１－１－１'
        ec.order_confirmation_form_email.should == 'test@test.jp'
        ec.order_confirmation_form_gender_name.should == '男性'
        ec.order_confirmation_form_birth_day.should == '1970/1/23'
        ec.order_confirmation_form_job_div_name.should == '会社員'
        ec.order_confirmation_form_payment_type.should == '代金引換'
        ec.order_confirmation_form_free_request.should == 'free'
        ec.order_confirmation_form_shipping_address.should == '〒0000000<br/>福岡県福岡市中央区 天神１－１－１'
        ec.order_confirmation_form_full_name.should == 'てすと じろう'
        ec.order_confirmation_form_shipping_full_name.should == 'てすと じろう'
        ec.if_address_japanese_format.should do
          ec.order_confirmation_form_kana_last_name.should == 'テスト'
          ec.order_confirmation_form_kana_first_name.should == 'ジロウ'
        end
        ec.if_shipping_address_japanese_format.should do
          ec.order_confirmation_form_kana_shipping_last_name.should == 'テスト'
          ec.order_confirmation_form_kana_shipping_first_name.should == 'ジロウ'
        end
        ec.if_stock_error.should_not do
          true.should be_true
        end
        ec.if_gift.should do
          #実行されない
        end
        ec.unless_gift.should_not do
          #実行される
        end
        ec.if_send_invoice_separately.should_not do
          #実行されない
        end
        ec.unless_send_invoice_separately.should do
          #実行される
        end
        ec.if_write_sender.should do
          #実行されない
        end
        ec.unless_write_sender.should_not do
          #実行される
        end
        ec.order_confirmation_form_country.should == 'Japan'
        ec.order_confirmation_form_tel_num.should == '000-0000-0000'
        ec.order_confirmation_form_shipping_country.should == 'Japan'
        ec.order_confirmation_form_shipping_tel_num.should == '000-0000-0000'
        @template.output_buffer
      end
      
    end

  end

end
