require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe Accounting do
  fixtures :kbn_codes, :kbn_code_names,
    :order_shippings, :orders, :users, 
    :user_emails, :order_items, :products, :product_skus, :user_addresses, :gmo_credit_cards, :gmo_transaction_histories,
    :product_sku_inventories,:product_sku_secure_inventories,
    :kbn_code_names, :kbn_codes, :shop_settings, :mail_deliveries, :mail_templates,
    :user_points, :point_settings, :accountings
  
  describe "create_for_order" do
    it "銀行振り込み　獲得ポイントなし　利用ポイントなし" do
      order_shipping = OrderShipping.find(4)
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :order_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :order_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 0
    end
    
    it "銀行振り込み　獲得ポイントなし　利用ポイントあり" do
      Accounting.create!(
        :order_id => 1,
        :debt_sales => 10000,
        :debt_account_div_key => :order_cash,
        :credit_sales => 10000,
        :credit_account_div_key => :order_cash,
        :given_point => 300,
        :used_point => 0,
        :previous_point => 300
      )
      order_shipping = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :order_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :order_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 100
      accounting[0].previous_point.should == 200
    end
  
    it "クレジット　獲得ポイントあり　利用ポイントなし" do
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :order_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :order_receivable
      accounting[0].given_point.should == 1000
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 1000
    end
  
    it "クレジット　獲得ポイントあり　利用ポイントあり" do
      order_shipping = OrderShipping.find(2)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_order(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :order_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :order_receivable
      accounting[0].given_point.should == 1000
      accounting[0].used_point.should == 100
      accounting[0].previous_point.should == 900
    end
  
    it "テスト注文は登録しない" do
      order_shipping = OrderShipping.find(2)
      order_shipping.order.test_flg = true
      order_shipping.order.save!
      lambda {
        Accounting.create_for_order(order_shipping)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_cancel" do
    it "銀行振り込み　獲得ポイントなし　ポイント利用なし" do
      UserPoint.destroy(4)
      order_shipping = OrderShipping.find(4)
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :cancel_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :cancel_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 0
    end
    
    it "銀行振り込み　獲得ポイントなし　ポイント利用あり" do
      Accounting.create!(
        :order_id => 1,
        :debt_sales => 10000,
        :debt_account_div_key => :cancel_receivable,
        :credit_sales => 10000,
        :credit_account_div_key => :cancel_receivable,
        :given_point => 300,
        :used_point => 0,
        :previous_point => 300
      )
      order_shipping = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 11340
      accounting[0].debt_account_div_key.should == :cancel_cash
      accounting[0].credit_sales.should == 11340
      accounting[0].credit_account_div_key.should == :cancel_cash
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == -100
      accounting[0].previous_point.should == 400
    end
  
    it "クレジット　ポ獲得ポイントあり　イント利用なし" do
      order_shipping = OrderShipping.find(2)
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :cancel_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :cancel_receivable
      accounting[0].given_point.should == -1000
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == -1000
    end
  
    it "クレジット　獲得ポイントあり　ポイント利用あり" do
      order_shipping = OrderShipping.find(2)
      UserPoint.create!(
        :user_id => order_shipping.order.user_id,
        :order_id => order_shipping.order_id,
        :term_date => "2018-01-02 16:15:21",
        :given_point => 0,
        :used_point => 100
      )
      Accounting.create_for_cancel(order_shipping)
      accounting = Accounting.find(:all, :conditions => ["order_id = ?", order_shipping.order_id])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 15750
      accounting[0].debt_account_div_key.should == :cancel_receivable
      accounting[0].credit_sales.should == 15750
      accounting[0].credit_account_div_key.should == :cancel_receivable
      accounting[0].given_point.should == -1000
      accounting[0].used_point.should == -100
      accounting[0].previous_point.should == -900
    end
  
    it "テスト注文は登録しない" do
      order_shipping = OrderShipping.find(2)
      order_shipping.order.test_flg = true
      order_shipping.order.save!
      lambda {
        Accounting.create_for_order(order_shipping)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_summary" do
    it "キャンセル含まない" do
      pending
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 1000, :used_point => 0
      )
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 0, :used_point => 300
      )
      Accounting.create_for_order(order_shipping_3)
      order_shipping_4 = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping_4.order.user_id, :order_id => order_shipping_4.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 400, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_4)
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.count.should == 4
      
      Accounting.create_for_point_summary
      expected_point_summary = 1000 + 1000 - 300 + 400 + 600
      accounting = Accounting.find(:all, :conditions => ["order_id is null"])
      accounting.length.should == 1
      accounting[0].debt_sales.should == expected_point_summary * 10
      accounting[0].debt_account_div_key.should == :point_summary
      accounting[0].credit_sales.should == expected_point_summary * 10
      accounting[0].credit_account_div_key.should == :point_summary
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == expected_point_summary
    end
    
    it "キャンセル含む" do
      pending
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 1000, :used_point => 0
      )
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 0, :used_point => 300
      )
      Accounting.create_for_order(order_shipping_3)
      order_shipping_4 = OrderShipping.find(4)
      UserPoint.create!(
        :user_id => order_shipping_4.order.user_id, :order_id => order_shipping_4.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 400, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_4)
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.create_for_cancel(order_shipping_3)
      Accounting.count.should == 5
      
      Accounting.create_for_point_summary
      expected_point_summary = 1000 + 1000 - 300 + 400 + 600 - 1000 + 300
      accounting = Accounting.find(:all, :conditions => ["order_id is null"])
      accounting.length.should == 1
      accounting[0].debt_sales.should == expected_point_summary * 10
      accounting[0].debt_account_div_key.should == :point_summary
      accounting[0].credit_sales.should == expected_point_summary * 10
      accounting[0].credit_account_div_key.should == :point_summary
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == expected_point_summary
    end
    
    it "キャンセル含む　一度マイナスになる" do
      pending
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      order_shipping_3 = OrderShipping.find(3)
      UserPoint.create!(
        :user_id => order_shipping_3.order.user_id, :order_id => order_shipping_3.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 0, :used_point => 300
      )
      Accounting.create_for_order(order_shipping_3)
      Accounting.find(:first, :order => 'id desc').previous_point.should == 700
      Accounting.create_for_cancel(order_shipping_2)
      Accounting.find(:first, :order => 'id desc').previous_point.should == -300
      order_shipping_9 = OrderShipping.find(9)
      UserPoint.create!(
        :user_id => order_shipping_9.order.user_id, :order_id => order_shipping_9.order_id, :term_date => "2018-01-02 16:15:21",
        :given_point => 600, :used_point => 0
      )
      Accounting.create_for_order(order_shipping_9)
      Accounting.count.should == 4
      
      Accounting.create_for_point_summary
      expected_point_summary = 600
      accounting = Accounting.find(:all, :conditions => ["order_id is null"])
      accounting.length.should == 1
      accounting[0].debt_sales.should == expected_point_summary * 10
      accounting[0].debt_account_div_key.should == :point_summary
      accounting[0].credit_sales.should == expected_point_summary * 10
      accounting[0].credit_account_div_key.should == :point_summary
      accounting[0].given_point.should == 0
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == expected_point_summary
    end
    
    it "ポイント制度を利用していない場合は登録しない" do
      Time.stub!(:now).and_return(Time.local(2019, 1, 1, 0, 10, 00))
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = false
      point_setting.save!
      lambda {
        Accounting.create_for_point_summary
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_expire" do
    it "ポイント期限切れ" do
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.create!(:user_id => 1001, :order_id => 2, :term_date => Date.today, :given_point => -1000, :used_point => 0)
      Accounting.create_for_point_expire(user_point)
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 1000 * 10
      accounting[0].debt_account_div_key.should == :point_expired
      accounting[0].credit_sales.should == 1000 * 10
      accounting[0].credit_account_div_key.should == :point_expired
      accounting[0].given_point.should == -1000
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 0
    end
  end
  
  describe "create_for_point_create" do
    it "ポイント追加" do
      order_shipping_2 = OrderShipping.find(2)
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.create!(:user_id => 1001, :term_date => Date.today, :given_point => 500)
      Accounting.create_for_point_create(user_point)
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 1
      accounting[0].debt_sales.should == 500 * 10
      accounting[0].debt_account_div_key.should == :point_created
      accounting[0].credit_sales.should == 500 * 10
      accounting[0].credit_account_div_key.should == :point_created
      accounting[0].given_point.should == 500
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 1500
    end
  end
  
  describe "create_for_point_update" do
    it "ポイント変更　付与ポイントを増やす" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 1100
      user_point.used_point = 0
      user_point.save
      Accounting.create_for_point_update(user_point)
      expected_point_summary = 1100 - 1000 - 0
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 1
      accounting[0].order_id.should == 2
      accounting[0].debt_sales.should == expected_point_summary * 10
      accounting[0].debt_account_div_key.should == :point_updated
      accounting[0].credit_sales.should == expected_point_summary * 10
      accounting[0].credit_account_div_key.should == :point_updated
      accounting[0].given_point.should == 100
      accounting[0].used_point.should == 0
      accounting[0].previous_point.should == 1100
    end
    
    it "ポイント追加　付与ポイントを減らす" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 800
      user_point.used_point = 100
      user_point.save
      Accounting.create_for_point_update(user_point)
      expected_point_summary = 800 - 1000 - 100
      accounting = Accounting.find(:all, :conditions => ["user_point_id is not null"])
      accounting.length.should == 1
      accounting[0].order_id.should == 2
      accounting[0].debt_sales.should == expected_point_summary.abs * 10
      accounting[0].debt_account_div_key.should == :point_updated
      accounting[0].credit_sales.should == expected_point_summary.abs * 10
      accounting[0].credit_account_div_key.should == :point_updated
      accounting[0].given_point.should == -200
      accounting[0].used_point.should == 100
      accounting[0].previous_point.should == 700
    end
    
    it "キャンセル、ポイント削除、ポイント期限切れがある場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      user_point.given_point = 1100
      user_point.used_point = 0
      user_point.save
      lambda {
        Accounting.create_for_point_update(user_point)
      }.should change(Accounting, :count).by(0)
    end
  end
  
  describe "create_for_point_destroy" do
    it "ポイント削除" do
      order_shipping_2 = OrderShipping.find(2)
      order_shipping_2.status_div_key = :delivered
      order_shipping_2.save!
      Accounting.create_for_order(order_shipping_2)
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      Accounting.create_for_point_destroy(user_point)
      accounting = Accounting.find(:all, :conditions => ["order_id = 2"], :order => "id")
      accounting.length.should == 2
      accounting[0].order_id.should == 2
      accounting[1].debt_sales.should == 1000 * 10
      accounting[1].debt_account_div_key.should == :point_destroyed
      accounting[1].credit_sales.should == 1000 * 10
      accounting[1].credit_account_div_key.should == :point_destroyed
      accounting[1].given_point.should == -1000
      accounting[1].used_point.should == 0
      accounting[1].previous_point.should == 0
    end
    
    it "キャンセル、ポイント削除、ポイント期限切れがある場合は登録しない" do
      user_point = UserPoint.find(:first, :conditions => ["order_id = 2"])
      lambda {
        Accounting.create_for_point_destroy(user_point)
      }.should change(Accounting, :count).by(0)
    end
  end
end
