# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe EmailForm do
  fixtures :user_emails, :users

  it "should clear validation check for format." do
    email_address = EmailForm.new(:email => 'ec-one_test@ec-one.com')
    email_address.should be_valid
  end

  it "should rise validation error when wrong format." do
    email_address = EmailForm.new(:email => 'ec-one')
    email_address.should_not be_valid
    email_address.errors.on(:email).should == "は正しくありません。"
    email_address = EmailForm.new(:email => 'ec-one@')
    email_address.should_not be_valid
    email_address.errors.on(:email).should == "は正しくありません。"
    email_address = EmailForm.new(:email => '@ec-one')
    email_address.should_not be_valid
    email_address.errors.on(:email).should == "は正しくありません。"
    email_address = EmailForm.new(:email => 'ec-one@ec-one')
    email_address.should_not be_valid
    email_address.errors.on(:email).should == "は正しくありません。"
  end

  describe 'is_unique_email?' do
    it "should clear validation check for duplicate.　本番データ" do
      email_address = EmailForm.new(:email => 'ec-one-ec-one@ec-one.com', :user_id => 1002, :test_flg => nil)
      email_address.should be_valid
    end

    it "should rise validation error when duplicate.　本番データ" do
      email_address = EmailForm.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001, :test_flg => nil)
      email_address.should_not be_valid
      email_address.errors.on(:email).should == "は、他のお客様が既にご使用されています。"
    end

    it "should be valid.　テストデータ" do
      email_address = EmailForm.new(:email => 'ec-one-ec-one@ec-one.com', :user_id => 1002, :test_flg => true)
      email_address.should be_valid
    end

    it "should be invalid.　テストデータ" do
      User.update(1002, :test_flg => true)
      email_address = EmailForm.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001, :test_flg => true)
      email_address.should_not be_valid
      email_address.errors.on(:email).should == "は、他のお客様が既にご使用されています。"
    end

    it "should be valid.　本番データとテストデータの重複は許可" do
      email_address = EmailForm.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001, :test_flg => true)
      email_address.should be_valid

      User.update(1002, :test_flg => true)
      email_address = EmailForm.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001, :test_flg => nil)
      email_address.should be_valid
    end
  end

  it "#228 email_validate_flg check" do
    email_address = EmailForm.new(:email => 'ec-one-ec-one@ec-one.com', :user_id => 1002)
    email_address.email_validate_flg = true
    email_address.should be_valid
    email_address = EmailForm.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001)
    email_address.email_validate_flg = false
    email_address.should_not be_valid
  end

end
