# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe PaymentErrorHistory do
  fixtures :gmo_transaction_histories, :sps_result_histories, :sps_parameter_histories, :orders
  
  describe "selectable_attr" do
    it "status_div" do
      PaymentErrorHistory.status_div_options.should == [
        ["対応必要なし", '0'], 
        ["未対応", '1'], 
        ["返答待ち", '2'], 
        ["対応完了", '3']
      ]
      PaymentErrorHistory.status_div_id_by_key(:no_need).should == "0"
      PaymentErrorHistory.status_div_id_by_key(:not_yet).should == "1"
      PaymentErrorHistory.status_div_id_by_key(:waiting).should == "2"
      PaymentErrorHistory.status_div_id_by_key(:complete).should == "3"
    end

    it "exe_div" do
      PaymentErrorHistory.exe_div_options.should == [
        ["初回実行", '1'], 
        ["再実行", '2']
      ]
      PaymentErrorHistory.exe_div_id_by_key(:first_time).should == "1"
      PaymentErrorHistory.exe_div_id_by_key(:again).should == "2"
    end

    it "err_code" do
      p = PaymentErrorHistory.new(:err_code => 'NG')
      p.err_key.should == :ng
      p.err_name.should == 'OK,'
      
      p.err_code = 'ECR_NG_CHECK_SUM'
      p.err_key.should == :ng_check_sum
      p.err_name.should == 'NG,購入要求情報のチェックサムが不正です。'
      
      p.err_code = 'ECR_NG_REQUEST'
      p.err_key.should == :ng_request
      p.err_name.should == 'NG,購入要求情報の内容が不正です。'
      
      p.err_code = 'ECR_NG_OUT_OF_STOCK'
      p.err_key.should == :ng_out_of_stock
      p.err_name.should == 'NG,在庫不足です。'
      
      p.err_code = 'ECR_NG_OTHER_EXCEPTION'
      p.err_key.should == :ng_other_exception
      p.err_name.should == 'NG,予期せぬエラーです。'
    end
  end
  
  describe "polymophic association" do
    it "data count" do
      PaymentErrorHistory.count().should == 8
      
      o = GmoTransactionHistory.create(
        :order_id => 8, :gmo_order_id => '11111', :approve => '11111', :tran_id => '012345', :tran_date => '20090101100000',
        :check_string => 'test_check_string', :err_code => 'E01', :err_info => 'E0101001', :exe_div => '1')
      
      PaymentErrorHistory.count().should == 9
      
      sph = SpsParameterHistory.create(
        :order_id => 9, :pay_method => 'test_method', :merchant_id => '111', :service_id => '111', :cust_code => '111', :sps_order_id => '111',
        :item_id => '111', :amount => '12500', :pay_type => '1', :service_type => '1', :request_date => '20090101100000', :payment_state_div => '1')
      srh = SpsResultHistory.create(
        :sps_parameter_history_id => sph.id, :pay_method => 'test_method', :merchant_id => '111', :service_id => '111', :cust_code => '111',
        :sps_order_id => '111', :item_id => '111', :amount => '12500', :pay_type => '1', :service_type => '1', :request_date => '20090101100000', 
        :res_pay_method => '1', :res_result => 'NG', :ecr_result => 'NG', :res_tracking_id => '12345', :res_date => '20090101110000')

      PaymentErrorHistory.count().should == 10
    end
    
    it "update GmoTransactionHistory" do
      p = PaymentErrorHistory.find(:first, :conditions => {:payment_system_div => '1'})
      p.class.should == PaymentErrorHistory
      p.updatable.class.should == GmoTransactionHistory
      p.status_div.should == '0'
      
      p.updatable.status_div = '2'
      p.updatable.save
      
      p = PaymentErrorHistory.find(:first, :conditions => {:updatable_id => p.updatable_id, :updatable_type => p.updatable_type})
      p.status_div.should == '2'
    end
    
    it "update SpsResultHistory" do
      p = PaymentErrorHistory.find(:first, :conditions => {:payment_system_div => '2'})
      p.class.should == PaymentErrorHistory
      p.updatable.class.should == SpsResultHistory
      p.status_div.should == '0'
      
      p.updatable.status_div = '2'
      p.updatable.save
      
      p = PaymentErrorHistory.find(:first, :conditions => {:updatable_id => p.updatable_id, :updatable_type => p.updatable_type})
      p.status_div.should == '2'
    end
  end
  describe "検索条件テスト" do
    it "本番決済のみ" do
      peh = PaymentErrorHistory.find(:first)
      peh.updatable.test_flg = true
      peh.updatable.save!
      form = PaymentErrorHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:production))
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 7
    end

    it "テスト決済のみ" do
      peh = PaymentErrorHistory.find(:first)
      peh.updatable.test_flg = true
      peh.updatable.save!
      form = PaymentErrorHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:test))
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 1
    end

    it "本番決済とテスト決済" do
      peh = PaymentErrorHistory.find(:first)
      peh.updatable.test_flg = true
      peh.updatable.save!
      form = PaymentErrorHistory::SearchConditionForm.new
      result = PaymentErrorHistory.find(:all, :conditions => PaymentErrorHistory.create_condition(form))
      result.length.should == 8
    end
  end
end
