# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../spec_helper'

describe Product::CategorySearchForm do
  fixtures :products, :product_displays, :product_categories, :product_category_displays, :product_prices,
    :kbn_codes, :kbn_code_names, :product_skus, :user_ranks,
    :product_variation_displays, :product_variation_item_displays, :product_variation_items, :product_variations, :cms_publishes

  before(:each) do
    @now = Time.local(2009, 1, 10, 12, 00, 00)
    Time.stub!(:now).and_return(@now)
  end

  describe "カテゴリ検索クエリ生成" do
    it "全カテゴリ選択の場合 ec_front" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and (present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", 'ja', @now, @now]
      }
    end
    
    it "全カテゴリ選択の場合 ec_admin 公開期間を見ないこと" do
      EcRider.config.stub!(:ec_admin).and_return(true)
      EcRider.config.stub!(:ec_test).and_return(false)
      I18n.stub!(:locale).and_return('ja')
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 " <<
            " and (present_flg is null or present_flg = 0)",
          'ja']
      }
    end
    
    it "全カテゴリ選択の場合 ec_test" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(true)
      I18n.stub!(:locale).and_return('ja')
      test_now = Time.local(2009,9,05, 11,34,57)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "", :condition_div => '0')
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and (present_flg is null or present_flg = 0)",
          'ja', test_now, test_now]
      }
    end

    it "親カテゴリ選択の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code14", :keyword => "", :condition_div => '0')
      serch_cond = [16, 18, 19, 20, 21, 22, 23, 14]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', @now, @now]
      }
    end

    it "子カテゴリ選択の場合(さらに子カテゴリが存在する場合)" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code20", :keyword => "", :condition_div => '0')
      serch_cond = [16,18,19,20]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) "<<
            "OR products.category_5_id IN (?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', @now, @now]
      }
    end

    it "子カテゴリ選択の場合(一番下の子カテゴリの場合)" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond, 'ja', @now, @now]
      }
    end
  end
  
  describe "キーワード検索クエリ生成" do
    it "キーワードをひとつだけ入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "shirt", :condition_div => '0')
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions => ["((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now]
      }
    end
    
    it "キーワードをひとつだけ入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "", :keyword => "shirt", :condition_div => '1')
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions=>["((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now]
      }
    end

    it "カテゴリを選択してキーワードをひとつだけ入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order=>"products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now]
      }
    end
    
    it "カテゴリを選択してキーワードをひとつだけ入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order => "products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now]
      }
    end
    
    it "カテゴリを選択してキーワードを複数入力した場合（すべて）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt pants shoes", :condition_div => '0')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :order=>"products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) AND " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) AND " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%",
            "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%",
            "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%",
            'ja', @now, @now]
      }
    end
    
    it "カテゴリを選択してキーワードを複数入力した場合（いづれか）" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt pants shoes", :condition_div => '1')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins => ["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id"],
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) OR " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?) OR " <<
            "(product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%",
            "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%", "%pants%",
            "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%", "%shoes%",
            'ja', @now, @now],
        :order => "products.display_order is null, products.display_order, products.product_code",

      }
    end
  end
  
  describe "並び替え" do
    it "並び替えの条件が金額の安い順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '1', :user_rank_id => '1')#'product_prices.standard_price, products.id')
      serch_cond = [16]
      form.to_find_options.should == {
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now],
        :joins=>["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id",
          "left outer join product_skus cond_product_skus on cond_product_skus.product_id = products.id",
            "left outer join product_prices cond_product_prices on cond_product_prices.product_sku_id = cond_product_skus.id and cond_product_prices.user_rank_id = 1"],
        :order => "cond_product_prices.standard_price, products.display_order is null, products.display_order, products.product_code"
      }
    end
    
    it "並び替えの条件が金額の高い順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '2', :user_rank_id => '2')#'product_prices.standard_price desc, products.id')
      serch_cond = [16]
      form.to_find_options.should == {
       :joins=>["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id",
          "left outer join product_skus cond_product_skus on cond_product_skus.product_id = products.id",
            "left outer join product_prices cond_product_prices on cond_product_prices.product_sku_id = cond_product_skus.id and cond_product_prices.user_rank_id = 2"],
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now],
          :order => "cond_product_prices.standard_price desc, products.display_order is null, products.display_order, products.product_code"
       }
    end
    
    it "並び替えの条件が新着順の場合" do
      EcRider.config.stub!(:ec_admin).and_return(false)
      EcRider.config.stub!(:ec_test).and_return(false)
      form = Product::CategorySearchForm.new(:category_code => "code16", :keyword => "shirt", :condition_div => '1', :order => '3')
      serch_cond = [16]
      form.to_find_options.should == {
        :joins=>["inner join product_displays on product_displays.product_id = products.id", "inner join product_skus on product_skus.product_id = products.id",
          "left outer join product_skus cond_product_skus on cond_product_skus.product_id = products.id",
            "left outer join product_prices cond_product_prices on cond_product_prices.product_sku_id = cond_product_skus.id and cond_product_prices.user_rank_id = 1"],
        :order => "products.publish_started_at desc, products.display_order is null, products.display_order, products.product_code",
        :conditions=>["(products.category_1_id IN (?) OR products.category_2_id IN (?) OR products.category_3_id IN (?) OR products.category_4_id IN (?) OR "<<
            "products.category_5_id IN (?)) AND "<<
            "((product_displays.product_name_1 like ? OR product_displays.product_name_2 like ? OR product_displays.product_outline like ? OR "<<
            "product_displays.product_explanation like ? OR product_displays.recommendation like ? OR product_displays.recommendation_2 like ? OR "<<
            "product_displays.product_keyword like ?)) AND " <<
            "products.deleted_at is null AND "<<
            "product_skus.deleted_at is null AND "<<
            "product_displays.locale = ? AND "<<
            "product_skus.primary_sku_flg = 1 AND "<<
            "((publish_started_at is null or publish_started_at <= ?) and (publish_finished_at is null or publish_finished_at >= ?)) and " <<
            "(present_flg is null or present_flg = 0) and " <<
            "(invisible_flg is null or invisible_flg = 0)", serch_cond, serch_cond, serch_cond, serch_cond, serch_cond,
            "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", "%shirt%", 'ja', @now, @now]
      }
    end
  end
end
