# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe SpsOrderTemporary do
  fixtures :sps_order_temporaries, :sps_order_shipping_temporaries, :sps_order_item_temporaries, :product_skus

  it "should create a new instance given valid attributes" do
    @valid_attributes = {
      :sps_order_id => 'sps_order_id'
    }
    SpsOrderTemporary.create!(@valid_attributes)
  end

  describe "test create_from_order" do
    it "should create" do
      sps_order_id = 'sps_order_id'

      # Orderインスタンス作成
      order_attributes = {
        :ec_order_code => 'ec_order_code',
        :user_id => 222,
        :ordered_at => '2007-12-10 11:09:38',
        :site_type_div => 't_div',
        :free_request => 'free_request',
        :gmo_credit_card_id => 333,
        :on_line_payment_exe_div => 'o_div',
        :locale => 'ja'
      }
      order = Order.new(order_attributes)

      # OrderShippingインスタンス作成
      order_shipping_attributes_1 = {
        :order_id => nil,
        :order_code => 'order_code1',
        :status_div => '1',
        :address_name => 'address_name1',
        :name_last => 'name_last1',
        :name_first => 'name_first1',
        :kana_name_last => 'kana_name_last1',
        :kana_name_first => 'kana_name_first1',
        :tel_num => 'tel_num1',
        :zip_code => 'zip_code1',
        :address_1 => 'address_1_1',
        :address_2 => 'address_2_1',
        :address_3 => 'address_3_1',
        :email => 'email1',
        :arrival_date => '2009-01-01',
        :arrival_hour => 'arri1',
        :payment_type_div => 'paym1',
        :total_amount => 1111,
        :delivery_div => 'deli1',
        :shipping_cost => 2221,
        :inquiry_number => 'inquiry_number1',
        :complete_sent_mail_flg => 1,
        :delivery_code => 'delivery_code1',
        :csv_output_div => '1',
        :delivery_content => 'delivery_content1',
        :country_code => 'co1'
      }
      order_shipping_1 = OrderShipping.new(order_shipping_attributes_1)

      order_shipping_attributes_2 = {
        :order_id => nil,
        :order_code => 'order_code2',
        :status_div => '2',
        :address_name => 'address_name2',
        :name_last => 'name_last2',
        :name_first => 'name_first2',
        :kana_name_last => 'kana_name_last2',
        :kana_name_first => 'kana_name_first2',
        :tel_num => 'tel_num2',
        :zip_code => 'zip_code2',
        :address_1 => 'address_1_2',
        :address_2 => 'address_2_2',
        :address_3 => 'address_3_2',
        :email => 'email2',
        :arrival_date => '2009-02-02',
        :arrival_hour => 'arri2',
        :payment_type_div => 'paym2',
        :total_amount => 1112,
        :delivery_div => 'deli2',
        :shipping_cost => 2222,
        :inquiry_number => 'inquiry_number2',
        :complete_sent_mail_flg => 2,
        :delivery_code => 'delivery_code2',
        :csv_output_div => '2',
        :delivery_content => 'delivery_content2',
        :country_code => 'co2'
      }
      order_shipping_2 = OrderShipping.new(order_shipping_attributes_2)

      # Orderインスタンス作成
      order_items_attributes_1 = {
        :order_shipping_id => nil,
        :product_name => 'product_name1',
        :order_product_name_1 => 'order_product_name_11',
        :order_product_name_2 => 'order_product_name_21',
        :course_div => 'cour1',
        :unit_price => 1111,
        :shipping_cost => 2221,
        :total_amount => 3331,
        :qty => 4441,
        :product_sku_id => "266"
      }
      order_item_1 = OrderItem.new(order_items_attributes_1)

      order_items_attributes_2 = {
        :order_shipping_id => nil,
        :product_name => 'product_name2',
        :order_product_name_1 => 'order_product_name_12',
        :order_product_name_2 => 'order_product_name_22',
        :course_div => 'cour2',
        :unit_price => 1112,
        :shipping_cost => 2222,
        :total_amount => 3332,
        :qty => 4442,
        :product_sku_id => "266"
      }
      order_item_2 = OrderItem.new(order_items_attributes_2)

      order_items_attributes_3 = {
        :order_shipping_id => nil,
        :product_name => 'product_name3',
        :order_product_name_1 => 'order_product_name_13',
        :order_product_name_2 => 'order_product_name_23',
        :course_div => 'cour3',
        :unit_price => 1113,
        :shipping_cost => 2223,
        :total_amount => 3333,
        :qty => 4443,
        :product_sku_id => "266"
      }
      order_item_3 = OrderItem.new(order_items_attributes_3)

      order_items_attributes_4 = {
        :order_shipping_id => nil,
        :product_name => 'product_name4',
        :order_product_name_1 => 'order_product_name_14',
        :order_product_name_2 => 'order_product_name_24',
        :course_div => 'cour4',
        :unit_price => 1114,
        :shipping_cost => 2224,
        :total_amount => 3334,
        :qty => 4444,
        :product_sku_id => "266"
      }
      order_item_4 = OrderItem.new(order_items_attributes_4)

      # 関連を生成
      order_shipping_1.order_items << order_item_1
      order_shipping_1.order_items << order_item_2
      order_shipping_2.order_items << order_item_3
      order_shipping_2.order_items << order_item_4
      order.order_shippings << order_shipping_1
      order.order_shippings << order_shipping_2

      # テスト対象のメソッドを実行
      sps_order_temporary = SpsOrderTemporary.create_from_order(order, sps_order_id)

      # 属性をチェック
      sps_order_temporary.id.should_not be_nil
      sps_order_temporary.sps_order_id.should == 'sps_order_id'
      sps_order_temporary.user_id.should == 222
      sps_order_temporary.ordered_at.strftime('%Y-%m-%d %H:%M:%S').should == '2007-12-10 11:09:38'
      sps_order_temporary.site_type_div.should == 't_div'
      sps_order_temporary.free_request.should == 'free_request'
      sps_order_temporary.gmo_credit_card_id.should == 333
      sps_order_temporary.on_line_payment_exe_div.should == 'o_div'
      sps_order_temporary.locale.should == 'ja'

      sps_order_shipping_temporary_1 = sps_order_temporary.sps_order_shipping_temporaries[0]
      sps_order_shipping_temporary_1.id.should_not be_nil
      sps_order_shipping_temporary_1.sps_order_temporary_id.should == sps_order_shipping_temporary_1.sps_order_temporary.id
      sps_order_shipping_temporary_1.status_div.should == '1'
      sps_order_shipping_temporary_1.address_name.should == 'address_name1'
      sps_order_shipping_temporary_1.name_last.should == 'name_last1'
      sps_order_shipping_temporary_1.name_first.should == 'name_first1'
      sps_order_shipping_temporary_1.kana_name_last.should == 'kana_name_last1'
      sps_order_shipping_temporary_1.kana_name_first.should == 'kana_name_first1'
      sps_order_shipping_temporary_1.tel_num.should == 'tel_num1'
      sps_order_shipping_temporary_1.zip_code.should == 'zip_code1'
      sps_order_shipping_temporary_1.address_1.should == 'address_1_1'
      sps_order_shipping_temporary_1.address_2.should == 'address_2_1'
      sps_order_shipping_temporary_1.address_3.should == 'address_3_1'
      sps_order_shipping_temporary_1.email.should == 'email1'
      sps_order_shipping_temporary_1.arrival_date.strftime('%Y-%m-%d %H:%M:%S').should == '2009-01-01 00:00:00'
      sps_order_shipping_temporary_1.arrival_hour.should == 'arri1'
      sps_order_shipping_temporary_1.payment_type_div.should == 'paym1'
      sps_order_shipping_temporary_1.total_amount.should == 1111
      sps_order_shipping_temporary_1.delivery_div.should == 'deli1'
      sps_order_shipping_temporary_1.shipping_cost.should == 2221
      sps_order_shipping_temporary_1.inquiry_number.should == 'inquiry_number1'
      sps_order_shipping_temporary_1.complete_sent_mail_flg.should == 1
      sps_order_shipping_temporary_1.delivery_code.should == 'delivery_code1'
      sps_order_shipping_temporary_1.csv_output_div.should == '1'
      sps_order_shipping_temporary_1.delivery_content.should == 'delivery_content1'
      sps_order_shipping_temporary_1.country_code.should == 'co1'

      sps_order_shipping_temporary_2 = sps_order_temporary.sps_order_shipping_temporaries[1]
      sps_order_shipping_temporary_2.id.should_not be_nil
      sps_order_shipping_temporary_2.sps_order_temporary_id.should == sps_order_shipping_temporary_2.sps_order_temporary.id
      sps_order_shipping_temporary_2.status_div.should == '2'
      sps_order_shipping_temporary_2.address_name.should == 'address_name2'
      sps_order_shipping_temporary_2.name_last.should == 'name_last2'
      sps_order_shipping_temporary_2.name_first.should == 'name_first2'
      sps_order_shipping_temporary_2.kana_name_last.should == 'kana_name_last2'
      sps_order_shipping_temporary_2.kana_name_first.should == 'kana_name_first2'
      sps_order_shipping_temporary_2.tel_num.should == 'tel_num2'
      sps_order_shipping_temporary_2.zip_code.should == 'zip_code2'
      sps_order_shipping_temporary_2.address_1.should == 'address_1_2'
      sps_order_shipping_temporary_2.address_2.should == 'address_2_2'
      sps_order_shipping_temporary_2.address_3.should == 'address_3_2'
      sps_order_shipping_temporary_2.email.should == 'email2'
      sps_order_shipping_temporary_2.arrival_date.strftime('%Y-%m-%d %H:%M:%S').should == '2009-02-02 00:00:00'
      sps_order_shipping_temporary_2.arrival_hour.should == 'arri2'
      sps_order_shipping_temporary_2.payment_type_div.should == 'paym2'
      sps_order_shipping_temporary_2.total_amount.should == 1112
      sps_order_shipping_temporary_2.delivery_div.should == 'deli2'
      sps_order_shipping_temporary_2.shipping_cost.should == 2222
      sps_order_shipping_temporary_2.inquiry_number.should == 'inquiry_number2'
      sps_order_shipping_temporary_2.complete_sent_mail_flg.should == 2
      sps_order_shipping_temporary_2.delivery_code.should == 'delivery_code2'
      sps_order_shipping_temporary_2.csv_output_div.should == '2'
      sps_order_shipping_temporary_2.delivery_content.should == 'delivery_content2'
      sps_order_shipping_temporary_2.country_code.should == 'co2'

      sps_order_item_temporary_1 = sps_order_shipping_temporary_1.sps_order_item_temporaries[0]
      sps_order_item_temporary_1.id.should_not be_nil
      sps_order_item_temporary_1.sps_order_shipping_temporary_id.should == sps_order_item_temporary_1.sps_order_shipping_temporary.id
#      sps_order_item_temporary_1.product_code.should == '1111'
      sps_order_item_temporary_1.product_name.should == 'product_name1'
#      sps_order_item_temporary_1.product_name_2.should == 'product_name_21'
      sps_order_item_temporary_1.order_product_name_1.should == 'order_product_name_11'
      sps_order_item_temporary_1.order_product_name_2.should == 'order_product_name_21'
#      sps_order_item_temporary_1.kana_product_name.should == 'kana_product_name1'
#      sps_order_item_temporary_1.category_code_1.should == 'cate11'
#      sps_order_item_temporary_1.category_code_2.should == 'cate21'
#      sps_order_item_temporary_1.category_code_3.should == 'cate31'
      sps_order_item_temporary_1.course_div.should == 'cour1'
#      sps_order_item_temporary_1.gift_div.should == 'g1'
      sps_order_item_temporary_1.unit_price.should == 1111
      sps_order_item_temporary_1.shipping_cost.should == 2221
      sps_order_item_temporary_1.total_amount.should == 3331
      sps_order_item_temporary_1.qty.should == 4441
#      sps_order_item_temporary_1.made_num.should == 11

      sps_order_item_temporary_2 = sps_order_shipping_temporary_1.sps_order_item_temporaries[1]
      sps_order_item_temporary_2.id.should_not be_nil
      sps_order_item_temporary_2.sps_order_shipping_temporary_id.should == sps_order_item_temporary_2.sps_order_shipping_temporary.id
#      sps_order_item_temporary_2.product_code.should == '1112'
      sps_order_item_temporary_2.product_name.should == 'product_name2'
#      sps_order_item_temporary_2.product_name_2.should == 'product_name_22'
      sps_order_item_temporary_2.order_product_name_1.should == 'order_product_name_12'
      sps_order_item_temporary_2.order_product_name_2.should == 'order_product_name_22'
#      sps_order_item_temporary_2.kana_product_name.should == 'kana_product_name2'
#      sps_order_item_temporary_2.category_code_1.should == 'cate12'
#      sps_order_item_temporary_2.category_code_2.should == 'cate22'
#      sps_order_item_temporary_2.category_code_3.should == 'cate32'
      sps_order_item_temporary_2.course_div.should == 'cour2'
#      sps_order_item_temporary_2.gift_div.should == 'g2'
      sps_order_item_temporary_2.unit_price.should == 1112
      sps_order_item_temporary_2.shipping_cost.should == 2222
      sps_order_item_temporary_2.total_amount.should == 3332
      sps_order_item_temporary_2.qty.should == 4442
#      sps_order_item_temporary_2.made_num.should == 12

      sps_order_item_temporary_3 = sps_order_shipping_temporary_2.sps_order_item_temporaries[0]
      sps_order_item_temporary_3.id.should_not be_nil
      sps_order_item_temporary_3.sps_order_shipping_temporary_id.should == sps_order_item_temporary_3.sps_order_shipping_temporary.id
#      sps_order_item_temporary_3.product_code.should == '1113'
      sps_order_item_temporary_3.product_name.should == 'product_name3'
#      sps_order_item_temporary_3.product_name_2.should == 'product_name_23'
      sps_order_item_temporary_3.order_product_name_1.should == 'order_product_name_13'
      sps_order_item_temporary_3.order_product_name_2.should == 'order_product_name_23'
#      sps_order_item_temporary_3.kana_product_name.should == 'kana_product_name3'
#      sps_order_item_temporary_3.category_code_1.should == 'cate13'
#      sps_order_item_temporary_3.category_code_2.should == 'cate23'
#      sps_order_item_temporary_3.category_code_3.should == 'cate33'
      sps_order_item_temporary_3.course_div.should == 'cour3'
#      sps_order_item_temporary_3.gift_div.should == 'g3'
      sps_order_item_temporary_3.unit_price.should == 1113
      sps_order_item_temporary_3.shipping_cost.should == 2223
      sps_order_item_temporary_3.total_amount.should == 3333
      sps_order_item_temporary_3.qty.should == 4443
#      sps_order_item_temporary_3.made_num.should == 13

      sps_order_item_temporary_4 = sps_order_shipping_temporary_2.sps_order_item_temporaries[1]
      sps_order_item_temporary_4.id.should_not be_nil
      sps_order_item_temporary_4.sps_order_shipping_temporary_id.should == sps_order_item_temporary_4.sps_order_shipping_temporary.id
#      sps_order_item_temporary_4.product_code.should == '1114'
      sps_order_item_temporary_4.product_name.should == 'product_name4'
#      sps_order_item_temporary_4.product_name_2.should == 'product_name_24'
      sps_order_item_temporary_4.order_product_name_1.should == 'order_product_name_14'
      sps_order_item_temporary_4.order_product_name_2.should == 'order_product_name_24'
#      sps_order_item_temporary_4.kana_product_name.should == 'kana_product_name4'
#      sps_order_item_temporary_4.category_code_1.should == 'cate14'
#      sps_order_item_temporary_4.category_code_2.should == 'cate24'
#      sps_order_item_temporary_4.category_code_3.should == 'cate34'
      sps_order_item_temporary_4.course_div.should == 'cour4'
#      sps_order_item_temporary_4.gift_div.should == 'g4'
      sps_order_item_temporary_4.unit_price.should == 1114
      sps_order_item_temporary_4.shipping_cost.should == 2224
      sps_order_item_temporary_4.total_amount.should == 3334
      sps_order_item_temporary_4.qty.should == 4444
#      sps_order_item_temporary_4.made_num.should == 14
    end
  end

  describe "テスト注文、本番注文切り替え" do
    it "ec_adminの場合　本番データ" do
      sps_order_temporary = SpsOrderTemporary.new
      sps_order_temporary.sps_order_id = 1
      sps_order_temporary.save!
      sps_order_temporary.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      sps_order_temporary = SpsOrderTemporary.new(:test_flg => true)
      sps_order_temporary.sps_order_id = 1
      sps_order_temporary.save!
      sps_order_temporary.test_flg.should be_true
    end

    it "ec_frontの場合" do
      sps_order_temporary = SpsOrderTemporary.new
      sps_order_temporary.sps_order_id = 1
      sps_order_temporary.save!
      sps_order_temporary.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_order_temporary = SpsOrderTemporary.new
      sps_order_temporary.sps_order_id = 1
      sps_order_temporary.save!
      sps_order_temporary.test_flg.should be_true
    end

    it "ec_testの場合　更新" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_order_temporary = SpsOrderTemporary.new(:test_flg => true)
      sps_order_temporary.sps_order_id = 1
      sps_order_temporary.save!
      sps_order_temporary.test_flg.should be_true
    end
  end
end
