# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe UserEmail do
  fixtures :user_emails, :users

  it "should return account." do
    UserEmail.find(1).account.should == 'email_address_001'
  end

  it "should return career." do
    UserEmail.find(1).career.should == '@ec-one.com'
  end

  describe 'is_unique_email?' do
    it "ec_frontの場合　重複あり　本番会員" do
      user_email = UserEmail.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001)
      user_email.should_not be_valid
      user_email.errors.on(:email).should == "は、他のお客様が既にご使用されています。"
    end

    it "ec_testの場合　重複あり　テスト会員" do
      EcRider.config.stub!(:ec_test).and_return(true)
      User.find(users(:users_00003).id).save
      User.find(users(:users_00003).id).test_flg.should be_true
      user_email = UserEmail.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001)
      user_email.should_not be_valid
      user_email.errors.on(:email).should == "は、他のお客様が既にご使用されています。"
    end

    it "ec_testの場合　本番会員とテスト会員の重複は許可" do
      EcRider.config.stub!(:ec_test).and_return(true)
      user_email = UserEmail.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001)
      user_email.should be_valid
    end

    it "非会員の場合の重複は許可" do
      user_email = UserEmail.new(:email => 'email_address_01002@ec-one.com', :user_id => 1001)
      User.update(1001, :member_div_key => :no_membership)
      user_email.should be_valid
    end

    it "退会会員の場合の重複は許可" do
      user_email = UserEmail.new(:email => 'ec-one-ec-one@ec-one.com', :user_id => 1002)
      user_email.should be_valid
    end
  end

  it "should return email address by user_id." do
    UserEmail.find_by_user_id(1002).should == UserEmail.find_by_email('email_address_01002@ec-one.com') # user_emails(:user_emails_01002)
  end

  it "should rise argument error." do
    lambda {UserEmail.find_by_user_id(nil)}.should raise_error(ArgumentError)
  end

  it "should set pc_mobile_div pc." do
    user_email = UserEmail.new(:email => 'ec-1@ec-one.com', :user_id => 1002)
    user_email.save
    user_email.mail_address_pc_mobile_div.should == '1'
  end

  it "should set pc_mobile_div mobile." do
    user_email = UserEmail.new(:email => 'ec-1@docomo.ne.jp', :user_id => 1002)
    user_email.save
    user_email.mail_address_pc_mobile_div.should == '2'
  end

  it "check_by_emall method check." do
    UserEmail.check_by_emall("email_address_01002@ec-one.com").should == '1'
    UserEmail.check_by_emall("ec-one@docomo.ne.jp").should == '2'
  end

  describe "equals to EmailForm(deleted model)'s validation" do

  it "should clear validation check for format." do
    user_email = UserEmail.new(:email => 'ec-one_test@ec-one.com')
    user_email.should be_valid
  end

  it "should rise validation error when wrong format." do
    user_email = UserEmail.new(:email => 'ec-one')
    user_email.should_not be_valid
    user_email.errors.on(:email).should == "は正しくありません。"
    user_email = UserEmail.new(:email => 'ec-one@')
    user_email.should_not be_valid
    user_email.errors.on(:email).should == "は正しくありません。"
    user_email = UserEmail.new(:email => '@ec-one')
    user_email.should_not be_valid
    user_email.errors.on(:email).should == "は正しくありません。"
    user_email = UserEmail.new(:email => 'ec-one@ec-one')
    user_email.should_not be_valid
    user_email.errors.on(:email).should == "は正しくありません。"
  end
  end
end
