# -*- coding: utf-8 -*-
class AccountController < PcController

  before_filter :authorize, :only => [:change_information, :confirm_update_information, :update_information,
  :update_resign, :logout, :change_password, :addMailAddress, :change_security_answer,
  :update_security_answer, :update_resign, :resign,:look_information]

  # 会員専用ページ
  def index
    if logged_in?
      redirect_to :controller => :member, :action => :index
    else
      redirect_to :action => :login
    end
  end

  # 個人情報保護方針
  def policy
    if logged_in?
      unless session[:forward_url].nil?
        redirect_to session.delete(:forward_url)
      else
        redirect_to :controller => :member, :action=> :index
      end
    else
      session[:signup] = nil
      render :action => :policy_information
    end
  end

  #
  # 個人情報保護方針（会員種別コードチェック）
  #
  def entry_signup
    if logged_in?
      redirect_to :controller => :member, :action=> :index
      return
    end

    @back_to = EcRider.config.ec_front_url
    @member_category_div = params[:member_category_code]
    @member_category_name = User.member_category_div_name_by_id(@member_category_div)

    # 受け取ったコードが正しいかどうかをチェック
    member_category_key = User.member_category_div_key_by_id(@member_category_div)
    if !member_category_key || member_category_key == :general
      @member_category_error = "URL #{I18n.t("activerecord.errors.messages.EM990007")}"
    end

    render :action => :policy_information
  end

  #
  # 会員登録
  #
  def signup
    if logged_in?
      if !session[:forward_url].nil?
        redirect_to session.delete(:forward_url)
      else
        redirect_to :controller => 'member', :action => 'index'
      end
    end

    # バリデーションエラーで戻った場合
    if flash[:user_profile]
      @user_profile = flash[:user_profile]
      @user_address = flash[:user_address]
      @user_email = flash[:user_email]
      @user_en_address = flash[:user_en_address]
      return
    end
    # 戻るボタンで戻った場合
    if session[:signup]
      @user_profile = session[:signup]
      @user_email = @user_profile.user_email
      @user_address = @user_profile.user_address
      @user_en_address = @user_profile.sub_address
    # 初期表示
    else
      member_category_div = request.post? ? params[:member_category_div] : nil
      @user_profile = User.new(:sex_key => :male, :dm_send_div_key => :prefer, :member_category_div => member_category_div)
      @user_email = UserEmail.new
      @user_address = UserAddress.new(:country_code => ShopSetting.get_setting(:default_address_country, I18n.locale))
      @user_en_address = UserAddress.new
    end
    @user_profile.not_decided_address = true
  end

  # 会員登録確認画面
  def confirm_signup
    return redirect_to(:action => 'signup') unless request.post?

    @user_email = UserEmail.new(params[:user_email])
    @user_email.pc_mobile_div = UserEmail.check_by_emall(@user_email.email)

    @user_address = UserAddress.new(params[:user_address])
    @user_address.locale = I18n.locale.to_s
    @user_address.address_div_key = :user

    if @user_en_address = UserAddress.hash_to_sub_address(params[:user_en_address])
      @user_en_address.set_sub_address_from_address(@user_address)
    end

    @user_profile = User.new(params[:user_profile])
    @user_profile.member_div_key = :membership
    @user_profile.regist_div_key = :pc
    @user_profile.user_email = @user_email
    @user_profile.user_address = @user_address
    @user_profile.user_addresses << @user_en_address if @user_en_address
    @user_profile.mirror_from_address(@user_address)
    @user_profile.pc_mail_magazine_send_div = User.pc_mail_magazine_send_div_id_by_key(:not_prefer) if @user_profile.pc_mail_magazine_send_div.nil?
    @user_profile.login = @user_address.fullname
    @user_profile.remember_question_check = true

    @user_email.user = @user_profile # ←メールアドレス重複チェック内でuser_email.userを取得するため必要な処理です。.
    @user_profile.valid?
    @user_address.valid?
    @user_email.valid?
    @user_en_address.valid? if @user_en_address
    error_count = @user_profile.errors.length + @user_address.errors.length + @user_email.errors.length
    error_count += @user_en_address.errors.length if @user_en_address
    if error_count > 0
      @user_profile.not_decided_address = true
      flash[:user_profile] = @user_profile
      flash[:user_address] = @user_address
      flash[:user_email] = @user_email
      flash[:user_en_address] = @user_en_address
      redirect_back_or_default :action => :signup
    end
    session[:signup] = @user_profile
  end

  # 登録
  def create_user
    return redirect_to(:action => 'signup') unless request.post?
    @user =  session[:signup]
    if logged_in?
      return redirect_to(:controller => :member, :action => :index, :id => @user.member_category_div)
    end
    User.transaction do
      @user.save!

      @user.user_address.address_div_key = :user
      @user.user_address.locale = I18n.locale.to_s
      @user.user_address.save!

      self.current_user = @user
      # 会員登録完了メール送信
      FrontSiteMailer.send_signup_mail(@user.id.to_s, @user.user_email.email)
      unless session[:forward_url].nil?
        return redirect_to(session.delete(:forward_url))
      else
        return redirect_to(:controller => :member, :action => :index, :id => @user.member_category_div)
      end
    end
    session[:signup] = nil
  end

  #
  # ログイン画面表示
  #
  def login
    if logged_in?
      unless session[:forward_url].nil?
        redirect_to session.delete(:forward_url)
      else
        return redirect_to(:controller => :member, :action=> :index)
      end
    end
    @user_login_form = flash[:user_login_form] || UserLoginForm.new
    @login_error_notice = flash[:login_error_notice] if flash[:login_error_notice]
  end

  #
  # ログイン認証
  # params[:completion_url]がわたってきた場合は、ログイン成功後、ログインエラー時はその値の遷移先に遷移します。
  # params[:completion_url] = {:controller => 遷移させたいコントローラ名, :action => 遷移させたいアクション名 }
  #
  def authenticate
    return redirect_to(:action => 'login') unless request.post?
    user_login_form = UserLoginForm.new(params[:user_login_form])
    #ユーザロックのチェック
    user_locked = false
    email_address = UserEmail.email_with_user(user_login_form.email)
    if email_address && email_address.user && email_address.user.locked?
        user_locked = true
    end
    self.current_user = user_login_form.authenticate unless user_locked
    if logged_in?
      self.current_user.clear_login_challenge_count 
      reset_session_with_keep_content # セッションを作り直しています。中身は一緒
      if params[:remember_me] && params[:remember_me] == "1"
        @remember_me = params[:remember_me]
        self.current_user.remember_me
        cookies[:auth_token] = {:value => self.current_user.remember_token , :expires => self.current_user.remember_token_expires_at }
      end

      # 他の遷移先が指定されている場合はその遷移先に遷移させます。ない場合は遷移元に遷移します。
      if other_redirect?(params[:completion_url])
        redirect_to :controller => params[:completion_url][:controller], :action => params[:completion_url][:action]
      elsif  !session[:forward_url].nil?
        redirect_to session.delete(:forward_url)
      else
        redirect_to :controller => :member, :action => :index
      end
    else
      #ログイン失敗
      if !user_locked && email_address && email_address.user
        email_address.user.increment_login_challenge_count
        user_locked = email_address.user.locked? #インクリメントした後ロックされたかどうか取得
      end

      #flashに情報をセット(パスワードはクリア)
      flash[:user_login_form] = user_login_form

      if other_redirect?(params[:completion_url]) && user_locked
        # カートから遷移した場合、エラーメッセージが簡易のものとなるため別のメッセージをあてました
        flash[:login_error_notice] = i18n_message(:EM030004)
        redirect_back_or_default(:controller => params[:completion_url][:controller], :action => params[:completion_url][:action])
      elsif other_redirect?(params[:completion_url])
        # カートから遷移した場合、エラーメッセージが簡易のものとなるため別のメッセージをあてました
        flash[:login_error_notice] = i18n_message(:EM030001)
        redirect_back_or_default(:controller => params[:completion_url][:controller], :action => params[:completion_url][:action])
      elsif user_locked
        flash[:login_error_notice] = i18n_message(:EM030004)
        redirect_back_or_default(:controller => '/account', :action => 'login')
      else
        redirect_back_or_default(:controller => '/account', :action => 'login')
      end
    end
  end

  #
  # = 他の遷移先が指定されている場合、trueを返します。 =
  # ログイン認証で使用しています。
  private
  def other_redirect?(completion_url)
    unless completion_url.nil?
      unless (completion_url[:controller].nil? || completion_url[:action].nil?)
        unless (completion_url[:controller].empty? || completion_url[:action].empty?)
          return true
        end
      end
      return false
    end
  end

  public
  #
  # ログアウト
  #
  def logout
    self.current_user.forget_me if logged_in?
    cookies.delete :auth_token
    reset_session
    session[:user] = nil
    redirect_to_root
  end

  #
  # パスワード変更
  #
  def change_password
    @user = self.current_user
    @user = flash[:user] || self.current_user
  end

  #
  # パスワード変更更新
  #
  def update_password
    return redirect_to(:action => 'change_password') unless request.post?

    @user = self.current_user
    if @user.change_password(params[:user][:password], params[:user][:password_confirmation])
      redirect_to :action => :index
    else
      flash[:user] = @user
      redirect_to :action => :change_password
    end
  end

  #
  # 登録情報変更
  #
  def change_information
    session[:account] = nil
    session[:delete_en_address] = nil
    @user_profile = flash[:user_profile] || self.current_user
    @user_address = flash[:user_address] || @user_profile.user_address
    @user_email = flash[:user_email] || @user_profile.user_email
    @user_en_address = flash[:user_en_address] || @user_profile.sub_address
    #複数メールアドレスに未対応
    @user_profile.not_decided_address = true
  end

  #
  # 変更内容確認
  #
  def confirm_update_information
    return redirect_to(:action => 'change_information') unless request.post?

    session[:account] ||= {}

    @user_profile = self.current_user
    @user_profile.attributes = params[:user_profile]
    @user_profile.user_address.attributes = params[:user_address]
    @user_profile.user_email.attributes = params[:user_email]
    @user_profile.mirror_from_address(@user_profile.user_address)

    if @user_en_address = UserAddress.hash_to_sub_address(params[:user_en_address])
      @user_en_address.set_sub_address_from_address(@user_profile.user_address)
    end
    # 会員住所（海外配送用）を更新する
    if @user_profile.sub_address
      @user_profile.sub_address.attributes = params[:user_en_address]
      @user_profile.sub_address.set_sub_address_from_address(@user_profile.user_address)
      # 会員住所を削除する
      session[:delete_en_address] = true unless @user_en_address
    # 会員住所（海外配送用）を生成する
    elsif @user_en_address
      @user_profile.user_addresses << @user_en_address
    end

    @user_address = @user_profile.user_address
    @user_email = @user_profile.user_email
    @user_profile.valid?
    @user_address.valid?
    @user_email.valid?
    @user_en_address.valid? if @user_en_address
    error_count = @user_profile.errors.length + @user_address.errors.length + @user_email.errors.length
    error_count += @user_en_address.errors.length if @user_en_address
    if error_count > 0
      flash[:user_profile] = @user_profile
      flash[:user_address] = @user_address
      flash[:user_email] = @user_email
      flash[:user_en_address] = @user_en_address
      redirect_back_or_default :action => :change_information
    end

    session[:account][:user_profile] = @user_profile
    session[:account][:user_email] = @user_profile.user_email
    session[:account][:user_address] = @user_profile.user_address
    session[:account][:user_en_address] = @user_profile.sub_address
  end

  #
  # 登録情報変更確認
  #
  def update_information
    return redirect_to(:action => 'change_information') unless request.post?
    User.transaction do
      session[:account][:user_profile].save!
      session[:account][:user_email].save!
      session[:account][:user_address].save!
      if session[:delete_en_address]
        session[:account][:user_en_address].destroy
      elsif session[:account][:user_en_address]
        session[:account][:user_en_address].save!
      end
      session[:account] = nil
      session[:delete_en_address] = nil
    end

    self.current_user.reload
    redirect_to :controller => :member, :action => :index
  end

  #
  # 秘密の質問の変更
  #
  def change_security_answer
    @user = self.current_user
    @user = flash[:user] || self.current_user
    session[:change_security_answer] = nil
  end

  #
  # 秘密の質問の変更確認
  #
  def confirm_security_answer
    return redirect_to(:action => 'change_security_answer') unless request.post?

    @user = self.current_user.dup
    @user.remember_question = params[:user][:remember_question]
    @user.remember_answer = params[:user][:remember_answer]
    @user.remember_question_check = true
    unless  @user.valid?
      flash[:user] = @user
      redirect_back_or_default :action => :change_security_answer
      return
    end
    session[:change_security_answer] = params[:user]
  end

  #
  # 秘密の質問の更新
  #
  def update_security_answer
    return redirect_to(:action => 'change_security_answer') unless request.post?
    begin
      @user = User.find(self.current_user.id)
    rescue
      return redirect_to(:action => :change_security_answer, :lang => 'ja')
    end
    @user.attributes = session[:change_security_answer]
    if @user.save
      redirect_to :action => :index
    else
      redirect_to :action => :change_security_answer
    end
    current_user.reload
  end
end
