# -*- coding: utf-8 -*-
class CartController < PcController
  before_filter :cart_in?, :except => [:index, :cart_no_input] # [:index, :reminder, :confirm_reminder]
  before_filter :clear_user => [:cart_no_input]

  private

  #
  # カートをセッションから取得します。
  # カートがなかったら空のカートを生成してセッションに格納して返します。
  #  
  def cart_in_session(mode = nil)
    cart = session[:cart]
    unless cart
      case mode
      when :get_or_new then
        session[:cart] = cart = Cart.new
      end
    end
    cart
  end

  # 
  # セッションからユーザーを取得するメソッド
  # 
  def current_user
    unless @current_user
      # 残念ながら、実装の不手際で session[:user] にUserオブジェクトが入っている場合もある。
      user_or_user_id = session[:user]
      if user_or_user_id
        @current_user = user_or_user_id.is_a?(User) ? user_or_user_id : User.find(user_or_user_id.to_i)
      end
    end
    @current_user
  end

  public
  #
  # カート表示
  #
  def index
    session[:cart_tmp_user] = nil
    session[:shipping_addresses] = nil
    session[:credit_auth_error] = nil
    
    # ユーザログイン情報を取得
    @user_login_form = flash[:user_login_form] || UserLoginForm.new
    @login_error_notice = flash[:login_error_notice] || ''
    @stock_error = flash[:stock_error]
    
    cart = cart_in_session(:get_or_new)
    if params[:sku_code]
      cart.add(params[:sku_code], 
        Cart::Item.course_div_key_by_id(params[:course_div]), params[:qty].to_i)
      cart.calculate
      redirect_to :action => 'index'
    else
      cart.calculate
      render :action => 'cart_index'
    end
  end

  #
  # 商品削除
  #
  def remove
    # AJAXのリクエストであること
    return redirect_to_404_unless_performed unless request.xml_http_request?
    return redirect_to(:action => 'index') unless params[:id]

    cart = cart_in_session
    cart.remove_by_item_key(params[:id])
    cart.calculate

    if cart_in_session.empty?
      # %{...} だとrcovがちゃんと認識してくれない場合があるので、<<-EOS...EOSに。
      responseText = <<-EOS
        <%= render_partial_for_ajax_response("cart_list") %>
        <script type="text/javascript">
          show_or_hide_button();
        </script>
      EOS
      responseText = responseText.split(/$/).map(&:strip).join("\n")
    else
      responseText = generate_cart_updating_html_and_js_text
    end
    render(:inline => responseText)
  end

  #
  # カート再計算
  #
  def recalculate
    # AJAXのリクエストであること
    return redirect_to_404_unless_performed unless request.xml_http_request?
    product_code, qty = params[:id], params[:qty] 
    # 数量が指定されない場合は、再計算を実施しません。
    cart = cart_in_session
    if qty || cart.items[product_code]
      cart.change_qty(product_code, qty.to_i)
    end
    cart.calculate
    response_text = generate_cart_updating_html_and_js_text
    render(:inline => response_text)
  end

  #カートに商品がありませんエラー画面
  def cart_no_input
  end
  
  private

  def generate_cart_updating_html_and_js_text
    total_htmlet = render_partial_for_ajax_response("cart_total_prices").
      gsub(/\'/, '"').split(/$/).map(&:strip).join
    responseText = <<-"EOS"
      <%= render_partial_for_ajax_response("cart_list") %>
      <script type="text/javascript">
      //<![CDATA[
        $('cart_total_price').innerHTML='#{total_htmlet}';
      //]]>
      </script>
    EOS
    erase_render_results
    responseText.split(/$/).map(&:strip).join("\n")
  end


  # = ユーザー情報をクリアする。 =
  def clear_user
    session[:cart_tmp_user] = nil
    session[:cart_tmp_order] = nil
  end

end
