# -*- coding: utf-8 -*-
module EcRider
  module EcHelper
    module UserLoginFormTag
      extend MethodDescription

      desc <<-EOS
        [概要:ログイン用のフォームを出力します。]
      EOS
      def user_login_form(&block)
        raise UsageError, "ここでは使用できません。" unless @template.instance_variable_get(:@user_login_form)
        # このメソッドは、ec_adminからも実行されるので、jpmobileがインストールされていないかもしれません。
        # ですので、mobile?メソッドが存在するかどうかを確認する必要があります。
        req = @template.__send__(:request)
        url = (req.respond_to?(:mobile?) && req.mobile?) ?
          {:controller => 'mobile_sessions', :action => :create} :
          {:controller => 'account', :action => :authenticate}
        @template.form_for(:user_login_form, :url => url) do |f|
          user_login_form_backup = @user_login_form
          @user_login_form = f
          begin
            @template.concat(@template.capture(&block))
          ensure
            @user_login_form = user_login_form_backup
          end
        end
      end

      desc <<-EOS
        [概要:ログイン時のエラーメッセージがあればそれを表示します。ログイン用のフォーム内で使用してください。]
        [使用例:
          {{{
          <ec:user_login_form_error_messages/>
          　　==> <div class="alert_msg"><ul><li>パスワードが正しくありません</li></ul></div>
          }}}
        ]
      EOS
      def user_login_form_error_messages
        if @template.instance_variable_get(:@login_error_notice)
          content_tag(:div, content_tag(:ul, content_tag(:li, @template.instance_variable_get(:@login_error_notice))),:class=> 'alert_msg')
        else
          @template.template_error_messages_for(:user_login_form)
        end
      end

      desc <<-EOS
        [概要:ログイン用メールアドレスを入力するinputタグを出力します。ログイン用のフォーム内でのみ使用できます。]
      EOS
      def user_login_form_email(options = {})
        raise UsageError, "ec:user_login_form_emailはec:user_login_formの中で使用してください" unless @user_login_form
        options = {:maxLength => 50, :size => 35,
          :class => :default, :style => "ime-mode: inactive"}.update(options || {})
        @user_login_form.text_field(:email, options)
      end

      desc <<-EOS
        [概要:ログイン用パスワードを入力するinputタグを出力します。ログイン用のフォーム内でのみ使用できます。]
      EOS
      def user_login_form_password(options = {})
        raise UsageError, "ec:user_login_form_passwordはec:user_login_formの中で使用してください" unless @user_login_form
        options = {:maxLength => 20, :size => 50}.update(options || {})
        @user_login_form.password_field(:password, options)
      end

      desc <<-EOS
        [概要:「次回からの入力を省略」のチェックボックスを出力します。ログイン用のフォーム内で使用してください。]
      EOS
      def user_login_form_remember_me(options = {})
        @template.check_box_tag("remember_me", 1, true, options)
      end

    end
  end
end
