# -*- coding: utf-8 -*-
class Accounting < ActiveRecord::Base
  belongs_to :order
  
  attr_accessor :order_accounting

  selectable_attr :debt_account_div do
    entry '01', :order_cash,        '注文（現金）'
    entry '02', :order_receivable,  '注文（売掛金）'
    entry '03', :cancel_cash,       'キャンセル（現金）' #経理データ出力時には「キャンセル（売上）」になる
    entry '04', :cancel_receivable, 'キャンセル（売掛金）' #経理データ出力時には「キャンセル（売上）」になる
    entry '05', :point_summary,   'ポイント集計'
    entry '06', :point_expired,   'ポイント失効'
    entry '07', :point_created,   'ポイント付与'
    entry '08', :point_updated,   'ポイント変更'
    entry '09', :point_destroyed, 'ポイント削除'
  end
  
  selectable_attr :credit_account_div do
    entry '01', :order_cash,        '注文（現金）' #経理データ出力時には「注文（売上）」になる
    entry '02', :order_receivable,  '注文（売掛金）' #経理データ出力時には「注文（売上）」になる
    entry '03', :cancel_cash,       'キャンセル（現金）'
    entry '04', :cancel_receivable, 'キャンセル（売掛金）'
    entry '05', :point_summary,   'ポイント集計'
    entry '06', :point_expired,   'ポイント失効'
    entry '07', :point_created,   'ポイント付与'
    entry '08', :point_updated,   'ポイント変更'
    entry '09', :point_destroyed, 'ポイント削除'
  end
  
  # 注文経理データ登録
  def self.create_for_order(order_shipping)
    return if order_shipping.order.test_flg
    account = Accounting.new(
      :order_id => order_shipping.order_id,
      :debt_sales => order_shipping.total_amount,
      :debt_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :order_cash : :order_receivable,
      :credit_sales => order_shipping.total_amount,
      :credit_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :order_cash : :order_receivable,
      :given_point => order_shipping.order.given_point,
      :used_point => order_shipping.order.used_point
    )
    account.culc_previous_point
    account.save!
  end
  
  # キャンセル経理データ登録
  def self.create_for_cancel(order_shipping)
    return if order_shipping.order.test_flg
    account = Accounting.new(
      :order_id => order_shipping.order_id,
      :debt_sales => order_shipping.total_amount,
      :debt_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :cancel_cash : :cancel_receivable,
      :credit_sales => order_shipping.total_amount,
      :credit_account_div_key => order_shipping.payment_type_div_key == :bank_transfer ? :cancel_cash : :cancel_receivable,
      :given_point => 0 - order_shipping.order.given_point,
      :used_point => 0 - order_shipping.order.used_point
    )
    account.culc_previous_point
    account.save!
  end
  
  # ポイント集計経理データ登録
  def self.create_for_point_summary
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg
    account = Accounting.new(
      :debt_account_div_key => :point_summary,
      :credit_account_div_key => :point_summary,
      :given_point => 0,
      :used_point => 0
    )
    account.culc_previous_point(true)
    converted_price = account.previous_point * point_setting.exchange_rate
    account.debt_sales = converted_price
    account.credit_sales = converted_price
    account.save!
  end
  
  # ポイント期限切れ経理データ登録
  def self.create_for_point_expire(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && find_order_accounting(user_point.order_id)
    account = Accounting.new(
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_expired,
      :credit_account_div_key => :point_expired,
      :given_point => user_point.given_point,
      :used_point => 0
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # ポイント追加経理データ登録
  def self.create_for_point_create(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg
    account = Accounting.new(
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_created,
      :credit_account_div_key => :point_created,
      :given_point => user_point.given_point,
      :used_point => 0
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # ポイント変更経理データ登録
  def self.create_for_point_update(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && order_accounting = find_order_accounting(user_point.order_id)
    account = Accounting.new(
      :order_id => user_point.order_id,
      :user_point_id => user_point.id,
      :debt_account_div_key => :point_updated,
      :credit_account_div_key => :point_updated,
      :given_point => user_point.given_point - order_accounting.given_point,
      :used_point => user_point.used_point - order_accounting.used_point
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # ポイント削除経理データ登録
  def self.create_for_point_destroy(user_point)
    return if user_point.user.test_flg
    point_setting = PointSetting.find(:first)
    return unless point_setting.available_flg && order_accounting = find_order_accounting(user_point.order_id)
    account = Accounting.new(
      :order_id => user_point.order_id,
      :debt_account_div_key => :point_destroyed,
      :credit_account_div_key => :point_destroyed,
      :given_point => 0 - order_accounting.given_point,
      :used_point => 0 - order_accounting.used_point
    )
    account.culc_previous_point
    converted_price = (account.given_point - account.used_point) * point_setting.exchange_rate
    account.debt_sales = converted_price.abs
    account.credit_sales = converted_price.abs
    account.save!
  end
  
  # 経理データ登録可能な場合はtrueを返します
  def self.find_order_accounting(order_id)
    accounting = Accounting.find(:first, :conditions => ["order_id = ?", order_id], :order => "id desc")
    if accounting && ![:cancel_cash, :cancel_receivable, :point_expired, :point_destroyed].include?(accounting.debt_account_div_key)
      return accounting
    end
  end
  
  # 経理上のポイント残高をセットします
  def culc_previous_point(monthly = false)
    if monthly
      condition = ["debt_account_div != ? and created_at < ?", Accounting.debt_account_div_id_by_key(:point_summary), Time.now.beginning_of_month]
    else
      condition = ["debt_account_div != ?", Accounting.debt_account_div_id_by_key(:point_summary)]
    end
    if previous_accounting = Accounting.find(:first, :conditions => condition, :order => 'id desc')
      if previous_accounting.previous_point &&  previous_accounting.previous_point > 0
        last_point = previous_accounting.previous_point
      end
    end
    self.previous_point = (last_point || 0) + self.given_point - self.used_point
  end
end
