# -*- coding: utf-8 -*-
class CmsFile < ActiveRecord::Base
  belongs_to :cms_project, :foreign_key=> "cms_project_id"

  # CmsPublish::FileActionでどのように動作するべきかを判断するための属性です。
  # CmsPublishedProject#calc_type_keyの取り得る値が入ります。
  attr_accessor :publish_calc_type_key

  # validateメソッド内で記述しているエラーメッセージは、ec_adminのみで使用するメッセージです
  def validate
    if File.basename(public_file_path) =~ /[一-龠]+|[ぁ-ん]+|[ァ-ヴー]+/ 
      errors.add_to_base "ディレクトリ名に全角文字が含まれています。"
    end
    if public_file_path =~ /^public\/(product_images|product_images_for_preview|ec_rider|projects|inquiry_images|tmp_inquiry_images)\/{0,1}$/
      errors.add_to_base "publicディレクトリの直下には#{$&.sub('public/', '').sub('/','')}という名前のファイルやディレクトリを作成できません。"
    end
  end

  #ディレクトリの場合はtrueを返します。
  # public_file_pathの最後の文字が'/'の場合がディレクトリです。
  def directory?
    public_file_path =~ /.*\/$/ ? true : false
  end

  def public_dir_path
    directory? ? public_file_path : (File.dirname(public_file_path) + '/')#.gsub('//', '/')
  end

  # ec_admin/public/projects/...のファイルパスを返します
  def pj_file_path
    File.expand_path(File.join(EcRider.config.ec_admin_root, 'public', 'projects', cms_project_id.to_s, public_file_path))
  end

  def current_project?(project_id)
    cms_project_id ? cms_project_id.to_s == project_id.to_s : false
  end

  #
  # =一覧の検索条件=
  # プロジェクト編集対象のファイルと、プロジェクト編集対象外かつ公開中のファイルを表示します。
  # ファイルパス名が同じファイルは、プロジェクトの優先度が高いものを表示します。
  #
  class SearchCondition < EcRider::ConditionForm
    attr_accessor :cms_project_id

    # 一覧に表示するディレクトリまたはファイルのIDのリストと対象プロジェクト分の公開時削除フラグのリストを取得します。
    def file_list_ids(cms_project_id)
      cms_files = CmsFile.find(:all, :include => :cms_project, :order => 'public_file_path',
        :conditions => ["cms_project_id = ? or (cms_project_id <> ? and cms_projects.status_cd = ?) or cms_project_id is null",
          cms_project_id, cms_project_id, CmsProject.status_id_by_key(:opened)])

      path_id = {}
      deleted_ids = []
      cms_files.each do |file|
        unless file.cms_project_id
          path_id[file.public_file_path] = file
          next
        end
        if path_id.keys.include?(file.public_file_path)
          if file.cms_project_id == cms_project_id
            if file.cms_project.status_key != :opened || file.cms_project.priority > path_id[file.public_file_path].cms_project.priority
              if file.delete_at_publish
                deleted_ids << path_id[file.public_file_path].id
              else
                path_id[file.public_file_path] = file
              end
            elsif file.delete_at_publish
              deleted_ids << path_id[file.public_file_path].id
            end
          else
            if path_id[file.public_file_path].cms_project_id == cms_project_id
              if path_id[file.public_file_path].cms_project.status_key == :opened && file.cms_project.priority > path_id[file.public_file_path].cms_project.priority
                if file.delete_at_publish
                  deleted_ids << path_id[file.public_file_path].id
                else
                  path_id[file.public_file_path] = file
                end
              end
            else
              if file.cms_project.priority > path_id[file.public_file_path].cms_project.priority
                if file.delete_at_publish
                  deleted_ids << path_id[file.public_file_path].id
                  path_id.delete(file.public_file_path)
                else
                  path_id[file.public_file_path] = file
                end
              end
            end
          end
        else
          path_id[file.public_file_path] = file
        end
      end
      [path_id.values.map{|cms_file| cms_file.id }, deleted_ids]
    end

    def create_condition(options = nil)
      delete_ids = []
      where = Condition.block do |c|
        file_ids, delete_ids = file_list_ids(cms_project_id)
        file_ids << '0' if file_ids.empty?
        c.and "cms_files.id", "in", file_ids
      end
      [where, delete_ids]
    end
  end

  # ファイルを[ec_front or ec_test]/publicにコピーします。
  def overwrite(file_action)
    case file_action.cms_publish.target_key
      when :production
        root_dir = EcRider.config.ec_front_root
      when :test
        root_dir = EcRider.config.ec_test_root
      else
        root_dir = EcRider.config.ec_admin_root
    end
    dest_file = File.join(root_dir, public_file_path)
    FileUtils.mkdir_p(File.dirname(dest_file)) unless File.exist?(File.dirname(dest_file))
    FileUtils.cp(pj_file_path, dest_file)
  end

  # [ec_front or ec_test]/publicからファイルを削除します。
  def delete(file_action)
    case file_action.cms_publish.target_key
      when :production
        root_dir = EcRider.config.ec_front_root
      when :test
        root_dir = EcRider.config.ec_test_root
      else
        root_dir = EcRider.config.ec_admin_root
    end
    FileUtils.rm_rf(File.join(root_dir, public_file_path))
  end

end
