# -*- coding: utf-8 -*-
class PointSetting < ActiveRecord::Base

  # ポイント制度利用区分
  selectable_attr :available_flg do
    entry false, :false, '使用しない'
    entry true,  :true, '使用する'
  end

  # 有効期限有無区分
  selectable_attr :terminable_flg do
    entry false, :false, 'なし'
    entry true,  :true, 'あり'
  end

  # 有効期限基準日区分
  selectable_attr :term_div do
    entry '0',  :last_point, '最終ポイント発行日'
    entry '99', :manual, '日付指定'
  end

  # 有効期間区分
  selectable_attr :point_period_div do
    entry '1',  1, '1ヶ月'
    entry '6',  6, '6ヶ月'
    entry '12', 12, '12ヶ月'
    entry '24', 24, '24ヶ月'
  end

  # ポイント有効期限を返却する
  def create_term_date(today)
    # ポイント制度利用区分 なしならnil
    return nil unless self.available_flg
    # 有効期限 なしならnil
    return nil unless self.terminable_flg
    case self.term_div_key
    when :last_point
      # 最終ポイント発行日
      # 発行日から有効期間を設定して
      today = today >> self.point_period_div_key
      return DateTime.new(today.year, today.month, today.day, 23, 59, 59)
    when :manual
      # 日付指定
      term_day = DateTime.new(today.year, self.term_month, self.term_day, 23, 59, 59)
      if term_day > today
        return term_day
      else
        # 来年を設定
        year = today.year.to_i
        return DateTime.new((year +1), self.term_month, self.term_day, 23, 59, 59)
      end
    end
  end


  #未入力バリデーション
  validates_presence_of :term_div, :if => :available_flg_true?
  validates_presence_of :point_period_div, :if => :available_flg_true?
  validates_presence_of :term_month, :if => :term_div_checked_manual?
  validates_presence_of :term_day, :if => :term_div_checked_manual?

  # 数値バリデーション
  validates_numericality_of :issue_price, :exchange_rate, :only_integer =>true, :message => :EM990003, :if => :available_flg_true?
  validates_numericality_of :exchangeable_price, :only_integer =>true, :message => :EM990003, :if => :exchangeable_price_checked?
  validates_numericality_of :exchangeable_point, :only_integer =>true, :message => :EM990003, :if => :exchangeable_point_checked?

  #文字形式バリデーション
  validates_inclusion_of   :term_month,  :in => 1..12, :message =>:EM990007, :if => :term_div_checked_manual?
  validates_inclusion_of   :term_day,    :in => 1..31, :message =>:EM990007, :if => :term_div_checked_manual?


  # ポイント制度が「使用する」の場合trueを返す
  def available_flg_true?
    self.available_flg_key == :true
  end

  # 有効期限が「あり」の場合trueを返す
  def terminable_flg_true?
    self.terminable_flg_key == :true
  end

  # 有効期限基準日が「日付指定」かどうかを返します。
  def term_div_checked_manual?
    available_flg_true? && terminable_flg_true? && self.term_div_key == :manual
  end

  # チェックがついている場合にtrueを返す
  def exchangeable_price_checked?
    available_flg_true? && self.exchangeable_price_flg
  end

  # チェックがついている場合にtrueを返す
  def exchangeable_point_checked?
    available_flg_true? && self.exchangeable_price_flg
  end
end
