# -*- coding: utf-8 -*-
class Product < ActiveRecord::Base

  #
  # ec_front 商品選択、商品カテゴリ検索で使用するフォーム
  # 商品取得の条件は
  # 指定したカテゴリコードより自カテゴリ含む子供のカテゴリを含む商品
  # かつ、公開期間中の商品
  # かつ、非表示フラグの立っていない商品
  # かつ、プレゼント商品ではない商品
  #
  class CategorySearchForm < EcRider::ConditionForm
    include SelectableAttr::Base
    attr_accessor :keyword, :condition_div, :render_name, :per_page, :category_code, :order, :user_rank_id, :category_codes

    selectable_attr :condition_div do
      entry '0', :all, "すべてを含む"
      entry '1', :which, "いずれかを含む"
    end
    
    selectable_attr :order do
      entry '0', :default, "標準"
      entry '1', :low_to_high, "価格の安い順"
      entry '2', :high_to_low, "価格の高い順"
      entry '3', :new_to_old, "新着順"
    end

    def to_find_options(options = nil)
      joins = ['inner join product_displays on product_displays.product_id = products.id']
      joins << 'inner join product_skus on product_skus.product_id = products.id'

      # 検索条件に設定するカテゴリIDを取得
      category_ids = []
      # カテゴリ(単数)
      unless category_code.blank?
        # 選択されたカテゴリの子カテゴリも取得する
        product_category = ProductCategory.find(:first, :conditions => ["code = ?", category_code])
        category_ids = product_category.descendant_ids(true)
      end
      # カテゴリ(複数)
      unless category_codes.blank?
        codes = category_codes.split(',').each{|category_code| category_code.strip! }
        # 選択されたカテゴリの子カテゴリも取得する
        codes.each do |code|
          product_category = ProductCategory.find(:first, :conditions => ["code = ?", code])
          unless product_category.nil?
            category_ids.concat(product_category.descendant_ids(true))
          end
        end
      end
      
      # 検索条件作成
      where = Condition.block do |c|
        # カテゴリ
        unless category_ids.empty?
          c.and do |c1|
            c1.or("products.category_1_id", "in", category_ids)
            c1.or("products.category_2_id", "in", category_ids)
            c1.or("products.category_3_id", "in", category_ids)
            c1.or("products.category_4_id", "in", category_ids)
            c1.or("products.category_5_id", "in", category_ids)
          end
        end

        # キーワード
        # 検索区分
        unless keyword.blank?
          keywords = keyword.split(' ')
          # 全てを含む
          if condition_div_key == :all
            c.and do |c2| # ←カテゴリとキーワードはAND
              keywords.each do |key|
                c2.and do |c3| # ←すべてのキーワードを含むのでここがAND
                  c3.or("product_displays.product_name_1", "like", "%#{key}%")
                  c3.or("product_displays.product_name_2", "like", "%#{key}%")
                  c3.or("product_displays.product_outline", "like", "%#{key}%")
                  c3.or("product_displays.product_explanation", "like", "%#{key}%")
                  c3.or("product_displays.recommendation", "like", "%#{key}%")
                  c3.or("product_displays.recommendation_2", "like", "%#{key}%")
                  c3.or("product_displays.product_keyword", "like", "%#{key}%")
                end
              end
            end
          # いづれかを含む
          else
            c.and do |c4| # ←カテゴリとキーワードはAND
              keywords.each do |key|
                c4.or do |c5| # ←いづれかのキーワードを含むのでここがOR
                  c5.or("product_displays.product_name_1", "like", "%#{key}%")
                  c5.or("product_displays.product_name_2", "like", "%#{key}%")
                  c5.or("product_displays.product_outline", "like", "%#{key}%")
                  c5.or("product_displays.product_explanation", "like", "%#{key}%")
                  c5.or("product_displays.recommendation", "like", "%#{key}%")
                  c5.or("product_displays.recommendation_2", "like", "%#{key}%")
                  c5.or("product_displays.product_keyword", "like", "%#{key}%")
                end
              end
            end
          end
        end
      end
      visible_condition = "products.deleted_at is null AND "<<
        "product_skus.deleted_at is null AND "<<
        "product_displays.locale = ? AND "<<
        "product_skus.primary_sku_flg = 1 "
      unless EcRider.config.ec_admin
        visible_condition << "AND ((publish_started_at is null or publish_started_at <= ?)"
        visible_condition << " and (publish_finished_at is null or publish_finished_at >= ?))"
      end
      visible_condition << " and (present_flg is null or present_flg = 0)"
      visible_condition << ' and (invisible_flg is null or invisible_flg = 0)' unless (EcRider.config.ec_test ||  EcRider.config.ec_admin)
      now = EcRider.config.ec_test ? CmsPublish.latest_test_publish.designated_at : Time.now
      
      if where
        where[0] << " AND #{visible_condition}"
        if EcRider.config.ec_admin
          where.push("#{I18n.locale || 'ja'}")
        else
          where.push("#{I18n.locale || 'ja'}",now, now)
        end
      else
        if EcRider.config.ec_admin
          where = [visible_condition,"#{I18n.locale || 'ja'}"]
        else
          where = [visible_condition,"#{I18n.locale || 'ja'}", now, now]
        end
      end
      unless order.nil?
        if user_rank_id.nil? || user_rank_id.blank?
          user_rank = UserRank.find(:first, :conditions=>['no_membership_flg = 1'])
          _user_rank_id = user_rank.id unless user_rank.nil?
        else
          _user_rank_id = user_rank_id.to_i
        end
        joins << "left outer join product_skus cond_product_skus on cond_product_skus.product_id = products.id"
        joins << "left outer join product_prices cond_product_prices on cond_product_prices.product_sku_id = cond_product_skus.id and cond_product_prices.user_rank_id = #{_user_rank_id}"
      end

      #ソート条件
      #標準
      order_condition = 'products.display_order is null, products.display_order, products.product_code'
      #追加条件
      case order_key
       when :low_to_high
         order_condition = "cond_product_prices.standard_price, #{order_condition}"
       when :high_to_low
         order_condition = "cond_product_prices.standard_price desc, #{order_condition}"
       when :new_to_old
         order_condition = "products.publish_started_at desc, #{order_condition}"
       end
      result = {:conditions => where}
      result[:order] = order_condition
      result[:joins] = joins if joins.size >= 1
      result.update(options) if options
      return result
    end
  end

end
