# -*- coding: utf-8 -*-
require 'digest/sha1'
require 'base64'
class SpsResultHistory < ActiveRecord::Base
  belongs_to :sps_parameter_history
  acts_as_paranoid

  selectable_attr :ecr_result do
    entry 'NG',                     :ng,                  'OK,', :default_status_div_key => :no_need # こちら側の処理が正常ならOKを返す
    entry 'ECR_NG_CHECK_SUM',       :ng_check_sum,        'NG,購入要求情報のチェックサムが不正です。', :default_status_div_key => :no_need
    entry 'ECR_NG_REQUEST',         :ng_request,          'NG,購入要求情報の内容が不正です。', :default_status_div_key => :not_yet
    entry 'ECR_NG_OUT_OF_STOCK',    :ng_out_of_stock,     'NG,在庫不足です。',                 :default_status_div_key => :not_yet
    entry 'ECR_NG_OTHER_EXCEPTION', :ng_other_exception,  'NG,予期せぬエラーです。',           :default_status_div_key => :not_yet
    entry 'OK',                     :ok,                  'OK,', :default_status_div_key => :no_need # 正常に決済が完了
  end

  before_validation :set_test_flg

  def set_test_flg
    self.test_flg = true if self.test_flg.nil? && EcRider.config.ec_test
  end

  # 対応区分 
  selectable_attr :status_div, :enum => GmoTransactionHistory.status_div_enum

  # 注文
  def order
    self.sps_parameter_history.order if self.sps_parameter_history
  end
  
  # 会員
  def user
    return @user if @user
    @user = self.order.user if self.order
    @user
  end
  
  # 会員ID
  def user_id
    self.user.id if self.user.is_a?(User)
  end
  
  # ロケール
  def locale
    self.order.locale if self.order
  end
  
  #
  # = SPSの購入結果通知をもとに、SpsResultHistoryインスタンスを生成し保存する =
  # 入力：SPSから送られた購入結果
  # 出力：SpsResultHistoryインスタンス
  #
  def self.create_from_params(sps_result, save = true)
    attributes = {}
    keys = ATTRIBUTE_NAMES_ALL
    keys.each{|key| attributes[key] = (sps_result[get_sps_key_name(key)].blank?) ? nil : sps_result[get_sps_key_name(key)]}
    sps_result_history = self.new(attributes)
    sps_parameter_history = SpsParameterHistory.find_by_sps_order_id(sps_result_history.sps_order_id)
    unless sps_parameter_history.nil?
      # sps_parameter_historysとひもづける
      sps_result_history.sps_parameter_history_id = sps_parameter_history.id
      # ※Alipay決済ではsps_result_history_itemsのレコードは作成しない。他の決済方式では必要な場合がある。
    end

    sps_result_history.save! if save
    sps_result_history
  end

  #
  # = 購入結果通知が購入要求内容と同じかどうかを検証する =
  # 出力：OKならtrue / NGならfalse
  #
  def request_ok?
    sps_parameter_history = self.sps_parameter_history
    # sps_parameter_historysとひもづかないならNG
    if sps_parameter_history.nil?
      return false
    end

    # リクエストの内容をチェック
    keys = ATTRIBUTE_NAMES_REQUEST
    result = true
    keys.each do |key|
      unless self.attributes[key] == sps_parameter_history.attributes[key]
        result = false
        break
      end
    end

    result
  end

  #
  # = 購入結果通知のチェックサムを検証する =
  # 出力：OKならtrue / NGならfalse
  #
  def sps_hashcode_ok?
    return false unless self.sps_hashcode
    str = ATTRIBUTE_NAMES_ALL.map{|attr| self.send(attr) unless attr=='sps_hashcode'}.join
    str << EcRider.config.sps_hash_key
    self.sps_hashcode.upcase == Digest::SHA1.hexdigest(str).upcase ? true : false
  end

  #
  # = 処理結果を保存する =
  # 入力：EC-Rider処理結果
  #
  def update_ecr_result(key)
    self.ecr_result_key = key
    self.status_div_key = self.ecr_result_entry[:default_status_div_key]
    self.ecr_response = self.ecr_result_name
    
    # SPS側でOKでない場合はSPSの処理結果ステータスをセット
    self.ecr_result = self.res_result if key == :ng 
    
    self.save!
  end

  #
  # 決済取り消し依頼メールで表示するチェック後に返却される項目情報を文字列にして返却する
  #
  def create_payment_checked_content
    columns = ATTRIBUTE_NAMES_ALL.map{|attr| attr + " : " + self.send(attr).to_s}
    str = ""
    columns.each{|column| str << column + "\n"}
    return str
  end

  private
  #
  # SPSでの属性名を返す
  #
  def self.get_sps_key_name(key)
    key_names = {'sps_order_id' => 'order_id'}
    return (key_names[key] ? key_names[key] : key)
  end
  
  #
  # モデルの属性名（リクエスト時の内容）
  #
  ATTRIBUTE_NAMES_REQUEST = [
    'pay_method',
    'merchant_id',
    'service_id',
    'cust_code',
    'sps_cust_no',
    'sps_payment_no',
    'sps_order_id',
    'item_id',
    'pay_item_id',
    'item_name',
    'tax',
    'amount',
    'pay_type',
    'auto_charge_type',
    'service_type',
    'div_settele',
    'last_charge_month',
    'camp_type',
    'tracking_id',
    'terminal_type',
    'free1',
    'free2',
    'free3',
    'request_date'
  ]

  #
  # モデルの属性名（全て）
  # チェックサム計算でも使用するので、順序も関係している
  #
  ATTRIBUTE_NAMES_ALL = [
    'pay_method',
    'merchant_id',
    'service_id',
    'cust_code',
    'sps_cust_no',
    'sps_payment_no',
    'sps_order_id',
    'item_id',
    'pay_item_id',
    'item_name',
    'tax',
    'amount',
    'pay_type',
    'auto_charge_type',
    'service_type',
    'div_settele',
    'last_charge_month',
    'camp_type',
    'tracking_id',
    'terminal_type',
    'free1',
    'free2',
    'free3',
    'request_date',
    'res_pay_method',
    'res_result',
    'res_tracking_id',
    'res_sps_cust_no',
    'res_sps_payment_no',
    'res_payinfo_key',
    'res_payment_date',
    'res_err_code',
    'res_date',
    'limit_second',
    'sps_hashcode'
  ]
end
