# -*- coding: utf-8 -*-
class UserRank < ActiveRecord::Base
  versioned

  has_many :prices, :class_name => "ProductPrice", :foreign_key => 'user_rank_id',
    :dependent => :destroy
  
  selectable_attr :available_div do
    entry '0', :image, '未使用'
    entry '1', :text, '使用'
  end

  # 登録rank_listされている会員ランクの一覧を返却します。
  def self.user_rank_list()
    list = []
    find(:all, :order => 'id').each do |rank|
      list << [rank.name, rank.id]
    end
    return list
  end
  
  # CSVのヘッダの凡例に使用します
  def self.rank_remark
    find(:all, :order => 'code').map{|rank| "#{rank.code}:#{rank.name}"}.join('、')
  end
  
  class SearchForm < EcRider::ConditionForm
    include SelectableAttr::Base

    attr_accessor :code # 会員ランクコード
    attr_accessor :name # 会員ランク名

    def to_find_options(options = nil)
      @where, @params = [], []
      build_code
      build_name
      result = {
        :order => "code asc, created_at asc"
      }
      @params.unshift(@where.join(' AND ')) unless @where.empty?
      result[:conditions] = @params unless @params.empty?
      result
    end
    
    private
    def build_code
      return if code.blank?
      @where << "code like ?"
      @params << "%#{code}%"
    end
    
    def build_name
      return if name.blank?
      @where << "name like ?"
      @params << "%#{name}%"
    end
  end
end
