# -*- coding: utf-8 -*-
require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe ReminderController do

  fixtures :users, :user_emails

  #Delete these examples and add some real ones
  it "should use ReminderController" do
    controller.should be_an_instance_of(ReminderController)
  end

  describe "GET 'index'" do
    it "should be successful" do
      get 'index'
      response.should be_success
    end

    it "should be successful with flash[:reminder_form]" do
      flash[:reminder_form] = ReminderForm.new
      get 'index'
      response.should be_success
    end
  end

  describe "POST 'reminder_completion'" do
    it "should redirect with invalid reminder_form" do
      post 'reminder_completion', :reminder_form => {}
      response.should redirect_to(:action => :index, :lang => 'ja')
    end

    it "should redirect with valid reminder_form and invalid reminder_answer" do
      post('reminder_completion',
        :reminder_form => {
          :email => "email_address_01006@ec-one.com",
          :remember_question => "1",
          :remember_answer => "wrong_reminder_answer",
          :password => "1234",
          :password_confirmation => "1234"
        })
      response.should redirect_to(:action => :index, :lang => 'ja')
    end

    it "should redirect with valid reminder_form and valid reminder_answer without passwords" do
      post('reminder_completion',
        :reminder_form => {
          :email => "email_address_01006@ec-one.com",
          :remember_question => "1",
          :remember_answer => "ママ",
          :password => "",
          :password_confirmation => ""
        })
      response.should redirect_to(:action => :index, :lang => 'ja')
    end

    it "should redirect with valid reminder_form and valid reminder_answer" do
      post('reminder_completion',
        :reminder_form => {
          :email => "email_address_01006@ec-one.com",
          :remember_question => "1",
          :remember_answer => "ママ",
          :password => "ABCDE1",
          :password_confirmation => "ABCDE1"
        })
      response.should be_success
    end

    it "completion_url nil" do
      post('reminder_completion',
        :reminder_form => {
          :email => "email_address_01006@ec-one.com",
          :remember_question => "1",
          :remember_answer => "ママ",
          :password => "ABCDE1",
          :password_confirmation => "ABCDE1",
          :completion_url_controller => '',
          :completion_url_action => ''
        })
      response.should be_success
    end

    it "completion_url" do
      post('reminder_completion',
        :reminder_form => {
          :email => "email_address_01006@ec-one.com",
          :remember_question => "1",
          :remember_answer => "ママ",
          :password => "ABCDE1",
          :password_confirmation => "ABCDE1",
          :completion_url_controller =>"cart",
          :completion_url_action => "index"
        })
      response.should redirect_to(:controller => :cart, :action => :index, :lang => 'ja')
    end

  end



  describe "GET 'get_reminder_question'" do
    it "should get remember_question with valid email" do
      post('get_remember_question', :email => "email_address_01006@ec-one.com")
      response.should have_text("1")
    end
    it "should get nil with invalid email" do
      get :get_remember_question, :email => "not_exists_email@ec-one.com"
      response.should be_success
    end
  end
end
