# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  fixtures :products, :product_displays, :product_skus, :product_prices, :user_ranks, :product_images,
    :users, :point_settings, :sps_parameter_histories, :orders, :order_shippings,
    :campaigns, :campaign_target_product_skus, :campaign_effects, :campaign_conditions,
    :product_variations, :product_variation_displays
  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @order_form = OrderForm.new(:send_address_index => 0)
    @template.instance_variable_set(:@order_form, @order_form)
    @template.extend(ApplicationHelper)
  end

  describe "tags which works with" do
    def set_credit_auth_error
      @error_msg = "クレジットカードの認証に失敗しました。"
      request.session[:credit_auth_error] = @error_msg
    end

    def clear_credit_auth_error
      request.session[:credit_auth_error] = nil
    end

    describe "ec:if_credit_auth_faild" do
      it 'works' do
        invoked = false
        set_credit_auth_error
        ec.if_cedit_auth_failed { invoked = true }
        invoked.should be_true
      end

      it 'not works' do
        invoked = false
        clear_credit_auth_error
        ec.if_cedit_auth_failed { invoked = true }
        invoked.should be_false
      end
    end

    describe "ec:if_credit_auth_error" do
      it 'works' do
        set_credit_auth_error
        ec.order_credit_auth_error.should == @error_msg
      end

      it 'not works' do
        clear_credit_auth_error
        ec.order_credit_auth_error.should == ""
      end
    end
  end

  describe "ec:order_product_list" do
    def set_order_product_list
      cart = Cart.new()
      cart.add('0000000000001-00-00-00')
      cart.add('0000000000002-00-00-00')
      cart.add('0000000000003-00-00-00')
      request.session[:cart] = cart
    end

    it "with cart 3 products" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_name
        ""
      end
      passed_products.size.should == 3
    end

    it "with ec:order_product_name" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_name
        ""
      end
      passed_products.sort.should == ["商品1の商品名1", "商品2の商品名1 (S-赤)", "商品3の商品名1"]
    end

    it "with ec:order_product_price" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_price
        ""
      end
      passed_products.should == ["8,000", "8,000", "8,000"]
    end
    
    it "with ec:order_product_qty" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_qty
        ""
      end
      passed_products.should == ["1", "1", "1"]
    end

    it "with ec:order_product_given_point" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_given_point
        ""
      end
      passed_products.should == ["8", "8", "8"]
    end

    it "with ec:order_product_sub_total_price" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_sub_total_price
        ""
      end
      passed_products.should == ["8,000", "8,000", "8,000"]
    end

    it "with ec:order_product_sub_total_price" do
      time_now = Time.parse("2009/04/03 12:00:00")
      Time.stub!(:now).and_return(time_now)
      cart = Cart.new()
      cart.add('0000000009010-00-00-00')
      cart.add('0000000000002-00-00-00')
      cart.add('0000000000003-00-00-00')
      cart.calculate
      request.session[:cart] = cart

      sub_total_prices = []
      price_x_qty = []
      discounts = []
      ec.order_product_list do
        price_x_qty << ec.order_product_price_x_qty
        sub_total_prices << ec.order_product_sub_total_price
        discounts << ec.order_product_discount_price
        ""
      end
      ec.order_product_total_discount_price.should == "0"
      ec.order_product_total_price.should == '19,900'
      price_x_qty.should == ["8,000", "8,000", "7,800"]
      sub_total_prices.should == ["8,000", "8,000", "3,900"]
      discounts.should == ["0", "0","3,900"]
    end

    it "with ec:order_product_total_price" do
      set_order_product_list
      session[:cart].calculate
      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_total_price
        ""
      end
      passed_products.should == ["24,000", "24,000", "24,000"]
    end

    it "with ec:order_total_given_point" do
      set_order_product_list

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_total_given_point
        ""
      end
      passed_products.should == ["24", "24", "24"]
    end
  end

  describe "with ec:order_product_image" do
    before do
      cart = Cart.new()
      cart.add('0000000000001-00-00-00')
      request.session[:cart] = cart
    end
    it "nil" do
      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_image
        ""
      end
      passed_products.pop.should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001.gif][alt="商品1の商品名1"]')
    end
    it "with ec:order_product_image small" do

      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_image(:type => :small)
        ""
      end
      passed_products.pop.should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_small.gif][alt="商品1の商品名1"]')
    end
    it "with ec:order_product_image medium" do
      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_image(:type => :medium)
        ""
      end
      passed_products.pop.should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_medium.gif][alt="商品1の商品名1"]')
    end
    it "with ec:order_product_image large" do
      passed_products = []
      ec.order_product_list do
        passed_products << ec.order_product_image(:type => :large)
        ""
      end
      passed_products.pop.should have_tag('img[src^=/ec_rider/product_images/0000/0001/product00001_large.gif][alt="商品1の商品名1"]')
    end
  end
  describe "ec:button_to_confirmation_order" do
    it "should return button" do
      ec.button_to_confirmation_order{"次へ進む→"}.should have_tag("input[type=?][value=?]", 'submit', '次へ進む→')
    end
  end
  
  describe "ec:button_to_sps_payment" do
    it "should raise UsageError" do
      @template.instance_variable_set(:@sps_parameter_history, nil)
      lambda{ec.button_to_sps_payment{"決済を続ける"}}.should raise_error(EcRider::EcHelper::UsageError)
    end

    it "should return button and hidden fields" do
      sps_parameter_history = SpsParameterHistory.find(3)
      @template.instance_variable_set(:@sps_parameter_history, sps_parameter_history)
      ec.button_to_sps_payment{"決済を続ける"}.should have_tag("form[action=?]", EcRider.config.sps_uri) do
        with_tag("input[type=?][value=?]", 'submit', '決済を続ける')
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'pay_method', sps_parameter_history.pay_method)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'merchant_id', sps_parameter_history.merchant_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'service_id', sps_parameter_history.service_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'cust_code', sps_parameter_history.cust_code)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'sps_cust_no', sps_parameter_history.sps_cust_no)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'sps_payment_no', sps_parameter_history.sps_payment_no)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'order_id', sps_parameter_history.sps_order_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'item_id', sps_parameter_history.item_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'pay_item_id', sps_parameter_history.pay_item_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'item_name', sps_parameter_history.item_name)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'tax', sps_parameter_history.tax)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'amount', sps_parameter_history.amount)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'pay_type', sps_parameter_history.pay_type)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'auto_charge_type', sps_parameter_history.auto_charge_type)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'service_type', sps_parameter_history.service_type)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'div_settele', sps_parameter_history.div_settele)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'last_charge_month', sps_parameter_history.last_charge_month)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'camp_type', sps_parameter_history.camp_type)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'tracking_id', sps_parameter_history.tracking_id)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'terminal_type', sps_parameter_history.terminal_type)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'success_url', sps_parameter_history.success_url)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'cancel_url', sps_parameter_history.cancel_url)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'error_url', sps_parameter_history.error_url)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'pagecon_url', sps_parameter_history.pagecon_url)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'free1', sps_parameter_history.free1)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'free2', sps_parameter_history.free2)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'free3', sps_parameter_history.free3)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'free_csv', sps_parameter_history.free_csv)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'request_date', sps_parameter_history.request_date)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'limit_second', sps_parameter_history.limit_second)
        with_tag("input[type=?][name=?][value=?]", 'hidden', 'sps_hashcode', sps_parameter_history.sps_hashcode)

      end
    end
  end

end
