# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
  # fixtures :products, :product_displays, :name_masters, :ms_code_names, :ms_shop_settings, :users

  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
    @template.extend(ApplicationHelper)
  end

  describe "tags which works with current_user" do
    def set_current_user
      pending
      current_user = User.new(
        :name_last => "漢字名（姓）",
        :name_first => "漢字名（名）",
        :kana_name_last => "カナメイ（セイ）",
        :kana_name_first => "カナメイ（メイ）",
        :sex => "1",
        :nickname => "ニックネーム")
      @request.session[:current_user] = current_user
    end

    describe "ec:if_logged_in" do
      it '<% ec:if_logged_in %>～<% end %>' do
        pending
        @request.session[:user] = 1000
        login_as(users(:user_01004))
        invoked = false
        ec.if_logged_in{ invoked = true }
        invoked.should be_true

        @request.session[:user] = 1000
        invoked = false
        ec.if_logged_in{ invoked = true }
        invoked.should be_false
      end

      it '<% ec:if_not_logged_in %>～<% end %>' do
        pending
        @request.session[:user] = 1000
        invoked = false
        ec.if_not_logged_in{ invoked = true }
        invoked.should be_false

        @request.session[:user] = nil
        invoked = false
        ec.if_not_logged_in{ invoked = true }
        invoked.should be_true
      end
    end

    it '<%= ec:name_last %>' do
      set_current_user
      ec.name_last.should == "漢字名（姓）"
    end

    it '<%= ec:kanji_family_name %>' do
      set_current_user
      ec.kanji_family_name.should == "漢字名（姓）"
    end

    it '<%= ec:name_first %>' do
      set_current_user
      ec.kanji_name.should == "漢字名（名）"
    end

    it '<%= ec:kanji_name %>' do
      set_current_user
      ec.kanji_name.should == "漢字名（名）"
    end

    it '<%= ec:kana_name_last %>' do
      set_current_user
      ec.kana_family_name.should == "カナメイ（セイ）"
    end

    it '<%= ec:kana_family_name %>' do
      set_current_user
      ec.kana_family_name.should == "カナメイ（セイ）"
    end

    it '<%= ec:kana_name_first %>' do
      set_current_user
      ec.kana_name.should == "カナメイ（メイ）"
    end

    it '<%= ec:kana_name %>' do
      set_current_user
      ec.kana_name.should == "カナメイ（メイ）"
    end

    it '<%= ec:full_name %>' do
      set_current_user
      ec.full_name.should == "漢字名（姓） 漢字名（名）"
    end

    it '<%= ec:honorific_prefix %>' do
      set_current_user
      ec.honorific_prefix.should == "Mr."
    end

    it '<%= ec:nickname %>' do
      pending
      set_current_user
      ec.nickname.should == "ニックネーム"
    end
  end


  describe '<%= ec:member_registration_link %>' do
    it "should work without options" do
      ec.member_registration_link.should have_tag('a')
      ec.member_registration_link.should_not have_tag('img')
    end

    it "should work with src" do
      ec.member_registration_link(:src => "aaa.gif").should have_tag('a') do
        with_tag('img[src^=/images/aaa.gif]')
      end
    end

    it "should work with text" do
      link = ec.member_registration_link(:text => "リンク")
      link.should have_tag('a')
      link.should include("リンク")
      link.should_not have_tag('img')
    end

    it "should work with block" do
      ec.member_registration_link(:text => "textは無視される"){"使われる文字列"}
      link = @template.output_buffer
      link.should have_tag('a')
      link.should include("使われる文字列")
      link.should_not include("textは無視される")
    end
  end

  it '<%= ec:login_link %>' do
    ec.login_link(:src => "aaa.gif").should have_tag('a') do
      with_tag('img[src^=/images/aaa.gif]')
    end

    ec.login_link(:text => "リンク").should have_tag('a')
    ec.login_link(:text => "リンク").should_not have_tag('img')

    ec.login_link.should have_tag('a')
    ec.login_link.should_not have_tag('img')
  end

  it '<%= ec:logout_link %>' do
    ec.logout_link(:src => "aaa.gif").should have_tag('a') do
      with_tag('img[src^=/images/aaa.gif]')
    end

    ec.logout_link(:text => "リンク").should have_tag('a')
    ec.logout_link(:text => "リンク").should_not have_tag('img')

    ec.logout_link.should have_tag('a')
    ec.logout_link.should_not have_tag('img')
  end

  describe '<%= ec:link_to_remider %>' do
    it "should not work without options" do
      ec.link_to_remider.should be_nil
    end

    it "should work with text" do
      link = ec.link_to_remider(:text => "リンク")
      link.should have_tag('a')
      link.should include("リンク")
    end

    it "should work with block" do
      ec.link_to_remider(:text => "textは無視される"){"使われる文字列"}
      link = @template.output_buffer
      link.should have_tag('a')
      link.should include("使われる文字列")
      link.should_not include("textは無視される")
    end
  end

  describe 'ec:banners_for banner' do
    fixtures :banners, :banner_displays
    describe '会員の場合' do
      before do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(false)
        @template.output_buffer = ""
        @template.extend(ApplicationHelper)
        user = User.find_by_id(1005)
        @template.stub(:logged_in?).and_return(true)
        @template.stub(:current_user).and_return(user)
      end
      it "画像バナーの場合 会員の場合" do
        I18n.stub(:locale).and_return('ja')

        ec.banners_for(:banner_code => "11111111", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
        @template.output_buffer.should have_tag('a') do
          with_tag('img[src^=/banner_images/banner_0000000001.png]')
        end
      end
      it "テキストバナーの場合 会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
      end
      it "多言語用バナーがない場合 会員の場合" do
        I18n.stub(:locale).and_return('ko')

        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should_not have_tag('a[href=?]', 'http://www.ec-one.com')
      end
    end
    describe '非会員の場合' do
      before do
        EcRider.config.stub!(:ec_admin).and_return(false)
        EcRider.config.stub!(:ec_test).and_return(false)
        @template.output_buffer = ""
        @template.extend(ApplicationHelper)
        user = User.new()
        @template.stub(:logged_in?).and_return(true)
        @template.stub(:current_user).and_return(user)
      end
      it "画像バナーの場合 非会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "11111111", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
        @template.output_buffer.should have_tag('a') do
          with_tag('img[src^=/banner_images/banner_0000000001.png]')
        end
      end
      it "テキストバナーの場合 非会員の場合" do
        I18n.stub(:locale).and_return('ja')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should have_tag('a[href=?]', 'http://www.ec-one.com')
      end
      it "多言語用バナーがない場合 非会員の場合" do
        I18n.stub(:locale).and_return('ko')
        ec.banners_for(:banner_code => "00000002", :display_qty => "6") do
          @template.concat(ec.banner_link())
        end
        @template.output_buffer.should_not have_tag('a[href=?]', 'http://www.ec-one.com')
      end
    end
  end
end
