# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../../../spec_helper'

describe EcRider::EcHelper do
   fixtures :products, :user_points, :users, :orders
  def ec
    helper.ec
  end

  before(:each) do
    @template.output_buffer = ""
  end

  def concat(*args)
    args.each{|arg| @template.concat(arg)}
  end

  describe "ec:user_point" do
    it "normal usage" do
      @template.extend(ApplicationHelper)
      @template.stub!(:current_user).and_return(User.find(1003))
      ec.user_point_total_point.should == "1400"
      ec.user_point_term_point.should == nil # pending
      order_code_list = []
      order_datetime_list = []
      order_product_name_list = []
      order_price_list = []
      given_point_list = []
      used_point_list = []
      ec.user_point_point_list do
        order_code_list << ec.user_point_order_code
        order_datetime_list << ec.user_point_order_datetime(:format => "%Y年%m月%d日 %X")
        ec.user_point_order_product_list do
          order_product_name_list << ec.user_point_order_product_name
          @template.concat("")
        end
        order_price_list << ec.user_point_order_price
        given_point_list << ec.user_point_given_point
        used_point_list << ec.user_point_used_point
        @template.concat("")
      end
      order_code_list[0].size == 3
    end
    describe "user_point_term_date" do
      it "日付フォーマットがデフォルトの場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1001))
        ec.user_point_term_date.should == "08/01/02"
      end
      it "日付フォーマット指定の場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1001))
        ec.user_point_term_date(:format => "%Y年%m月%d日").should == "2008年01月02日"
      end
    end

    describe "user_point_order_datetime" do
      describe "with fixtures" do
        fixtures :orders

        it "日付フォーマットがデフォルトの場合" do
          @template.extend(ApplicationHelper)
          @template.stub!(:current_user).and_return(User.find(1001))
          order_datetime_list = []
          ec.user_point_point_list do
            order_datetime_list << ec.user_point_order_datetime
            @template.concat("")
          end
          order_datetime_list[0].should == "07/12/10"
          order_datetime_list[1].should == "07/12/10"
        end
        it "日付フォーマット指定の場合" do
          @template.extend(ApplicationHelper)
          @template.stub!(:current_user).and_return(User.find(1001))
          order_datetime_list = []
          ec.user_point_point_list do
            order_datetime_list << ec.user_point_order_datetime(:format => "%Y年%m月%d日")
            @template.concat("")
          end
          order_datetime_list[0].should == "2007年12月10日"
        end
      end
      
      it "注文がない場合の場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_datetime_list = []
        ec.user_point_point_list do
          order_datetime_list << ec.user_point_order_datetime(:empty_format => "-")
          @template.concat("")
        end
        order_datetime_list[0].should == "-"
      end
    end
    describe "user_point_order_code" do
      it "注文がない場合の場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_code_list = []
        ec.user_point_point_list do
          order_code_list << ec.user_point_order_code(:empty_format => "-")
          @template.concat("")
        end
        order_code_list[0].should == ["-", {}]
      end      
    end
    describe "user_point_order_price" do
      it "注文がない場合の場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        order_price_list = []
        ec.user_point_point_list do
          order_price_list << ec.user_point_order_price(:empty_format => "-")
          @template.concat("")
        end
        order_price_list[0].should == "-"
      end
    end
    describe "user_point_order_product_list" do
      it "注文がない場合の場合" do
        @template.extend(ApplicationHelper)
        @template.stub!(:current_user).and_return(User.find(1005))
        flag = true
        ec.user_point_point_list do
          ec.user_point_order_product_list do
            @template.concat("")
            flag = false
          end
          @template.concat("")
        end
        flag.should == true
      end
    end

  end


end
