# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'
require 'pathname'

describe EcRider::CmsSetup do
  fixtures :cms_bind_contents, :cms_bind_items, :cms_binds, :cms_page_titles, :cms_pages, :cms_projects,
    :cms_templates, :cms_template_contents, :cms_files

  include EcRider::CmsSetup

  describe "create_ecrider_cms_pages" do
    it "ec-rider提供ページ" do
      create_ecrider_cms_pages

      cms_pages = CmsPage.find(:all, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles, :cms_project],
        :conditions => ["controller = ?", 'account'])
      cms_pages.length.should == 10
      cms_pages.each do |page|
        page.cms_project_id.should == 1
        page.action.should_not be_nil
        page.binds.length.should == 5
        page.binds.each do |bind|
          if bind.position_key == :center
            bind.cms_template.name.should_not be_nil
            bind.cms_template.cms_project_id.should == 1
            bind.cms_template.development_code.should == "account/#{page.action}"
            bind.cms_template.contents.length.should == 6
            bind.cms_template.contents.each do |content|
              content.locale.should_not be_nil
              content.descriptions.should_not be_nil
            end
          end
        end
        page.titles.length.should == 6
        page.titles.each do |title|
          title.locale.should_not be_nil
          title.name.should_not be_nil
        end
      end
    end

    it "DBに無いロケールを登録" do
      create_ecrider_cms_pages
      CmsTemplateContent.destroy_all(:locale => 'th')
      CmsTemplateContent.count(:conditions => ['locale = ? ', 'th']).should == 0
      create_ecrider_cms_pages
      CmsTemplateContent.count(:conditions => ['locale = ? ', 'th']).should > 0
    end
  end

  describe "create_shop_base_cms_pages" do
    it "店舗独自ページ" do
      create_shop_base_cms_pages

      cms_pages = CmsPage.find(:all, :include => [{:binds => [{:items => :contents}, :cms_template]}, :titles, :cms_project],
        :conditions => [" cms_project_id = ? and controller = ? and action = ?", CmsProject::BASE_PROJECT_ID, 'cms', 'index'])
      cms_pages.length.should == 1
      cms_pages.each do |page|
        page.cms_project_id.should == 1
        page.action.should_not be_nil
        page.binds.length.should == 5
        page.binds.each do |bind|
          if bind.position_key == :center
            bind.cms_template.name.should_not be_nil
            bind.cms_template.cms_project_id.should == 1
            bind.cms_template.development_code.should == "cms/index"
            bind.cms_template.contents.length.should == 6
            bind.cms_template.contents.each do |content|
              content.locale.should_not be_nil
              content.descriptions.should_not be_nil
            end
          end
        end
        page.titles.length.should == 6
        page.titles.each do |title|
          title.locale.should_not be_nil
          title.name.should_not be_nil
        end
      end
    end
  end

  describe "create_shop_base_cms_templates" do
    it "中央以外のテンプレート" do
      create_shop_base_cms_templates

      cms_templates = CmsTemplate.find(:all, :conditions => ["development_code = ?", 'cms/top'])
      cms_templates.length.should == 1
      cms_templates.each do |template|
        template.cms_project_id.should == 1
        template.contents.length.should == 6
        template.contents.each do |content|
          content.locale.should_not be_nil
          content.descriptions.should_not be_nil
        end
        template.default_template_position_key.should == :top
      end
    end
  end

  describe "create_base_cms_file" do
    before(:each) do
      tmp_dir = File.expand_path(File.join(Dir.tmpdir, "test_cms_projects"))
      Dir.glob(File.join(tmp_dir, '*')) .each{|f| FileUtils.rm_rf(f) }
      FileUtils.mkdir_p(File.join(tmp_dir, 'admin', 'public', 'projects', '1', 'public'))
      EcRider.config.stub!(:ec_admin_root).and_return(File.join(tmp_dir, 'admin'))
      # EcRider.config.stub!(:ec_front_root).and_return(File.join(tmp_dir, 'front'))
    end
    
    it "既存ディレクトリ、新規ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'images', 'buttons'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'images', 'buttons', 'botton_cart.gif'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsFile, :count).by(2)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/images/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/images/buttons/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?' ,CmsProject::BASE_PROJECT_ID, 'public/images/buttons/botton_cart.gif']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'images', 'buttons', 'botton_cart.gif')).should be_true
    end
    
    it "新規ディレクトリ、新規ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style', 'product.css'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'newdir', 'style', 'common.css'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsFile, :count).by(4)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/newdir/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/newdir/style/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/newdir/style/product.css']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/newdir/style/common.css']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'newdir', 'style', 'product.css')).should be_true
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'newdir', 'style', 'common.css')).should be_true
    end
    
    it "既存ディレクトリ、既存ファイル" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', '500.html'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsTemplate, :count).by(0)
      CmsFile.count(:conditions => ['cms_project_id is null and public_file_path=?', 'public/']).should == 1
      CmsFile.count(:conditions => ['cms_project_id=? and public_file_path=?', CmsProject::BASE_PROJECT_ID, 'public/500.html']).should == 1
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', '500.html')).should be_true
    end
    
    it "対象外" do
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', '.ignorefile'))
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'javascripts'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'javascripts', '.ignorefile'))
      FileUtils.mkdir_p(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'ignoredir'))
      FileUtils.touch(File.join(EcRider.config.ec_admin_root, 'initial_data', 'cms', 'public', 'ignoredir', '.ignorefile'))
    
      lambda{
        create_base_cms_file
      }.should change(CmsTemplate, :count).by(0)
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', '.ignorefile')).should be_false
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'javascripts', '.ignorefile')).should be_false
      File.exist?(File.join(EcRider.config.ec_admin_root, 'public', 'projects', '1', 'public', 'ignoredir', '.ignorefile')).should be_false
    end
  end

  describe "cleanup_project" do
    it "プロジェクトを初期化" do
      cleanup_project
      CmsProject.count(:conditions => ['id <> ?', CmsProject::BASE_PROJECT_ID]).should == 0
      base_project = CmsProject.find(CmsProject::BASE_PROJECT_ID)
      base_project.acceptance_type_key.should == :no_approval
      base_project.status_key.should == :approved
    end
  end

end
