# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe CreditForm do

  describe "クレジットカード番号のバリデーション" do
    before(:each) do
      @attrs = {:month => 12, :year => 2010, :name => "econe"}
    end

    describe :visa do
      it "valid length(16)" do
        credit_form = CreditForm.new({
            :kind => :visa,
            :card_no => '1' * 16
          }.update(@attrs))
        credit_form.should be_valid
      end

      it "invalid length 17" do
        credit_form = CreditForm.new({
            :kind => 'visa',
            :card_no => '1' * 17
          }.update(@attrs))
        credit_form.should_not be_valid
        credit_form.errors.on(:card_no).should == "は16桁以下で入力してください。"
      end
    end

    describe :amex do
      it "valid length 15" do
        credit_form = CreditForm.new({
            :kind => 'amex',
            :card_no => '1' * 15
          }.update(@attrs))
        credit_form.should be_valid
      end

      it "invalid length 16" do
        credit_form = CreditForm.new({
            :kind => 'amex',
            :card_no => '1' * 16
          }.update(@attrs))
        credit_form.should_not be_valid
        credit_form.errors.on(:card_no).should == "は15桁以下で入力してください。"
      end
    end

    describe :dinners do
      it "valid length 14" do
        credit_form = CreditForm.new({
            :kind => 'diners',
            :card_no => '1' * 14
          }.update(@attrs))
        credit_form.should be_valid
      end

      it "invalid length 15" do
        credit_form = CreditForm.new({
            :kind => 'diners',
            :card_no => '1' * 15
          }.update(@attrs))
        credit_form.should_not be_valid
        credit_form.errors.on(:card_no).should == "は14桁以下で入力してください。"
      end
    end
  end
  
  describe :expire do
    it "string year and month" do
      form = CreditForm.new(:year => '10', :month => '02')
      form.expire.should == '1002'
    end
    
    it "integer year and month" do
      form = CreditForm.new(:year => 2010, :month => 8)
      form.expire.should == '1008'
    end
  end

end
