# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe GmoTransactionHistory do
  fixtures :users, :user_emails, :user_addresses, :orders, :order_shippings, :order_items, :gmo_transaction_histories

  it "status_div means kind of status for online payment." do
    GmoTransactionHistory.status_div_options.should == [
      ["対応必要なし", '0'],
      ["未対応", '1'],
      ["返答待ち", '2'],
      ["対応完了", '3']
    ]
    GmoTransactionHistory.status_div_id_by_key(:no_need).should == "0"
    GmoTransactionHistory.status_div_id_by_key(:not_yet).should == "1"
    GmoTransactionHistory.status_div_id_by_key(:waiting).should == "2"
    GmoTransactionHistory.status_div_id_by_key(:complete).should == "3"
  end

  it "exe_div means which the online payment is first time or again." do
    GmoTransactionHistory.exe_div_options.should == [
      ["初回実行", '1'],
      ["再実行", '2']
    ]
    GmoTransactionHistory.exe_div_id_by_key(:first_time).should == "1"
    GmoTransactionHistory.exe_div_id_by_key(:again).should == "2"
  end


  it "GmoTransactionHistory.test_is_system_err_code? true pattern" do
    assert_equal true, GmoTransactionHistory.is_system_err_code?('E01')
  end

  it "GmoTransactionHistory.test_is_system_err_code? false pattern" do
    assert_equal false, GmoTransactionHistory.is_system_err_code?('G01')
  end

  it "OnlinePaymentHistory#test_is_system_err_code? true pattern" do
    history = GmoTransactionHistory.new(:err_code => 'E01')
    assert_equal true, history.is_system_err_code?
  end

  it "OnlinePaymentHistory#test_is_system_err_code? false pattern" do
    history = GmoTransactionHistory.new(:err_code => 'G01')
    assert_equal false, history.is_system_err_code?
  end



   #
  # 実行日開始日のバリデーションテスト
  # YYYY/MM/DDのフォーマットの場合、trueが返されること
  it "test_validation_exe_date_1" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_from => '2008/1/14')
    assert form.valid?
  end

  #
  # 実行日開始日のバリデーションテスト
  # YYYY/MM/DDのフォーマットではない場合、エラーが返されること
  it "test_validation_exe_date_2" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_from => '2008/114')
    assert_equal false, form.valid?
  end

  #
  # 実行日開始日のバリデーションテスト
  # 存在しない日付の場合、エラーが返されること
  it "test_validation_exe_date_3" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_from => '2008/2/30')
    assert_equal false, form.valid?
  end

  #
  # 実行日時間FROMのバリデーションテスト
  # HH:mm:ssのフォーマットで存在する時刻の場合trueが返されること
  it "test_validation_exe_time_from_1" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_from => '12:43:12')
    assert form.valid?
  end

  #
  # 実行日時間FROMのバリデーションテスト
  # HH:mm:ssのフォーマットでない場合はfalseが返されること
  it "test_validation_exe_time_from_2" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_from => '1243:12')
    assert_equal false, form.valid?
  end

  #
  # 実行日時間FROMのバリデーションテスト
  # 存在しない時間の場合falseが返されること
  it "test_validation_exe_time_from_3" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_from => '12:43:99')
    assert_equal false, form.valid?
  end

  #
  # 実行日TOのバリデーションテスト
  # YYYY/MM/DDのフォーマットの場合、trueが返されること
  it "test_validation_exe_date_to_1" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_to => '2008/1/14')
    assert form.valid?
  end

  #
  # 実行日TOのバリデーションテスト
  # YYYY/MM/DDのフォーマットではない場合、エラーが返されること
  it "test_validation_exe_date_to_2" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_to => '2008/114')
    assert_equal false, form.valid?
  end

  #
  # 実行日TOのバリデーションテスト
  # 存在しない日付の場合、エラーが返されること
  it "test_validation_exe_date_to_3" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_date_to => '2008/2/30')
    assert_equal false, form.valid?
  end

  #
  # 実行日時間TOのバリデーションテスト
  # HH:mm:ssのフォーマットで存在する時刻の場合trueが返されること
  it "test_validation_exe_time_to_1" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_to => '12:43:12')
    assert form.valid?
  end

  #
  # 実行日時間toのバリデーションテスト
  # HH:mm:ssのフォーマットでない場合はfalseが返されること
  it "test_validation_exe_to_from_2" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_to => '1243:12')
    assert_equal false, form.valid?
  end

  #
  # 実行日時間TOのバリデーションテスト
  # 存在しない時間の場合falseが返されること
  it "test_validation_exe_to_from_3" do
    form =  GmoTransactionHistory::SearchConditionForm.new(:exe_time_to => '12:43:99')
    assert_equal false, form.valid?
  end

  describe 'create_condition' do
    #
    # 検索テスト
    # 検索条件が何も指定されなかった場合
    def through_test_search_error_gmo_transaction_histories_1
      form = GmoTransactionHistory::SearchConditionForm.new
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 5, result.size
    end

    #
    # 検索テスト
    # 範囲指定(対象の範囲が存在しないケース
    # 検索結果が一件も返されないこと
    it "test_search_error_gmo_transaction_histories_2" do
      form = GmoTransactionHistory::SearchConditionForm.new(:exe_date_from=>'2007/12/12', :exe_time_from=>'23:59:59')
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 0, result.size
    end

    #
    # 検索テスト
    # 範囲指定(対象の日付が3件存在する場合)
    # 3件の結果が返されること
    it "test_search_error_gmo_transaction_histories_3" do
      form = GmoTransactionHistory::SearchConditionForm.new(:exe_date_from=>'2007/12/10', :exe_time_from=>'00:00:00',
        :exe_date_to => '2007/12/10', :exe_time_to => '23:59:59')
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 5, result.size
    end

    #
    # 検索テスト
    # 範囲指定(対象の日付が3件存在する場合) + GpayOrderIdの指定
    # 1件の結果が返されること
    it "test_search_error_gmo_transaction_histories_4" do
      form = GmoTransactionHistory::SearchConditionForm.new(:exe_date_from=>'2007/12/10', :exe_time_from=>'00:00:00',
        :exe_date_to => '2007/12/10', :exe_time_to => '23:59:59', :gmo_order_id => '40000003')
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 1, result.size
    end

    #
    # 検索テスト
    # 範囲指定(対象の日付が5件存在する場合) + 初回実行のみの絞込み
    # 1件の結果が返されること
    it "test_search_error_gmo_transaction_histories_5" do
      form = GmoTransactionHistory::SearchConditionForm.new(:exe_date_from=>'2007/12/10', :exe_time_from=>'00:00:00',
        :exe_date_to => '2007/12/12', :exe_time_to => '23:59:59', :exe_div => '2')
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 1, result.size
    end

    #
    # 検索テスト
    # エラー詳細コードでの絞り込み
    # 1件の結果が返されること
    it "test_search_error_gmo_transaction_histories_6" do
      form = GmoTransactionHistory::SearchConditionForm.new(:err_code => 'E010')
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      assert_equal 1, result.size
    end

    it "本番決済のみ" do
      GmoTransactionHistory.update(1, :test_flg => true)
      form = GmoTransactionHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:production))
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      result.length.should == 6
    end

    it "テスト決済のみ" do
      GmoTransactionHistory.update(1, :test_flg => true)
      form = GmoTransactionHistory::SearchConditionForm.new(:test_flg => CmsPublish.target_id_by_key(:test))
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      result.length.should == 1
    end

    it "本番決済とテスト決済" do
      GmoTransactionHistory.update(1, :test_flg => true)
      form = GmoTransactionHistory::SearchConditionForm.new
      result = GmoTransactionHistory.find(:all, :conditions => GmoTransactionHistory.create_condition(form))
      result.length.should == 7
    end
  end

  describe "test?" do
    it "テスト決済" do
      GmoTransactionHistory.new(:test_flg => true).test?.should be_true
    end

    it "本番決済" do
      GmoTransactionHistory.new().test?.should be_false
      GmoTransactionHistory.new(:test_flg => "").test?.should be_false
      GmoTransactionHistory.new(:test_flg => nil).test?.should be_false
      GmoTransactionHistory.new(:test_flg => false).test?.should be_false
    end
  end

  describe 'set_test_flg' do
    it "ec_adminの場合　本番データ" do
      gmo = GmoTransactionHistory.new
      gmo.set_test_flg
      gmo.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      gmo = GmoTransactionHistory.new(:test_flg => true)
      gmo.set_test_flg
      gmo.test_flg.should be_true
    end

    it "ec_frontの場合" do
      gmo = GmoTransactionHistory.new
      gmo.set_test_flg
      gmo.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      gmo = GmoTransactionHistory.new
      gmo.set_test_flg
      gmo.test_flg.should be_true
    end

    it "ec_frontの場合　test_flgはnilで登録" do
      gmo = GmoTransactionHistory.new(:order_id => '12345')
      gmo.save!
      GmoTransactionHistory.find(:first, :conditions => ["order_id = ?", '12345']).test_flg.should be_nil
    end

    it "ec_testの場合　test_flgはtrueで登録" do
      EcRider.config.stub!(:ec_test).and_return(true)
      gmo = GmoTransactionHistory.new(:order_id => '12345')
      gmo.save!
      GmoTransactionHistory.find(:first, :conditions => ["order_id = ?", '12345']).test_flg.should be_true
    end
  end
  describe "test user" do
    it "get user" do
      sps_result_history = GmoTransactionHistory.find(1)
      sps_result_history.user_id.should == 1005

      user = sps_result_history.user
      user.is_a?(User).should be_true
      user.id.should == 1005
      user.fullname.should == "山下 菜々子"
      user.user_email.email.should == "email_address_01005@ec-one.com"
    end
  end
end
