# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe OrderForm do
  fixtures :distributors, :distributor_delivery_times, :distributor_delivery_time_displays, :countries

  describe "in japanese." do
    describe "Validate check." do
      describe "The required check." do
        it "The required check of the send_address_index." do
          @param = {:send_address_index => "", :sent_one_place => "1"}
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:send_address_index).should == "を入力してください。"
        end
      end
      describe "A number of the figures check" do
        before(:each) do
          @free_request = ""
          @param = {:send_address_index => "1", :sent_one_place => "1"}
        end

        it "An error does not occur by a figure number of free_request check" do
          1000.times {@free_request += "A"}
          @param[:free_request] = @free_request
          order_form = OrderForm.new(@param)
          # 桁数チェック結果
          order_form.should be_valid
        end

        it "An error occurs by a figure number of free_request check" do
          1001.times {@free_request += "A"}
          @param[:free_request] = @free_request
          order_form = OrderForm.new(@param)
          # 桁数チェック結果
          order_form.should_not be_valid
          order_form.errors.on(:free_request).should == "は1000文字以内で入力してください。"
        end
      end

      describe "Credit card expire check" do
        before(:each) do
          @param = {:send_address_index => "1", :sent_one_place => "1", :normal_payment_type => '2', :card_kind => 'visa', :card_name => "ECONE", :no_1 => "1111", :no_2 => "1111", :no_3 => "1111", :no_4 => "1111"}
        end

        it "The normal system check of the expiration date of the credit card" do
          @param[:expire_year] = "10"
          @param[:expire_month] = "09"
          order_form = OrderForm.new(@param)
          order_form.should be_valid
        end

        it "A system check of the length of the expiration date of the credit card" do
          @param[:expire_year] = "1"
          @param[:expire_month] = "9"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire_year).should == "は２桁の数値で入力してください。"
          order_form.errors.on(:expire_month).should == "は２桁の数値で入力してください。"
        end

        it "The system check of the number of the expiration date of the credit card" do
          @param[:expire_year] = "aa"
          @param[:expire_month] = "bb"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire_year).should == "は２桁の数値で入力してください。"
          order_form.errors.on(:expire_month).should == "は２桁の数値で入力してください。"
        end

        it "A system check of the validity of the expiration date of the credit card" do
          @param[:expire_year] = "10"
          @param[:expire_month] = "13"
          order_form = OrderForm.new(@param)
          order_form.should_not be_valid
          order_form.errors.on(:expire).should == "は正しくありません。"
        end

        it "amex test" do
          @param[:card_kind] = 'amex'
          model = OrderForm.new(@param)
          model.credit_card_not_amex?.should be_false
        end
      end
    end
  end
  
  describe "other method." do
    it "The assembling of the card number" do
      @param = {:no_1 => "1234",:no_2 => "1234",:no_3 => "1234",:no_4 => "1234"}
      test_order_form = OrderForm.new(@param)
      test_order_form.card_no.should == "1234123412341234"
    end
  end

  describe "配送希望日時" do
    before(:each) do
      @distributor = Distributor.first
      countory = Country.find_by_code("JP")
      countory.distributor_id = @distributor.id
      countory.save!
    end

    it "配送希望日の一覧を取得する（引数に国を指定していない場合は、「日本」の設定が返却される）" do
      delivery_start = @distributor.deliverable_term_start
      delivery_end = @distributor.deliverable_term_end
      available_delivery_date_count = (delivery_end - delivery_start) + 1

      order = OrderForm.new()
      available_delivery_date = order.available_delivery_date
      # 配送希望日を未選択とするブランクが追加さてているので「 + 1」しています
      available_delivery_date.count.should == available_delivery_date_count + 1
    end

    it "配送希望時間の一覧を表示する（引数に国を指定していない場合は、「日本」の設定が返却される）" do
      distributor_id = @distributor.id

      order = OrderForm.new()
      available_delivery_time = order.available_delivery_time
      available_delivery_time.each do |dt|
        dt.distributor_id.should == distributor_id
        dt.displays.should_not be_empty
      end
    end
  end

end
