# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe PointSetting do
fixtures :point_settings
  describe "区分値テスト" do
    it "ポイント制度利用区分 取得テスト" do
      PointSetting.available_flg_options.should == [
          ['使用しない', false],
          ['使用する', true]
      ]
    end

    it "有効期限有無区分 取得テスト" do
      PointSetting.terminable_flg_options.should == [
          ['なし', false],
          ['あり', true]
      ]
    end

    it "有効期限基準日区分 取得テスト" do
      PointSetting.term_div_options.should == [
          ['最終ポイント発行日', '0'],
          ['日付指定',  '99']
      ]
    end

    it "有効期間区分 取得テスト" do
      PointSetting.point_period_div_options.should == [
          ['1ヶ月', '1'],
          ['6ヶ月',  '6'],
          ['12ヶ月', '12'],
          ['24ヶ月',  '24']
      ]
    end
  end

  describe "バリデーションテスト" do
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.term_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:term_div).should == "を入力してください。"
    end
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.point_period_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:point_period_div).should == "を入力してください。"
    end
    it "必須項目テスト term_div" do
      point_setting = PointSetting.find(:first)
      point_setting.available_flg = true
      point_setting.term_div = ""
      point_setting.should_not be_valid
      point_setting.errors.on(:term_div).should == "を入力してください。"
    end
  end
  describe "有効期限表示テスト" do
    it "ポイント発行なしの場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = false
      term_day = point_setting.create_term_date(Date.today)
      term_day.should == nil
    end
    it "有効期限無しの場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = false
      term_day = point_setting.create_term_date(Date.today)
      term_day.should == nil
    end
    it "最終ポイント付与の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:last_point)
      day = Date.strptime('2008/03/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2008/04/26'
    end
    it "日付指定：期限が今年の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:manual)
      point_setting.term_day = '31'
      point_setting.term_month = '03'
      day = Date.strptime('2008/03/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2008/03/31'
    end
    it "日付指定：期限が来年の場合" do
      point_setting = PointSetting.new()
      point_setting.available_flg = true
      point_setting.terminable_flg = true
      point_setting.point_period_div = 1
      point_setting.term_div = PointSetting.term_div_id_by_key(:manual)
      point_setting.term_day = '31'
      point_setting.term_month = '03'
      day = Date.strptime('2008/04/26', "%Y/%m/%d")
      term_day = point_setting.create_term_date(day)
      term_day.strftime('%Y/%m/%d').should == '2009/03/31'
    end
  end
end
