require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe SpsParameterHistory do
  fixtures :sps_parameter_histories


  it "should create a new instance given valid attributes" do
    @valid_attributes = {
      :pay_method => 'pay_method',
      :merchant_id => '111',
      :service_id => '222',
      :cust_code => 'cust_code',
      :sps_order_id => 'order_id',
      :item_id => 'item_id',
      :amount => 333,
      :pay_type => 4,
      :service_type => 5,
      :request_date => 20091212334455,
      :payment_state_div => 6
    }
    SpsParameterHistory.create!(@valid_attributes)
  end

  describe "test create_from_order" do
    it "should create (when free_csv is nil)" do
      order_shipping_1 = OrderShipping.new
      order_shipping_1.total_amount = 5000
      order_shipping_1.shipping_cost = 1000

      order_shipping_2 = OrderShipping.new
      order_shipping_2.total_amount = 1111
      order_shipping_2.shipping_cost = 111

      order = Order.new
      order.order_shippings << order_shipping_1
      order.order_shippings << order_shipping_2

      sps_parameter_history = SpsParameterHistory.create_from_order(order)

      sps_parameter_history.id.nil?.should == false
      sps_parameter_history.pay_method.should == 'alipay'
      sps_parameter_history.merchant_id.should == EcRider.config.sps_merchant_id
      sps_parameter_history.service_id.should == EcRider.config.sps_service_id
      sps_parameter_history.cust_code.should == 'cust' + sps_parameter_history.id.to_s
      sps_parameter_history.sps_cust_no.should == nil
      sps_parameter_history.sps_payment_no.should == nil
      sps_parameter_history.sps_order_id.should == 'order' + sps_parameter_history.id.to_s
      sps_parameter_history.item_id.should == 'item' +  sps_parameter_history.id.to_s
      sps_parameter_history.pay_item_id.should == nil
      sps_parameter_history.item_name.should == nil
      sps_parameter_history.tax.should == nil
      sps_parameter_history.amount.should == 7222
      sps_parameter_history.pay_type.should == 0
      sps_parameter_history.auto_charge_type.should == nil
      sps_parameter_history.service_type.should == 0
      sps_parameter_history.div_settele.should == nil
      sps_parameter_history.last_charge_month.should == nil
      sps_parameter_history.camp_type.should == nil
      sps_parameter_history.tracking_id.should == nil
      sps_parameter_history.terminal_type.should == 0
      sps_parameter_history.success_url.should == EcRider.config.sps_success_url
      sps_parameter_history.cancel_url.should == EcRider.config.sps_cancel_url
      sps_parameter_history.error_url.should == EcRider.config.sps_error_url
      sps_parameter_history.pagecon_url.should == EcRider.config.sps_pagecon_url
      sps_parameter_history.free1.should == nil
      sps_parameter_history.free2.should == nil
      sps_parameter_history.free3.should == nil
      sps_parameter_history.free_csv.should == nil
      sps_parameter_history.request_date.to_s.length.should == 14
      sps_parameter_history.limit_second.should == EcRider.config.sps_limit_second
      sps_parameter_history.sps_hashcode.blank?.should == false
      sps_parameter_history.payment_state_div.should == 1
      sps_parameter_history.order_id.should == nil
    end

    it "should create (when free_csv is not nil)" do
      order_shipping_1 = OrderShipping.new
      order_shipping_1.total_amount = 5000
      order_shipping_1.shipping_cost = 1000

      order_shipping_2 = OrderShipping.new
      order_shipping_2.total_amount = 1111
      order_shipping_2.shipping_cost = 111

      order = Order.new
      order.order_shippings << order_shipping_1
      order.order_shippings << order_shipping_2

      ms_shop_setting = ShopSetting.new
      ms_shop_setting.setting_div = '0017'
      ms_shop_setting.value = '受理了订购'
      ms_shop_setting.save!

      sps_parameter_history = SpsParameterHistory.create_from_order(order)

      sps_parameter_history.id.nil?.should == false
      sps_parameter_history.pay_method.should == 'alipay'
      sps_parameter_history.merchant_id.should == EcRider.config.sps_merchant_id
      sps_parameter_history.service_id.should == EcRider.config.sps_service_id
      sps_parameter_history.cust_code.should == 'cust' + sps_parameter_history.id.to_s
      sps_parameter_history.sps_cust_no.should == nil
      sps_parameter_history.sps_payment_no.should == nil
      sps_parameter_history.sps_order_id.should == 'order' + sps_parameter_history.id.to_s
      sps_parameter_history.item_id.should == 'item' +  sps_parameter_history.id.to_s
      sps_parameter_history.pay_item_id.should == nil
      sps_parameter_history.item_name.should == nil
      sps_parameter_history.tax.should == nil
      sps_parameter_history.amount.should == 7222
      sps_parameter_history.pay_type.should == 0
      sps_parameter_history.auto_charge_type.should == nil
      sps_parameter_history.service_type.should == 0
      sps_parameter_history.div_settele.should == nil
      sps_parameter_history.last_charge_month.should == nil
      sps_parameter_history.camp_type.should == nil
      sps_parameter_history.tracking_id.should == nil
      sps_parameter_history.terminal_type.should == 0
      sps_parameter_history.success_url.should == EcRider.config.sps_success_url
      sps_parameter_history.cancel_url.should == EcRider.config.sps_cancel_url
      sps_parameter_history.error_url.should == EcRider.config.sps_error_url
      sps_parameter_history.pagecon_url.should == EcRider.config.sps_pagecon_url
      sps_parameter_history.free1.should == nil
      sps_parameter_history.free2.should == nil
      sps_parameter_history.free3.should == nil
      sps_parameter_history.free_csv.should == "aXRlbV9uYW1lPcrcwO3By7apubo="
      sps_parameter_history.request_date.to_s.length.should == 14
      sps_parameter_history.limit_second.should == EcRider.config.sps_limit_second
      sps_parameter_history.sps_hashcode.blank?.should == false
      sps_parameter_history.payment_state_div.should == 1
      sps_parameter_history.order_id.should == nil
    end
  end

  describe "test calc_sps_hashcode" do
    it "should correct" do
      sps_parameter_history = SpsParameterHistory.new
      sps_parameter_history.pay_method = 'alipay'
      sps_parameter_history.merchant_id = '12345'
      sps_parameter_history.service_id = '123'
      sps_parameter_history.cust_code = 'cust41'
      sps_parameter_history.sps_cust_no = nil
      sps_parameter_history.sps_payment_no = nil
      sps_parameter_history.order_id = 'order41'
      sps_parameter_history.item_id = 'item41'
      sps_parameter_history.pay_item_id = nil
      sps_parameter_history.item_name = nil
      sps_parameter_history.tax = nil
      sps_parameter_history.amount = 7222
      sps_parameter_history.pay_type = 0
      sps_parameter_history.auto_charge_type = nil
      sps_parameter_history.service_type = 0
      sps_parameter_history.div_settele = nil
      sps_parameter_history.last_charge_month = nil
      sps_parameter_history.camp_type = nil
      sps_parameter_history.tracking_id = nil
      sps_parameter_history.terminal_type = 0
      sps_parameter_history.success_url = 'http://localhost:3001/sps_payment/complete_register'
      sps_parameter_history.cancel_url = 'http://localhost:3001/cash_register/send_one_place'
      sps_parameter_history.error_url = 'http://localhost:3001/cash_register/send_one_place'
      sps_parameter_history.pagecon_url = 'http://localhost:3001/sps_payment/receive_notification'
      sps_parameter_history.free1 = nil
      sps_parameter_history.free2 = nil
      sps_parameter_history.free3 = nil
      sps_parameter_history.free_csv = nil
      sps_parameter_history.request_date = 20090804152106
      sps_parameter_history.limit_second = 1000

      sps_parameter_history.calc_sps_hashcode

      sps_parameter_history.sps_hashcode.should == '95447095383ee98ec0cd7d12d62e8ffb267f4636'
    end
  end

  describe "test update_when_pay_success" do
    it "should update" do
      order_id = 999
      sps_order_id = '00000000000000000000000000000000000002'
      sps_parameter_history = SpsParameterHistory.find_by_sps_order_id(sps_order_id)
      sps_parameter_history.update_when_pay_success(order_id)
      sps_parameter_history.id.should == 2
      sps_parameter_history.lock_version.should == 1
      sps_parameter_history.payment_state_div.should == 2
      sps_parameter_history.order_id.should == 999
    end
  end

  describe "create_payment_content" do
    # Alipay決済ではsps_parameter_history_itemsを利用しないため
    # sps_parameter_history_itemsを作成しない方針になりました。
    # 現在は必ず商品履歴が０となるため、
    # テストをコメントアウトします。
#    it "商品履歴が１つの場合" do
#      sps_parameter_history = SpsParameterHistory.find(3)
#      string = sps_parameter_history.create_payment_content
#      string.should == <<-EOS
#pay_method : alipay
#merchant_id : 12345
#service_id : 123
#cust_code : 0000000001
#sps_cust_no : 1
#sps_payment_no : 1
#order_id : 00000000000000000000000000000000000001
#item_id : 00000000000000000000000000000001
#pay_item_id : 1
#item_name : ITEM
#tax : 1
#amount : 1050
#pay_type : 0
#auto_charge_type : 1
#service_type : 0
#div_settele : 1
#last_charge_month : 1
#camp_type : 1
#tracking_id : 1
#terminal_type : 0
#success_url : https://localhost:3001/sps_payment/complete_register
#cancel_url : https://localhost:3001/cash_register/send_one_place
#error_url : https://localhost:3001/cash_register/send_one_place
#pagecon_url : https://localhost:3001/sps_payment/receive_notification
#free1 : free1
#free2 : free2
#free3 : free3
#free_csv : =?gbk?B?YQ==?=
#dtl_rowno :
#dtl_item_id :
#dtl_item_name :
#dtl_item_count :
#dtl_tax :
#dtl_amount :
#dtl_free1 :
#dtl_free2 :
#dtl_free3 :
#request_date : 20090810010000
#limit_second : 1800
#      EOS
#    end
#    it "商品履歴が２つの場合" do
#      sps_parameter_history = SpsParameterHistory.find(4)
#      string = sps_parameter_history.create_payment_content
#      string.should == <<-EOS
#pay_method :
#merchant_id :
#service_id :
#cust_code :
#sps_cust_no :
#sps_payment_no :
#order_id :
#item_id :
#pay_item_id :
#item_name :
#tax :
#amount :
#pay_type :
#auto_charge_type :
#service_type :
#div_settele :
#last_charge_month :
#camp_type :
#tracking_id :
#terminal_type :
#success_url :
#cancel_url :
#error_url :
#pagecon_url :
#free1 :
#free2 :
#free3 :
#free_csv :
#dtl_rowno :
#dtl_item_id :
#dtl_item_name :
#dtl_item_count :
#dtl_tax :
#dtl_amount :
#dtl_free1 :
#dtl_free2 :
#dtl_free3 :
#dtl_rowno :
#dtl_item_id :
#dtl_item_name :
#dtl_item_count :
#dtl_tax :
#dtl_amount :
#dtl_free1 :
#dtl_free2 :
#dtl_free3 :
#request_date :
#limit_second :
#sps_hashcode :
#      EOS
#    end
    it "商品履歴が０の場合(在庫切れ)" do
      sps_parameter_history = SpsParameterHistory.find(3)
      string = sps_parameter_history.create_payment_content
      string.should == <<-EOS
pay_method : alipay
merchant_id : 12345
service_id : 123
cust_code : 0000000001
sps_cust_no : 1
sps_payment_no : 1
sps_order_id : 00000000000000000000000000000000000003
item_id : 00000000000000000000000000000001
pay_item_id : 1
item_name : ITEM
tax : 1
amount : 1050
pay_type : 0
auto_charge_type : 1
service_type : 0
div_settele : 1
last_charge_month : 1
camp_type : 1
tracking_id : 1
terminal_type : 0
success_url : https://localhost:3001/sps_payment/complete_register
cancel_url : https://localhost:3001/cash_register/send_one_place
error_url : https://localhost:3001/cash_register/send_one_place
pagecon_url : https://localhost:3001/sps_payment/receive_notification
free1 : free1
free2 : free2
free3 : free3
free_csv : =?gbk?B?YQ==?=
request_date : 20090810010000
limit_second : 1800
      EOS
    end
  end

  describe "テスト注文、本番注文切り替え" do
    before(:each) do
      @valid_attributes = {
        :pay_method => 'pay_method',
        :merchant_id => '111',
        :service_id => '222',
        :cust_code => 'cust_code',
        :sps_order_id => 'order_id',
        :item_id => 'item_id',
        :amount => 333,
        :pay_type => 4,
        :service_type => 5,
        :request_date => 20091212334455,
        :payment_state_div => 6
      }
    end
    it "ec_adminの場合　本番データ" do
      sps_parameter_history = SpsParameterHistory.new(@valid_attributes)
      sps_parameter_history.save!
      sps_parameter_history.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      sps_parameter_history = SpsParameterHistory.new(@valid_attributes)
      sps_parameter_history.test_flg = true
      sps_parameter_history.save!
      sps_parameter_history.test_flg.should be_true
    end

    it "ec_frontの場合" do
      sps_parameter_history = SpsParameterHistory.new(@valid_attributes)
      sps_parameter_history.save!
      sps_parameter_history.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_parameter_history = SpsParameterHistory.new(@valid_attributes)
      sps_parameter_history.save!
      sps_parameter_history.test_flg.should be_true
    end

    it "ec_testの場合　更新" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_parameter_history = SpsParameterHistory.new(@valid_attributes)
      sps_parameter_history.test_flg = true
      sps_parameter_history.save!
      sps_parameter_history.test_flg.should be_true
    end
  end

end
