require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe SpsResultHistory do
  fixtures :sps_parameter_histories, :sps_result_histories, :sps_user_temporaries, :sps_order_temporaries, :sps_user_address_temporaries, :sps_user_email_temporaries
  
  it "status_div means kind of status for online payment." do
    SpsResultHistory.status_div_options.should == [
      ["対応必要なし", '0'], 
      ["未対応", '1'], 
      ["返答待ち", '2'], 
      ["対応完了", '3']
    ]
    SpsResultHistory.status_div_id_by_key(:no_need).should == "0"
    SpsResultHistory.status_div_id_by_key(:not_yet).should == "1"
    SpsResultHistory.status_div_id_by_key(:waiting).should == "2"
    SpsResultHistory.status_div_id_by_key(:complete).should == "3"
  end
  
  describe "selectable_attr ecr_result" do
    it "selectable_attr ecr_result" do
      srh = SpsResultHistory.new(:ecr_result_key => :ng)
      srh.ecr_result.should == 'NG'
      srh.ecr_result_key.should == :ng
      srh.ecr_result_name.should == 'OK,'
      srh.ecr_result_entry[:default_status_div_key].should == :no_need

      srh.ecr_result = 'ECR_NG_CHECK_SUM'
      srh.ecr_result.should == 'ECR_NG_CHECK_SUM'
      srh.ecr_result_key.should == :ng_check_sum
      srh.ecr_result_name.should == 'NG,購入要求情報のチェックサムが不正です。'
      srh.ecr_result_entry[:default_status_div_key].should == :no_need

      srh.ecr_result_key = :ng_request
      srh.ecr_result.should == 'ECR_NG_REQUEST'
      srh.ecr_result_key.should == :ng_request
      srh.ecr_result_name.should == 'NG,購入要求情報の内容が不正です。'
      srh.ecr_result_entry[:default_status_div_key].should == :not_yet

      srh.ecr_result_key = :ng_out_of_stock
      srh.ecr_result.should == 'ECR_NG_OUT_OF_STOCK'
      srh.ecr_result_key.should == :ng_out_of_stock
      srh.ecr_result_name.should == 'NG,在庫不足です。'
      srh.ecr_result_entry[:default_status_div_key].should == :not_yet

      srh.ecr_result_key = :ng_other_exception
      srh.ecr_result.should == 'ECR_NG_OTHER_EXCEPTION'
      srh.ecr_result_key.should == :ng_other_exception
      srh.ecr_result_name.should == 'NG,予期せぬエラーです。'
      srh.ecr_result_entry[:default_status_div_key].should == :not_yet

      srh.ecr_result_key = :ok
      srh.ecr_result.should == 'OK'
      srh.ecr_result_key.should == :ok
      srh.ecr_result_name.should == 'OK,'
      srh.ecr_result_entry[:default_status_div_key].should == :no_need
    end
  end
  
  it "should create a new instance given valid attributes" do
    valid_attributes = {
      :sps_parameter_history_id => 111,
      :pay_method => 'pay_method',
      :merchant_id => '111',
      :service_id => '222',
      :cust_code => 'cust_code',
      :sps_order_id => 'order_id',
      :item_id => 'item_id',
      :amount => 333,
      :pay_type => 4,
      :service_type => 5,
      :request_date => 20091212334455,
      :res_pay_method => 'res_pay_method',
      :res_result => 'OK',
      :res_tracking_id => 'res_tracking',
      :res_date => 20091212445533
    }
    SpsResultHistory.create!(valid_attributes)
  end

  describe "test create_from_params" do
    before(:each) do
      @post_params = {
        'pay_method' => 'alipay',
        'merchant_id' => '12345',
        'service_id' => '123',
        'cust_code' => '0000000002',
        'sps_cust_no' => nil,
        'sps_payment_no' => nil,
        'order_id' => '00000000000000000000000000000000000002',
        'item_id' => '00000000000000000000000000000002',
        'pay_item_id' => nil,
        'item_name' => 'ITEM',
        'tax' => nil,
        'amount' => 6145,
        'pay_type' => 0,
        'auto_charge_type' => nil,
        'service_type' => 0,
        'div_settele' => nil,
        'last_charge_month' => nil,
        'camp_type' => nil,
        'tracking_id' => nil,
        'terminal_type' => 0,
        'free1' => nil,
        'free2' => nil,
        'free3' => nil,
        'request_date' => 20090810020000,
        'res_pay_method' => 'alipay',
        'res_result' => 'OK',
        'res_tracking_id' => 'res_tracking',
        'res_sps_cust_no' => 1111,
        'res_sps_payment_no' => 2222,
        'res_payinfo_key' => 'abcdefgh',
        'res_payment_date' => 20090810023344,
        'res_err_code' => 'xxx',
        'res_date' => 20090810024455,
        'limit_second' => 1000,
        'sps_hashcode' => 'sps_hashcode'
      }
    end

    it "should create (sps_parameter_history_id is not null" do
      sps_result_history = SpsResultHistory.create_from_params(@post_params)
      sps_result_history.id.should_not be_nil
      sps_result_history.sps_parameter_history_id.should == 2
      sps_result_history.pay_method.should == 'alipay'
      sps_result_history.merchant_id.should == '12345'
      sps_result_history.service_id.should == '123'
      sps_result_history.cust_code.should == '0000000002'
      sps_result_history.sps_cust_no.should == nil
      sps_result_history.sps_payment_no.should == nil
      sps_result_history.sps_order_id.should == '00000000000000000000000000000000000002'
      sps_result_history.item_id.should == '00000000000000000000000000000002'
      sps_result_history.pay_item_id.should == nil
      sps_result_history.item_name.should == 'ITEM'
      sps_result_history.tax.should == nil
      sps_result_history.amount.should == 6145
      sps_result_history.pay_type.should == 0
      sps_result_history.auto_charge_type.should == nil
      sps_result_history.service_type.should == 0
      sps_result_history.div_settele.should == nil
      sps_result_history.last_charge_month.should == nil
      sps_result_history.camp_type.should == nil
      sps_result_history.tracking_id.should == nil
      sps_result_history.terminal_type.should == 0
      sps_result_history.free1.should == nil
      sps_result_history.free2.should == nil
      sps_result_history.free3.should == nil
      sps_result_history.request_date.should == 20090810020000
      sps_result_history.res_pay_method.should == 'alipay'
      sps_result_history.res_result.should == 'OK'
      sps_result_history.res_tracking_id.should == 'res_tracking'
      sps_result_history.res_sps_cust_no.should == 1111
      sps_result_history.res_sps_payment_no.should == 2222
      sps_result_history.res_payinfo_key.should == 'abcdefgh'
      sps_result_history.res_payment_date.should == 20090810023344
      sps_result_history.res_err_code.should == 'xxx'
      sps_result_history.res_date.should == 20090810024455
      sps_result_history.limit_second.should == 1000
      sps_result_history.sps_hashcode.should == 'sps_hashcode'
    end

    it "should create (sps_parameter_history_id is null" do
      @post_params['order_id'] = '0000000000000000000000000000000000000x'
      sps_result_history = SpsResultHistory.create_from_params(@post_params)
      sps_result_history.id.should_not be_nil
      sps_result_history.sps_parameter_history_id.should == nil
      sps_result_history.pay_method.should == 'alipay'
      sps_result_history.merchant_id.should == '12345'
      sps_result_history.service_id.should == '123'
      sps_result_history.cust_code.should == '0000000002'
      sps_result_history.sps_cust_no.should == nil
      sps_result_history.sps_payment_no.should == nil
      sps_result_history.sps_order_id.should == '0000000000000000000000000000000000000x'
      sps_result_history.item_id.should == '00000000000000000000000000000002'
      sps_result_history.pay_item_id.should == nil
      sps_result_history.item_name.should == 'ITEM'
      sps_result_history.tax.should == nil
      sps_result_history.amount.should == 6145
      sps_result_history.pay_type.should == 0
      sps_result_history.auto_charge_type.should == nil
      sps_result_history.service_type.should == 0
      sps_result_history.div_settele.should == nil
      sps_result_history.last_charge_month.should == nil
      sps_result_history.camp_type.should == nil
      sps_result_history.tracking_id.should == nil
      sps_result_history.terminal_type.should == 0
      sps_result_history.free1.should == nil
      sps_result_history.free2.should == nil
      sps_result_history.free3.should == nil
      sps_result_history.request_date.should == 20090810020000
      sps_result_history.res_pay_method.should == 'alipay'
      sps_result_history.res_result.should == 'OK'
      sps_result_history.res_tracking_id.should == 'res_tracking'
      sps_result_history.res_sps_cust_no.should == 1111
      sps_result_history.res_sps_payment_no.should == 2222
      sps_result_history.res_payinfo_key.should == 'abcdefgh'
      sps_result_history.res_payment_date.should == 20090810023344
      sps_result_history.res_err_code.should == 'xxx'
      sps_result_history.res_date.should == 20090810024455
      sps_result_history.limit_second.should == 1000
      sps_result_history.sps_hashcode.should == 'sps_hashcode'
    end
  end

  describe "test request_ok?" do
    before(:each) do
      @sps_result_history_attributes = {
        :pay_method => 'alipay',
        :merchant_id => '12345',
        :service_id => '123',
        :cust_code => '0000000002',
        :sps_cust_no => nil,
        :sps_payment_no => nil,
        :sps_order_id => '00000000000000000000000000000000000002',
        :item_id => '00000000000000000000000000000002',
        :pay_item_id => nil,
        :item_name => 'ITEM',
        :tax => nil,
        :amount => 6145,
        :pay_type => 0,
        :auto_charge_type => nil,
        :service_type => 0,
        :div_settele => nil,
        :last_charge_month => nil,
        :camp_type => nil,
        :tracking_id => nil,
        :terminal_type => 0,
        :free1 => nil,
        :free2 => nil,
        :free3 => nil,
        :request_date => 20090810020000
      }
    end
    it "should request ok" do
      sps_result_history = SpsResultHistory.new(@sps_result_history_attributes)
      sps_parameter_history = SpsParameterHistory.find(2)
      # ひもづける
      sps_result_history.sps_parameter_history_id = sps_parameter_history.id
      sps_result_history.request_ok?.should be_true
    end

    it "should request ng when no sps_payment_history" do
      sps_result_history = SpsResultHistory.new(@sps_result_history_attributes)
      sps_parameter_history = SpsParameterHistory.find(2)
      # ひもづけない
      sps_result_history.request_ok?.should be_false
    end

    it "should request ng" do
      @sps_result_history_attributes[:pay_type] = 1
      sps_result_history = SpsResultHistory.new(@sps_result_history_attributes)
      sps_parameter_history = SpsParameterHistory.find(2)
      # ひもづける
      sps_result_history.sps_parameter_history_id = sps_parameter_history.id
      sps_result_history.request_ok?.should be_false
    end
  end

  describe "test sps_hashcode_ok?" do
    before(:each) do
      @sps_result_history_attributes = {
        :sps_parameter_history_id => 999,
        :pay_method => 'alipay',
        :merchant_id => '12345',
        :service_id => '123',
        :cust_code => '0000000002',
        :sps_cust_no => nil,
        :sps_payment_no => nil,
        :sps_order_id => '00000000000000000000000000000000000002',
        :item_id => '00000000000000000000000000000002',
        :pay_item_id => nil,
        :item_name => 'ITEM',
        :tax => nil,
        :amount => 6145,
        :pay_type => 0,
        :auto_charge_type => nil,
        :service_type => 0,
        :div_settele => nil,
        :last_charge_month => nil,
        :camp_type => nil,
        :tracking_id => nil,
        :terminal_type => 0,
        :free1 => nil,
        :free2 => nil,
        :free3 => nil,
        :request_date => 20090810020000,
        :res_pay_method => 'alipay',
        :res_result => 'res_tracking',
        :res_sps_cust_no => 1111,
        :res_sps_payment_no => 2222,
        :res_payinfo_key => 'abcdefgh',
        :res_payment_date => 20090810023344,
        :res_err_code => 'xxx',
        :res_date => 20090810024455,
        :limit_second => 1000,
        :sps_hashcode => 'sps_hashcode'
      }
    end
    it "should hashcode ok" do
      @sps_result_history_attributes[:sps_hashcode] = '01a1d77e4922a94dcf02722e825bc3b8214c2c11'
      sps_result_history = SpsResultHistory.new(@sps_result_history_attributes)
      sps_result_history.sps_hashcode_ok?.should be_true
    end

    it "should hashcode ng" do
      sps_result_history = SpsResultHistory.new(@sps_result_history_attributes)
      sps_result_history.sps_hashcode_ok?.should be_false
    end
  end

  describe "test update_ecr_result" do
    it "when :ng" do
      sps_result_history = SpsResultHistory.find(1)
      sps_result_history.res_result = 'XX'
      sps_result_history.update_ecr_result(:ng)
      sps_result_history.ecr_result.should == 'XX'
      sps_result_history.status_div_key.should == :no_need
      sps_result_history.ecr_response.should == 'OK,'
    end

    it "when not :ng" do
      sps_result_history = SpsResultHistory.find(1)
      sps_result_history.res_result = 'XX'
      sps_result_history.update_ecr_result(:ng_check_sum)
      sps_result_history.ecr_result.should == 'ECR_NG_CHECK_SUM'
      sps_result_history.status_div_key.should == :no_need
      sps_result_history.ecr_response.should == 'NG,購入要求情報のチェックサムが不正です。'
    end
    
    it "when key is :ng_request" do
      sps_result_history = SpsResultHistory.find(1)
      sps_result_history.res_result = 'XX'
      sps_result_history.update_ecr_result(:ng_request)
      sps_result_history.ecr_result.should == 'ECR_NG_REQUEST'
      sps_result_history.status_div_key.should == :not_yet
      sps_result_history.ecr_response.should == 'NG,購入要求情報の内容が不正です。'
    end
  end

  describe "test order" do
    it "get order" do
      sps_result_history = SpsResultHistory.find(1)
      sps_result_history.order.id.should == 1
    end
  end
  
  describe "test user" do
    it "get user" do
      sps_result_history = SpsResultHistory.find(2)
      sps_result_history.user_id.should == 1003
      
      user = sps_result_history.user
      user.is_a?(User).should be_true
      user.id.should == 1003
      user.fullname.should == "佐藤 花子"
      user.user_email.email.should == "email_address_01003@docomo.ne.jp"

      SpsOrderTemporary.update(1, :user_id => nil)
      
      sps_result_history = SpsResultHistory.find(2)
      sps_result_history.user_id.should == 1003
      
      user = sps_result_history.user
      user.is_a?(User).should be_true
      user.id.should == 1003
      user.fullname.should == "佐藤 花子"
      user.user_email.email.should == "email_address_01003@docomo.ne.jp"
    end
  end
  describe "テスト注文、本番注文切り替え" do
    before(:each) do
      @valid_attributes = {
        :sps_parameter_history_id => 111,
        :pay_method => 'pay_method',
        :merchant_id => '111',
        :service_id => '222',
        :cust_code => 'cust_code',
        :sps_order_id => 'order_id',
        :item_id => 'item_id',
        :amount => 333,
        :pay_type => 4,
        :service_type => 5,
        :request_date => 20091212334455,
        :res_pay_method => 'res_pay_method',
        :res_result => 'OK',
        :res_tracking_id => 'res_tracking',
        :res_date => 20091212445533
      }
    end
    it "ec_adminの場合　本番データ" do
      sps_result_history = SpsResultHistory.new(@valid_attributes)
      sps_result_history.save!
      sps_result_history.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      sps_result_history = SpsResultHistory.new(@valid_attributes)
      sps_result_history.test_flg = true
      sps_result_history.save!
      sps_result_history.test_flg.should be_true
    end

    it "ec_frontの場合" do
      sps_result_history = SpsResultHistory.new(@valid_attributes)
      sps_result_history.save!
      sps_result_history.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_result_history = SpsResultHistory.new(@valid_attributes)
      sps_result_history.save!
      sps_result_history.test_flg.should be_true
    end

    it "ec_testの場合　更新" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_result_history = SpsResultHistory.new(@valid_attributes)
      sps_result_history.test_flg = true
      sps_result_history.save!
      sps_result_history.test_flg.should be_true
    end
  end
end
