require File.expand_path(File.dirname(__FILE__) + '/../spec_helper')

describe SpsUserTemporary do
  fixtures :sps_user_temporaries, :sps_user_email_temporaries, :sps_user_address_temporaries

  it "should create a new instance given valid attributes" do
    valid_attributes = {
      :sps_order_id => 'sps_order_id',
      :name_last => 'kanji_name_last',
      :name_first => 'kanji_name_first',
      :kana_name_last => 'kana_name_last',
      :kana_name_first => 'kana_name',
      :tel_num => 'tel_num',
      :member_div => 'userd'
    }
    SpsUserTemporary.create!(valid_attributes)
  end

  describe "test create_from_user" do
    it "should create" do
      sps_order_id = 'sps_order_id'

      # Userインスタンス作成
      user_attributes = {
        :user_code => 'user_code',
        :name_last => 'kanji_name_last',
        :name_first => 'kanji_name_first',
        :kana_name_last => 'kana_family_name',
        :kana_name_first => 'kana_name',
        :tel_num => 'tel_num',
        :birth_year => 2009,
        :birth_month => 1,
        :birth_day => 2,
        :resigened_at => '2007-12-10 11:09:38',
        :sex => 'sex',
        :member_div => 'user',
        :login => 'login',
        :crypted_password => 'crypted_password',
        :salt => 'salt',
        :remember_answer => 'remember_answer',
        :remember_token => 'remember_token',
        :remember_token_expires_at => 'remember_token_expires_at',
        :regist_div => 'regis',
        :job_div => 'job',
        :dm_send_div => 4,
        :remember_question => 'remember_question',
        :member_category_div => '001',
        :password => 'password',
        :pc_mail_magazine_send_div => "3",
        :mobile_mail_magazine_send_div => "3",
        :test_flg => 1,
        :user_rank_id => 3,
        :login_challenge_count => 3,
        :locked_at => '2009-11-12 22:33:44'
      }
      user = User.new(user_attributes)

      # EmailAddressインスタンス作成
      email_address_attributes = {
        :user_id => nil,
        :email => 'email',
        :pc_mobile_div => 'pc',
        :undeliverable_flg => 1,
        :mail_address_pc_mobile_div => 'mail_address_pc'
      }
      email_address = UserEmail.new(email_address_attributes)

      # Addressインスタンス作成
      address_attributes1 = {
        :user_id => nil,
        :name_last => 'kanji_name_last1',
        :name_first => 'kanji_name_first1',
        :kana_name_last => 'kana_family_name1',
        :kana_name_first => 'kana_name_first1',
        :tel_num => 'tel_num1',
        :zip_code => 'zip_code1',
        :address_1 => 'address_1_1',
        :address_2 => 'address_2_1',
        :address_3 => 'address_3_1',
        :address_div => 'address_div1',
        :country_code => 'co1',
        :locale => 'loca1'
      }
      address1 = UserAddress.new(address_attributes1)
      address_attributes2 = {
        :user_id => nil,
        :name_last => 'kanji_name_last2',
        :name_first => 'kanji_name_first2',
        :kana_name_last => 'kana_family_name2',
        :kana_name_first => 'kana_name_first2',
        :tel_num => 'tel_num2',
        :zip_code => 'zip_code2',
        :address_1 => 'address_1_2',
        :address_2 => 'address_2_2',
        :address_3 => 'address_3_2',
        :address_div => 'address_div2',
        :country_code => 'co2',
        :locale => 'loca2'
      }
      address2 = UserAddress.new(address_attributes2)

      # 関連を生成
      user.user_email = email_address
      user.user_addresses << address1
      user.user_addresses << address2

      # テスト対象のメソッドを実行
      sps_user_temporary = SpsUserTemporary.create_from_user(user, sps_order_id)

      # 属性をチェック
      sps_user_temporary.id.should_not be_nil
      sps_user_temporary.sps_order_id.should == 'sps_order_id'
      sps_user_temporary.user_code.should == 'user_code'
      sps_user_temporary.name_last.should == 'kanji_name_last'
      sps_user_temporary.name_first.should == 'kanji_name_first'
      sps_user_temporary.kana_name_last.should == 'kana_family_name'
      sps_user_temporary.kana_name_first.should == 'kana_name'
      sps_user_temporary.tel_num.should == 'tel_num'
      sps_user_temporary.birth_year.should == 2009
      sps_user_temporary.birth_month.should == 1
      sps_user_temporary.birth_day.should == 2
      sps_user_temporary.resigened_at.strftime('%Y-%m-%d %H:%M:%S').should == '2007-12-10 11:09:38'
      sps_user_temporary.sex.should == 'sex'
      sps_user_temporary.member_div.should == 'user'
      sps_user_temporary.login.should == 'login'
      sps_user_temporary.crypted_password.should_not be_nil
      sps_user_temporary.salt.should_not be_nil
      sps_user_temporary.remember_answer.should == 'remember_answer'
      sps_user_temporary.remember_token.should == 'remember_token'
      sps_user_temporary.remember_token_expires_at.should == 'remember_token_expires_at'
      sps_user_temporary.regist_div.should == 'regis'
      sps_user_temporary.job_div.should == 'job'
      sps_user_temporary.dm_send_div.should == 4
      sps_user_temporary.remember_question.should == 'remember_question'
      sps_user_temporary.member_category_div.should == '001'
      sps_user_temporary.pc_mail_magazine_send_div.should == '3'
      sps_user_temporary.mobile_mail_magazine_send_div.should == '3'
      sps_user_temporary.test_flg.should == true
      sps_user_temporary.user_rank_id.should == 3
      sps_user_temporary.login_challenge_count.should == 3
      sps_user_temporary.locked_at.strftime('%Y-%m-%d %H:%M:%S').should == '2009-11-12 22:33:44'

      sps_emal_address_temporary = sps_user_temporary.sps_user_email_temporary
      sps_emal_address_temporary.id.should_not be_nil
      sps_emal_address_temporary.sps_user_temporary_id.should == sps_user_temporary.id
      sps_emal_address_temporary.email.should == 'email'
      sps_emal_address_temporary.pc_mobile_div.should == 'pc'
      sps_emal_address_temporary.undeliverable_flg.should == 1
      sps_emal_address_temporary.mail_address_pc_mobile_div.should == 'mail_address_pc'

      sps_address_temporary1 = sps_user_temporary.sps_user_address_temporaries[0]
      sps_address_temporary1.id.should_not be_nil
      sps_address_temporary1.sps_user_temporary_id.should == sps_user_temporary.id
      sps_address_temporary1.name_last.should == 'kanji_name_last1'
      sps_address_temporary1.name_first.should == 'kanji_name_first1'
      sps_address_temporary1.kana_name_last.should == 'kana_family_name1'
      sps_address_temporary1.kana_name_first.should == 'kana_name_first1'
      sps_address_temporary1.tel_num.should == 'tel_num1'
      sps_address_temporary1.zip_code.should == 'zip_code1'
      sps_address_temporary1.address_1.should == 'address_1_1'
      sps_address_temporary1.address_2.should == 'address_2_1'
      sps_address_temporary1.address_3.should == 'address_3_1'
      sps_address_temporary1.address_div.should == 'address_div1'
      sps_address_temporary1.country_code.should == 'co1'
      sps_address_temporary1.locale.should == 'loca1'

      sps_address_temporary2 = sps_user_temporary.sps_user_address_temporaries[1]
      sps_address_temporary2.id.should_not be_nil
      sps_address_temporary2.sps_user_temporary_id.should == sps_user_temporary.id
      sps_address_temporary2.name_last.should == 'kanji_name_last2'
      sps_address_temporary2.name_first.should == 'kanji_name_first2'
      sps_address_temporary2.kana_name_last.should == 'kana_family_name2'
      sps_address_temporary2.kana_name_first.should == 'kana_name_first2'
      sps_address_temporary2.tel_num.should == 'tel_num2'
      sps_address_temporary2.zip_code.should == 'zip_code2'
      sps_address_temporary2.address_1.should == 'address_1_2'
      sps_address_temporary2.address_2.should == 'address_2_2'
      sps_address_temporary2.address_3.should == 'address_3_2'
      sps_address_temporary2.address_div.should == 'address_div2'
      sps_address_temporary2.country_code.should == 'co2'
      sps_address_temporary2.locale.should == 'loca2'
    end
  end

  describe "テスト注文、本番注文切り替え" do
    before(:each) do
      @valid_attributes = {
      :sps_order_id => 'sps_order_id',
      :name_last => 'kanji_name_last',
      :name_first => 'kanji_name_first',
      :kana_name_last => 'kana_name_last',
      :kana_name_first => 'kana_name',
      :tel_num => 'tel_num',
      :member_div => 'userd'
    }
    end
    it "ec_adminの場合　本番データ" do
      sps_user_temporary = SpsUserTemporary.new(@valid_attributes)
      sps_user_temporary.save!
      sps_user_temporary.test_flg.should be_nil
    end

    it "ec_adminの場合　テストデータ" do
      sps_user_temporary = SpsUserTemporary.new(@valid_attributes)
      sps_user_temporary.test_flg = true
      sps_user_temporary.save!
      sps_user_temporary.test_flg.should be_true
    end

    it "ec_frontの場合" do
      sps_user_temporary = SpsUserTemporary.new(@valid_attributes)
      sps_user_temporary.save!
      sps_user_temporary.test_flg.should be_nil
    end

    it "ec_testの場合" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_user_temporary = SpsUserTemporary.new(@valid_attributes)
      sps_user_temporary.save!
      sps_user_temporary.test_flg.should be_true
    end

    it "ec_testの場合　更新" do
      EcRider.config.stub!(:ec_test).and_return(true)
      sps_user_temporary = SpsUserTemporary.new(@valid_attributes)
      sps_user_temporary.test_flg = true
      sps_user_temporary.save!
      sps_user_temporary.test_flg.should be_true
    end
  end
end
