# -*- coding: utf-8 -*-
require File.dirname(__FILE__) + '/../spec_helper'

describe UserAddress do
  fixtures :user_addresses,:users

  before(:each) do
    @address = UserAddress.new
  end


  describe "Validation Japanese" do
    before do
      @param = UserAddress.find(1011).attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the name_first" do
      @param[:name_first] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the kana_name_last" do
      @param[:kana_name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_first" do
      @param[:kana_name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_last" do
      @param[:kana_name_last] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kana_name_first" do
      @param[:kana_name_first] = "abc"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = "abcdefg"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = "+81-00-0000-0000"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
  end

  describe "Validation English" do
    before do
      address = UserAddress.find(1012)
      address.address_div_key = :user
      @param = address.attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the zip_code at oversea" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the tel_num at oversea" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "Validation Japanese user's English address" do
    before do
      address = UserAddress.find(1012)
      address.address_div_key = :user_en
      @param = address.attributes.dup
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The required check of the name_last" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_last" do
      @param[:name_last] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the name_first" do
      @param[:name_first] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The required check of the zip_code at oversea" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the tel_num at oversea" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "The length check of the address_1" do
      @param[:address_1] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_2" do
      @param[:address_2] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "The length check of the address_3" do
      @param[:address_3] = "てすと"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "Validation Chinese" do
    before do
      @param = UserAddress.find(1011).attributes.dup
      @param[:country_code] = 'CN'
      @param[:name_last] = '支付宝'
      @param[:name_first] = '支付宝'
      @param[:address_1] = '北京市'
      @param[:address_2] = '朝阳区东三环北路38号'
      @param[:address_3] = '泰康金融大厦1座25层'
    end
    it "validation true" do
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the kanji_family_name" do
      @param[:name_last] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The required check of the kanji_name" do
      @param[:name_first] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_family_name with japanese" do
      @param[:name_last] = "中国語"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_name with japanese" do
      @param[:name_first] = "労働"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The format check of the kanji_family_name with english" do
      @param[:name_last] = "yamada"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The format check of the kanji_name with english" do
      @param[:name_first] = "taro"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The required check of the zip_code" do
      @param[:zip_code] = "abcdefg"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the tel_num" do
      @param[:tel_num] = "+81-00-0000-0000"
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
    it "The length check of the address_1" do
      @param[:address_1] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_2" do
      @param[:address_2] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_false
    end
    it "The length check of the address_3" do
      @param[:address_3] = ""
      test_address = UserAddress.new(@param)
      test_address.valid?.should be_true
    end
  end

  describe "fullname" do
    it "test fullname english" do
      test_address = UserAddress.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'en')
      test_address.fullname.should == 'name2 name1'
    end
    it "test fullname japanese" do
      test_address = UserAddress.new(:name_last => 'name1', :name_first => 'name2', :country_code => 'JP', :locale => 'ja')
      test_address.fullname.should == 'name1 name2'
    end
  end

  describe "fulladdress" do
    describe "without address_1" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress.should == 'add3, add2'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress.should == 'add2 add3'
      end
    end

    describe "with address_1" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress(true).should == 'add3, add2, add1'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress(true).should == 'add1 add2 add3'
      end
    end

    describe "with address_1 and :no_space => true" do
      it "test fulladdress english" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'en')
        test_address.fulladdress(true, :delimeter => ',').should == 'add3,add2,add1'
      end
      it "test fulladdress japanese" do
        test_address = UserAddress.new(:address_1 => 'add1', :address_2 => 'add2', :address_3 => 'add3', :country_code => 'JP', :locale => 'ja')
        test_address.fulladdress(true, :delimeter => '').should == 'add1add2add3'
      end
    end
  end

  describe "validates_length_of test" do
    before do
      @param = UserAddress.find(1011).attributes.dup
    end
    it "name_last in japanese to be valid" do
      @param[:name_last] = "１２３４５６７８９０１２３４５６７８９０１２３４５６７８９０"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "name_last in japanese to be invalid" do
      @param[:name_last] = "１２３４５６７８９０１２３４５６７８９０１２３４５６７８９０１"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
    it "name_last in english to be valid" do
      @param[:name_last] = "123456789012345678901234567890"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == true
    end
    it "name_last in english to be invalid" do
      @param[:name_last] = "1234567890123456789012345678901"
      test_address = UserAddress.new(@param)
      test_address.valid?.should == false
    end
  end

  describe "is_japanese_format?" do
    it "locale is ja, country is JP" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_true
    end
    it "locale is ja, country is JP, sub_address" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
      address.is_japanese_format?.should be_true
    end
    it "locale is ja, country is JP" do
      address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
    it "locale is en, country is JP" do
      address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
    it "locale is zh-CN, country is JP" do
      address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
      address.is_japanese_format?.should be_false
    end
  end

  describe "is_chinese_format?" do
    describe "システム設定値　簡体字入力を許す場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(true)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is CN" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_true
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_true
      end
    end
    describe "システム設定値　簡体字入力を許さない場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(false)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_chinese_format?.should be_false
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_chinese_format?.should be_false
      end
    end
  end

  describe "is_english_format?" do
    describe "システム設定値　簡体字入力を許す場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(true)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_english_format?.should be_true
      end
      it "locale is ja, country is CN" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
    end
    describe "システム設定値　簡体字入力を許さない場合" do
      before do
        EcRider.config.stub!(:chinese_format_using).and_return(false)
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is ja, country is JP, sub_address" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'JP', :address_div_key => :user_en)
        address.is_english_format?.should be_true
      end
      it "locale is ja, country is JP" do
        address = UserAddress.new(:locale => 'ja', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is en, country is JP" do
        address = UserAddress.new(:locale => 'en', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
      it "locale is zh-CN, country is JP" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'JP', :address_div_key => :user)
        address.is_english_format?.should be_false
      end
      it "locale is zh-CN, country is CN" do
        address = UserAddress.new(:locale => 'zh_CN', :country_code => 'CN', :address_div_key => :user)
        address.is_english_format?.should be_true
      end
    end
  end
end
